@@ -1,5 +1,5 @@
 /*
- * Copyright 2002-2012 the original author or authors.
+ * Copyright 2002-2015 the original author or authors.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -16,18 +16,17 @@
 
 package org.springframework.beans.factory.aspectj;
 
-import junit.framework.TestCase;
+import org.junit.Test;
 
 import org.springframework.context.support.ClassPathXmlApplicationContext;
 
-public class SpringConfiguredWithAutoProxyingTests extends TestCase {
+public class SpringConfiguredWithAutoProxyingTests {
 
-	@Override
-	protected void setUp() throws Exception {
+	@Test
+	@SuppressWarnings("resource")
+	public void springConfiguredAndAutoProxyUsedTogether() {
+		// instantiation is sufficient to trigger failure if this is going to fail...
 		new ClassPathXmlApplicationContext("org/springframework/beans/factory/aspectj/springConfigured.xml");
 	}
 
-	public void testSpringConfiguredAndAutoProxyUsedTogether() {
-		; // set up is sufficient to trigger failure if this is going to fail...
-	}
 }
