@@ -1,5 +1,5 @@
 /*
- * Copyright 2002-2014 the original author or authors.
+ * Copyright 2002-2015 the original author or authors.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -23,28 +23,33 @@ import org.springframework.tests.sample.beans.TestBean;
 import static org.junit.Assert.*;
 
 /**
- * Unit tests for {@link DirectFieldAccessor}
+ * Specific {@link DirectFieldAccessor} tests.
  *
  * @author Jose Luis Martin
  * @author Chris Beams
+ * @author Stephane Nicoll
  */
-public class DirectFieldAccessorTests extends AbstractConfigurablePropertyAccessorTests {
+public class DirectFieldAccessorTests extends AbstractPropertyAccessorTests {
 
 	@Override
-	protected ConfigurablePropertyAccessor createAccessor(Object target) {
+	protected DirectFieldAccessor createAccessor(Object target) {
 		return new DirectFieldAccessor(target);
 	}
 
+
 	@Test
 	public void withShadowedField() throws Exception {
+		final StringBuilder sb = new StringBuilder();
+
 		@SuppressWarnings("serial")
-		TestBean tb = new TestBean() {
+		TestBean target = new TestBean() {
 			@SuppressWarnings("unused")
-			StringBuilder name = new StringBuilder();
+			StringBuilder name = sb;
 		};
 
-		DirectFieldAccessor dfa = new DirectFieldAccessor(tb);
+		DirectFieldAccessor dfa = createAccessor(target);
 		assertEquals(StringBuilder.class, dfa.getPropertyType("name"));
+		assertEquals(sb, dfa.getPropertyValue("name"));
 	}
 
 }
