@@ -1,5 +1,5 @@
 /*
- * Copyright 2002-2012 the original author or authors.
+ * Copyright 2002-2016 the original author or authors.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -21,17 +21,18 @@ import java.io.InputStream;
 import java.io.InputStreamReader;
 import java.text.MessageFormat;
 import java.util.ArrayList;
-import java.util.HashMap;
 import java.util.List;
 import java.util.Locale;
 import java.util.Map;
 import java.util.Properties;
+import java.util.concurrent.ConcurrentHashMap;
+import java.util.concurrent.ConcurrentMap;
+import java.util.concurrent.locks.ReentrantLock;
 
 import org.springframework.context.ResourceLoaderAware;
 import org.springframework.core.io.DefaultResourceLoader;
 import org.springframework.core.io.Resource;
 import org.springframework.core.io.ResourceLoader;
-import org.springframework.util.Assert;
 import org.springframework.util.DefaultPropertiesPersister;
 import org.springframework.util.PropertiesPersister;
 import org.springframework.util.StringUtils;
@@ -48,31 +49,22 @@ import org.springframework.util.StringUtils;
  * reloading files based on timestamp changes, but also of loading properties files
  * with a specific character encoding. It will detect XML property files as well.
  *
- * <p>In contrast to {@link ResourceBundleMessageSource}, this class supports
- * reloading of properties files through the {@link #setCacheSeconds "cacheSeconds"}
- * setting, and also through programmatically clearing the properties cache.
- * Since application servers typically cache all files loaded from the classpath,
- * it is necessary to store resources somewhere else (for example, in the
- * "WEB-INF" directory of a web app). Otherwise changes of files in the
- * classpath will <i>not</i> be reflected in the application.
- *
- * <p>Note that the base names set as {@link #setBasenames "basenames"} property
+ * <p>Note that the basenames set as {@link #setBasenames "basenames"} property
  * are treated in a slightly different fashion than the "basenames" property of
  * {@link ResourceBundleMessageSource}. It follows the basic ResourceBundle rule of not
  * specifying file extension or language codes, but can refer to any Spring resource
  * location (instead of being restricted to classpath resources). With a "classpath:"
  * prefix, resources can still be loaded from the classpath, but "cacheSeconds" values
- * other than "-1" (caching forever) will not work in this case.
- *
- * <p>This MessageSource implementation is usually slightly faster than
- * {@link ResourceBundleMessageSource}, which builds on {@link java.util.ResourceBundle}
- * - in the default mode, i.e. when caching forever. With "cacheSeconds" set to 1,
- * message lookup takes about twice as long - with the benefit that changes in
- * individual properties files are detected with a maximum delay of 1 second.
- * Higher "cacheSeconds" values usually <i>do not</i> make a significant difference.
+ * other than "-1" (caching forever) might not work reliably in this case.
  *
+ * <p>For a typical web application, message files could be placed in {@code WEB-INF}:
+ * e.g. a "WEB-INF/messages" basename would find a "WEB-INF/messages.properties",
+ * "WEB-INF/messages_en.properties" etc arrangement as well as "WEB-INF/messages.xml",
+ * "WEB-INF/messages_en.xml" etc. Note that message definitions in a <i>previous</i>
+ * resource bundle will override ones in a later bundle, due to sequential lookup.
+
  * <p>This MessageSource can easily be used outside of an
- * {@link org.springframework.context.ApplicationContext}: It will use a
+ * {@link org.springframework.context.ApplicationContext}: it will use a
  * {@link org.springframework.core.io.DefaultResourceLoader} as default,
  * simply getting overridden with the ApplicationContext's resource loader
  * if running in a context. It does not have any other specific dependencies.
@@ -92,105 +84,40 @@ import org.springframework.util.StringUtils;
  * @see ResourceBundleMessageSource
  * @see java.util.ResourceBundle
  */
-public class ReloadableResourceBundleMessageSource extends AbstractMessageSource
-		implements ResourceLoaderAware {
+public class ReloadableResourceBundleMessageSource extends AbstractResourceBasedMessageSource implements ResourceLoaderAware {
 
 	private static final String PROPERTIES_SUFFIX = ".properties";
 
 	private static final String XML_SUFFIX = ".xml";
 
 
-	private String[] basenames = new String[0];
-
-	private String defaultEncoding;
-
 	private Properties fileEncodings;
 
-	private boolean fallbackToSystemLocale = true;
-
-	private long cacheMillis = -1;
+	private boolean concurrentRefresh = true;
 
 	private PropertiesPersister propertiesPersister = new DefaultPropertiesPersister();
 
 	private ResourceLoader resourceLoader = new DefaultResourceLoader();
 
 	/** Cache to hold filename lists per Locale */
-	private final Map<String, Map<Locale, List<String>>> cachedFilenames =
-			new HashMap<String, Map<Locale, List<String>>>();
+	private final ConcurrentMap<String, Map<Locale, List<String>>> cachedFilenames =
+			new ConcurrentHashMap<String, Map<Locale, List<String>>>();
 
 	/** Cache to hold already loaded properties per filename */
-	private final Map<String, PropertiesHolder> cachedProperties = new HashMap<String, PropertiesHolder>();
+	private final ConcurrentMap<String, PropertiesHolder> cachedProperties =
+			new ConcurrentHashMap<String, PropertiesHolder>();
 
 	/** Cache to hold merged loaded properties per locale */
-	private final Map<Locale, PropertiesHolder> cachedMergedProperties = new HashMap<Locale, PropertiesHolder>();
+	private final ConcurrentMap<Locale, PropertiesHolder> cachedMergedProperties =
+			new ConcurrentHashMap<Locale, PropertiesHolder>();
 
 
-	/**
-	 * Set a single basename, following the basic ResourceBundle convention of
-	 * not specifying file extension or language codes, but in contrast to
-	 * {@link ResourceBundleMessageSource} referring to a Spring resource location:
-	 * e.g. "WEB-INF/messages" for "WEB-INF/messages.properties",
-	 * "WEB-INF/messages_en.properties", etc.
-	 * <p>XML properties files are also supported: .g. "WEB-INF/messages" will find
-	 * and load "WEB-INF/messages.xml", "WEB-INF/messages_en.xml", etc as well.
-	 * @param basename the single basename
-	 * @see #setBasenames
-	 * @see org.springframework.core.io.ResourceEditor
-	 * @see java.util.ResourceBundle
-	 */
-	public void setBasename(String basename) {
-		setBasenames(basename);
-	}
-
-	/**
-	 * Set an array of basenames, each following the basic ResourceBundle convention
-	 * of not specifying file extension or language codes, but in contrast to
-	 * {@link ResourceBundleMessageSource} referring to a Spring resource location:
-	 * e.g. "WEB-INF/messages" for "WEB-INF/messages.properties",
-	 * "WEB-INF/messages_en.properties", etc.
-	 * <p>XML properties files are also supported: .g. "WEB-INF/messages" will find
-	 * and load "WEB-INF/messages.xml", "WEB-INF/messages_en.xml", etc as well.
-	 * <p>The associated resource bundles will be checked sequentially when resolving
-	 * a message code. Note that message definitions in a <i>previous</i> resource
-	 * bundle will override ones in a later bundle, due to the sequential lookup.
-	 * @param basenames an array of basenames
-	 * @see #setBasename
-	 * @see java.util.ResourceBundle
-	 */
-	public void setBasenames(String... basenames) {
-		if (basenames != null) {
-			this.basenames = new String[basenames.length];
-			for (int i = 0; i < basenames.length; i++) {
-				String basename = basenames[i];
-				Assert.hasText(basename, "Basename must not be empty");
-				this.basenames[i] = basename.trim();
-			}
-		}
-		else {
-			this.basenames = new String[0];
-		}
-	}
-
-	/**
-	 * Set the default charset to use for parsing properties files.
-	 * Used if no file-specific charset is specified for a file.
-	 * <p>Default is none, using the {@code java.util.Properties}
-	 * default encoding: ISO-8859-1.
-	 * <p>Only applies to classic properties files, not to XML files.
-	 * @param defaultEncoding the default charset
-	 * @see #setFileEncodings
-	 * @see org.springframework.util.PropertiesPersister#load
-	 */
-	public void setDefaultEncoding(String defaultEncoding) {
-		this.defaultEncoding = defaultEncoding;
-	}
-
 	/**
 	 * Set per-file charsets to use for parsing properties files.
 	 * <p>Only applies to classic properties files, not to XML files.
 	 * @param fileEncodings Properties with filenames as keys and charset
 	 * names as values. Filenames have to match the basename syntax,
-	 * with optional locale-specific appendices: e.g. "WEB-INF/messages"
+	 * with optional locale-specific components: e.g. "WEB-INF/messages"
 	 * or "WEB-INF/messages_en".
 	 * @see #setBasenames
 	 * @see org.springframework.util.PropertiesPersister#load
@@ -200,35 +127,18 @@ public class ReloadableResourceBundleMessageSource extends AbstractMessageSource
 	}
 
 	/**
-	 * Set whether to fall back to the system Locale if no files for a specific
-	 * Locale have been found. Default is "true"; if this is turned off, the only
-	 * fallback will be the default file (e.g. "messages.properties" for
-	 * basename "messages").
-	 * <p>Falling back to the system Locale is the default behavior of
-	 * {@code java.util.ResourceBundle}. However, this is often not desirable
-	 * in an application server environment, where the system Locale is not relevant
-	 * to the application at all: Set this flag to "false" in such a scenario.
+	 * Specify whether to allow for concurrent refresh behavior, i.e. one thread
+	 * locked in a refresh attempt for a specific cached properties file whereas
+	 * other threads keep returning the old properties for the time being, until
+	 * the refresh attempt has completed.
+	 * <p>Default is "true": this behavior is new as of Spring Framework 4.1,
+	 * minimizing contention between threads. If you prefer the old behavior,
+	 * i.e. to fully block on refresh, switch this flag to "false".
+	 * @since 4.1
+	 * @see #setCacheSeconds
 	 */
-	public void setFallbackToSystemLocale(boolean fallbackToSystemLocale) {
-		this.fallbackToSystemLocale = fallbackToSystemLocale;
-	}
-
-	/**
-	 * Set the number of seconds to cache loaded properties files.
-	 * <ul>
-	 * <li>Default is "-1", indicating to cache forever (just like
-	 * {@code java.util.ResourceBundle}).
-	 * <li>A positive number will cache loaded properties files for the given
-	 * number of seconds. This is essentially the interval between refresh checks.
-	 * Note that a refresh attempt will first check the last-modified timestamp
-	 * of the file before actually reloading it; so if files don't change, this
-	 * interval can be set rather low, as refresh attempts will not actually reload.
-	 * <li>A value of "0" will check the last-modified timestamp of the file on
-	 * every message access. <b>Do not use this in a production environment!</b>
-	 * </ul>
-	 */
-	public void setCacheSeconds(int cacheSeconds) {
-		this.cacheMillis = (cacheSeconds * 1000);
+	public void setConcurrentRefresh(boolean concurrentRefresh) {
+		this.concurrentRefresh = concurrentRefresh;
 	}
 
 	/**
@@ -250,6 +160,7 @@ public class ReloadableResourceBundleMessageSource extends AbstractMessageSource
 	 * @see org.springframework.core.io.DefaultResourceLoader
 	 * @see org.springframework.context.ResourceLoaderAware
 	 */
+	@Override
 	public void setResourceLoader(ResourceLoader resourceLoader) {
 		this.resourceLoader = (resourceLoader != null ? resourceLoader : new DefaultResourceLoader());
 	}
@@ -261,7 +172,7 @@ public class ReloadableResourceBundleMessageSource extends AbstractMessageSource
 	 */
 	@Override
 	protected String resolveCodeWithoutArguments(String code, Locale locale) {
-		if (this.cacheMillis < 0) {
+		if (getCacheMillis() < 0) {
 			PropertiesHolder propHolder = getMergedProperties(locale);
 			String result = propHolder.getProperty(code);
 			if (result != null) {
@@ -269,7 +180,7 @@ public class ReloadableResourceBundleMessageSource extends AbstractMessageSource
 			}
 		}
 		else {
-			for (String basename : this.basenames) {
+			for (String basename : getBasenameSet()) {
 				List<String> filenames = calculateAllFilenames(basename, locale);
 				for (String filename : filenames) {
 					PropertiesHolder propHolder = getProperties(filename);
@@ -289,7 +200,7 @@ public class ReloadableResourceBundleMessageSource extends AbstractMessageSource
 	 */
 	@Override
 	protected MessageFormat resolveCode(String code, Locale locale) {
-		if (this.cacheMillis < 0) {
+		if (getCacheMillis() < 0) {
 			PropertiesHolder propHolder = getMergedProperties(locale);
 			MessageFormat result = propHolder.getMessageFormat(code, locale);
 			if (result != null) {
@@ -297,7 +208,7 @@ public class ReloadableResourceBundleMessageSource extends AbstractMessageSource
 			}
 		}
 		else {
-			for (String basename : this.basenames) {
+			for (String basename : getBasenameSet()) {
 				List<String> filenames = calculateAllFilenames(basename, locale);
 				for (String filename : filenames) {
 					PropertiesHolder propHolder = getProperties(filename);
@@ -321,26 +232,32 @@ public class ReloadableResourceBundleMessageSource extends AbstractMessageSource
 	 * cached forever.
 	 */
 	protected PropertiesHolder getMergedProperties(Locale locale) {
-		synchronized (this.cachedMergedProperties) {
-			PropertiesHolder mergedHolder = this.cachedMergedProperties.get(locale);
-			if (mergedHolder != null) {
-				return mergedHolder;
-			}
-			Properties mergedProps = new Properties();
-			mergedHolder = new PropertiesHolder(mergedProps, -1);
-			for (int i = this.basenames.length - 1; i >= 0; i--) {
-				List filenames = calculateAllFilenames(this.basenames[i], locale);
-				for (int j = filenames.size() - 1; j >= 0; j--) {
-					String filename = (String) filenames.get(j);
-					PropertiesHolder propHolder = getProperties(filename);
-					if (propHolder.getProperties() != null) {
-						mergedProps.putAll(propHolder.getProperties());
+		PropertiesHolder mergedHolder = this.cachedMergedProperties.get(locale);
+		if (mergedHolder != null) {
+			return mergedHolder;
+		}
+		Properties mergedProps = newProperties();
+		long latestTimestamp = -1;
+		String[] basenames = StringUtils.toStringArray(getBasenameSet());
+		for (int i = basenames.length - 1; i >= 0; i--) {
+			List<String> filenames = calculateAllFilenames(basenames[i], locale);
+			for (int j = filenames.size() - 1; j >= 0; j--) {
+				String filename = filenames.get(j);
+				PropertiesHolder propHolder = getProperties(filename);
+				if (propHolder.getProperties() != null) {
+					mergedProps.putAll(propHolder.getProperties());
+					if (propHolder.getFileTimestamp() > latestTimestamp) {
+						latestTimestamp = propHolder.getFileTimestamp();
 					}
 				}
 			}
-			this.cachedMergedProperties.put(locale, mergedHolder);
-			return mergedHolder;
 		}
+		mergedHolder = new PropertiesHolder(mergedProps, latestTimestamp);
+		PropertiesHolder existing = this.cachedMergedProperties.putIfAbsent(locale, mergedHolder);
+		if (existing != null) {
+			mergedHolder = existing;
+		}
+		return mergedHolder;
 	}
 
 	/**
@@ -354,36 +271,34 @@ public class ReloadableResourceBundleMessageSource extends AbstractMessageSource
 	 * @see #calculateFilenamesForLocale
 	 */
 	protected List<String> calculateAllFilenames(String basename, Locale locale) {
-		synchronized (this.cachedFilenames) {
-			Map<Locale, List<String>> localeMap = this.cachedFilenames.get(basename);
-			if (localeMap != null) {
-				List<String> filenames = localeMap.get(locale);
-				if (filenames != null) {
-					return filenames;
-				}
+		Map<Locale, List<String>> localeMap = this.cachedFilenames.get(basename);
+		if (localeMap != null) {
+			List<String> filenames = localeMap.get(locale);
+			if (filenames != null) {
+				return filenames;
 			}
-			List<String> filenames = new ArrayList<String>(7);
-			filenames.addAll(calculateFilenamesForLocale(basename, locale));
-			if (this.fallbackToSystemLocale && !locale.equals(Locale.getDefault())) {
-				List<String> fallbackFilenames = calculateFilenamesForLocale(basename, Locale.getDefault());
-				for (String fallbackFilename : fallbackFilenames) {
-					if (!filenames.contains(fallbackFilename)) {
-						// Entry for fallback locale that isn't already in filenames list.
-						filenames.add(fallbackFilename);
-					}
-				}
-			}
-			filenames.add(basename);
-			if (localeMap != null) {
-				localeMap.put(locale, filenames);
-			}
-			else {
-				localeMap = new HashMap<Locale, List<String>>();
-				localeMap.put(locale, filenames);
-				this.cachedFilenames.put(basename, localeMap);
-			}
-			return filenames;
 		}
+		List<String> filenames = new ArrayList<String>(7);
+		filenames.addAll(calculateFilenamesForLocale(basename, locale));
+		if (isFallbackToSystemLocale() && !locale.equals(Locale.getDefault())) {
+			List<String> fallbackFilenames = calculateFilenamesForLocale(basename, Locale.getDefault());
+			for (String fallbackFilename : fallbackFilenames) {
+				if (!filenames.contains(fallbackFilename)) {
+					// Entry for fallback locale that isn't already in filenames list.
+					filenames.add(fallbackFilename);
+				}
+			}
+		}
+		filenames.add(basename);
+		if (localeMap == null) {
+			localeMap = new ConcurrentHashMap<Locale, List<String>>();
+			Map<Locale, List<String>> existing = this.cachedFilenames.putIfAbsent(basename, localeMap);
+			if (existing != null) {
+				localeMap = existing;
+			}
+		}
+		localeMap.put(locale, filenames);
+		return filenames;
 	}
 
 	/**
@@ -431,16 +346,46 @@ public class ReloadableResourceBundleMessageSource extends AbstractMessageSource
 	 * @return the current PropertiesHolder for the bundle
 	 */
 	protected PropertiesHolder getProperties(String filename) {
-		synchronized (this.cachedProperties) {
-			PropertiesHolder propHolder = this.cachedProperties.get(filename);
-			if (propHolder != null &&
-					(propHolder.getRefreshTimestamp() < 0 ||
-					 propHolder.getRefreshTimestamp() > System.currentTimeMillis() - this.cacheMillis)) {
-				// up to date
+		PropertiesHolder propHolder = this.cachedProperties.get(filename);
+		long originalTimestamp = -2;
+
+		if (propHolder != null) {
+			originalTimestamp = propHolder.getRefreshTimestamp();
+			if (originalTimestamp == -1 || originalTimestamp > System.currentTimeMillis() - getCacheMillis()) {
+				// Up to date
 				return propHolder;
 			}
+		}
+		else {
+			propHolder = new PropertiesHolder();
+			PropertiesHolder existingHolder = this.cachedProperties.putIfAbsent(filename, propHolder);
+			if (existingHolder != null) {
+				propHolder = existingHolder;
+			}
+		}
+
+		// At this point, we need to refresh...
+		if (this.concurrentRefresh && propHolder.getRefreshTimestamp() >= 0) {
+			// A populated but stale holder -> could keep using it.
+			if (!propHolder.refreshLock.tryLock()) {
+				// Getting refreshed by another thread already ->
+				// let's return the existing properties for the time being.
+				return propHolder;
+			}
+		}
+		else {
+			propHolder.refreshLock.lock();
+		}
+		try {
+			PropertiesHolder existingHolder = this.cachedProperties.get(filename);
+			if (existingHolder != null && existingHolder.getRefreshTimestamp() > originalTimestamp) {
+				return existingHolder;
+			}
 			return refreshProperties(filename, propHolder);
 		}
+		finally {
+			propHolder.refreshLock.unlock();
+		}
 	}
 
 	/**
@@ -451,7 +396,7 @@ public class ReloadableResourceBundleMessageSource extends AbstractMessageSource
 	 * @param propHolder the current PropertiesHolder for the bundle
 	 */
 	protected PropertiesHolder refreshProperties(String filename, PropertiesHolder propHolder) {
-		long refreshTimestamp = (this.cacheMillis < 0 ? -1 : System.currentTimeMillis());
+		long refreshTimestamp = (getCacheMillis() < 0 ? -1 : System.currentTimeMillis());
 
 		Resource resource = this.resourceLoader.getResource(filename + PROPERTIES_SUFFIX);
 		if (!resource.exists()) {
@@ -460,7 +405,7 @@ public class ReloadableResourceBundleMessageSource extends AbstractMessageSource
 
 		if (resource.exists()) {
 			long fileTimestamp = -1;
-			if (this.cacheMillis >= 0) {
+			if (getCacheMillis() >= 0) {
 				// Last-modified timestamp of file will just be read if caching with timeout.
 				try {
 					fileTimestamp = resource.lastModified();
@@ -475,8 +420,7 @@ public class ReloadableResourceBundleMessageSource extends AbstractMessageSource
 				catch (IOException ex) {
 					// Probably a class path resource: cache it forever.
 					if (logger.isDebugEnabled()) {
-						logger.debug(
-								resource + " could not be resolved in the file system - assuming that is hasn't changed", ex);
+						logger.debug(resource + " could not be resolved in the file system - assuming that it hasn't changed", ex);
 					}
 					fileTimestamp = -1;
 				}
@@ -517,7 +461,7 @@ public class ReloadableResourceBundleMessageSource extends AbstractMessageSource
 	 */
 	protected Properties loadProperties(Resource resource, String filename) throws IOException {
 		InputStream is = resource.getInputStream();
-		Properties props = new Properties();
+		Properties props = newProperties();
 		try {
 			if (resource.getFilename().endsWith(XML_SUFFIX)) {
 				if (logger.isDebugEnabled()) {
@@ -531,7 +475,7 @@ public class ReloadableResourceBundleMessageSource extends AbstractMessageSource
 					encoding = this.fileEncodings.getProperty(filename);
 				}
 				if (encoding == null) {
-					encoding = this.defaultEncoding;
+					encoding = getDefaultEncoding();
 				}
 				if (encoding != null) {
 					if (logger.isDebugEnabled()) {
@@ -553,6 +497,19 @@ public class ReloadableResourceBundleMessageSource extends AbstractMessageSource
 		}
 	}
 
+	/**
+	 * Template method for creating a plain new {@link Properties} instance.
+	 * The default implementation simply calls {@link Properties#Properties()}.
+	 * <p>Allows for returning a custom {@link Properties} extension in subclasses.
+	 * Overriding methods should just instantiate a custom {@link Properties} subclass,
+	 * with no further initialization or population to be performed at that point.
+	 * @return a plain Properties instance
+	 * @since 4.2
+	 */
+	protected Properties newProperties() {
+		return new Properties();
+	}
+
 
 	/**
 	 * Clear the resource bundle cache.
@@ -560,12 +517,8 @@ public class ReloadableResourceBundleMessageSource extends AbstractMessageSource
 	 */
 	public void clearCache() {
 		logger.debug("Clearing entire resource bundle cache");
-		synchronized (this.cachedProperties) {
-			this.cachedProperties.clear();
-		}
-		synchronized (this.cachedMergedProperties) {
-			this.cachedMergedProperties.clear();
-		}
+		this.cachedProperties.clear();
+		this.cachedMergedProperties.clear();
 	}
 
 	/**
@@ -582,7 +535,7 @@ public class ReloadableResourceBundleMessageSource extends AbstractMessageSource
 
 	@Override
 	public String toString() {
-		return getClass().getName() + ": basenames=[" + StringUtils.arrayToCommaDelimitedString(this.basenames) + "]";
+		return getClass().getName() + ": basenames=" + getBasenameSet();
 	}
 
 
@@ -594,30 +547,34 @@ public class ReloadableResourceBundleMessageSource extends AbstractMessageSource
 	 */
 	protected class PropertiesHolder {
 
-		private Properties properties;
+		private final Properties properties;
 
-		private long fileTimestamp = -1;
+		private final long fileTimestamp;
 
-		private long refreshTimestamp = -1;
+		private volatile long refreshTimestamp = -2;
+
+		private final ReentrantLock refreshLock = new ReentrantLock();
 
 		/** Cache to hold already generated MessageFormats per message code */
-		private final Map<String, Map<Locale, MessageFormat>> cachedMessageFormats =
-				new HashMap<String, Map<Locale, MessageFormat>>();
+		private final ConcurrentMap<String, Map<Locale, MessageFormat>> cachedMessageFormats =
+				new ConcurrentHashMap<String, Map<Locale, MessageFormat>>();
+
+		public PropertiesHolder() {
+			this.properties = null;
+			this.fileTimestamp = -1;
+		}
 
 		public PropertiesHolder(Properties properties, long fileTimestamp) {
 			this.properties = properties;
 			this.fileTimestamp = fileTimestamp;
 		}
 
-		public PropertiesHolder() {
-		}
-
 		public Properties getProperties() {
-			return properties;
+			return this.properties;
 		}
 
 		public long getFileTimestamp() {
-			return fileTimestamp;
+			return this.fileTimestamp;
 		}
 
 		public void setRefreshTimestamp(long refreshTimestamp) {
@@ -625,7 +582,7 @@ public class ReloadableResourceBundleMessageSource extends AbstractMessageSource
 		}
 
 		public long getRefreshTimestamp() {
-			return refreshTimestamp;
+			return this.refreshTimestamp;
 		}
 
 		public String getProperty(String code) {
@@ -639,26 +596,27 @@ public class ReloadableResourceBundleMessageSource extends AbstractMessageSource
 			if (this.properties == null) {
 				return null;
 			}
-			synchronized (this.cachedMessageFormats) {
-				Map<Locale, MessageFormat> localeMap = this.cachedMessageFormats.get(code);
-				if (localeMap != null) {
-					MessageFormat result = localeMap.get(locale);
-					if (result != null) {
-						return result;
-					}
-				}
-				String msg = this.properties.getProperty(code);
-				if (msg != null) {
-					if (localeMap == null) {
-						localeMap = new HashMap<Locale, MessageFormat>();
-						this.cachedMessageFormats.put(code, localeMap);
-					}
-					MessageFormat result = createMessageFormat(msg, locale);
-					localeMap.put(locale, result);
+			Map<Locale, MessageFormat> localeMap = this.cachedMessageFormats.get(code);
+			if (localeMap != null) {
+				MessageFormat result = localeMap.get(locale);
+				if (result != null) {
 					return result;
 				}
-				return null;
 			}
+			String msg = this.properties.getProperty(code);
+			if (msg != null) {
+				if (localeMap == null) {
+					localeMap = new ConcurrentHashMap<Locale, MessageFormat>();
+					Map<Locale, MessageFormat> existing = this.cachedMessageFormats.putIfAbsent(code, localeMap);
+					if (existing != null) {
+						localeMap = existing;
+					}
+				}
+				MessageFormat result = createMessageFormat(msg, locale);
+				localeMap.put(locale, result);
+				return result;
+			}
+			return null;
 		}
 	}
 
