@@ -1,5 +1,5 @@
 /*
- * Copyright 2002-2015 the original author or authors.
+ * Copyright 2002-2016 the original author or authors.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -16,6 +16,7 @@
 
 package org.springframework.aop.framework;
 
+import org.springframework.aop.support.AopUtils;
 import org.springframework.beans.factory.annotation.Autowired;
 import org.springframework.stereotype.Component;
 import org.springframework.util.Assert;
@@ -28,6 +29,8 @@ public class ClassWithComplexConstructor {
 
 	private final Dependency dependency;
 
+	@Autowired ClassWithComplexConstructor selfReference;
+
 	@Autowired
 	public ClassWithComplexConstructor(Dependency dependency) {
 		Assert.notNull(dependency);
@@ -39,6 +42,7 @@ public class ClassWithComplexConstructor {
 	}
 
 	public void method() {
+		Assert.isTrue(this.selfReference != this && AopUtils.isCglibProxy(this.selfReference));
 		this.dependency.method();
 	}
 
