@@ -1,5 +1,5 @@
 /*
- * Copyright 2002-2014 the original author or authors.
+ * Copyright 2002-2015 the original author or authors.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -16,6 +16,7 @@
 
 package org.springframework.cache.config;
 
+import java.io.IOException;
 import java.util.concurrent.atomic.AtomicLong;
 
 import org.springframework.cache.annotation.CacheEvict;
@@ -40,13 +41,35 @@ public class AnnotatedClassCacheableService implements CacheableService<Object>
 		return counter.getAndIncrement();
 	}
 
+	@Override
+	public Object cacheNull(Object arg1) {
+		return null;
+	}
+
+	@Override
+	@Cacheable(cacheNames = "testCache", sync = true)
+	public Object cacheSync(Object arg1) {
+		return counter.getAndIncrement();
+	}
+
+	@Override
+	@Cacheable(cacheNames = "testCache", sync = true)
+	public Object cacheSyncNull(Object arg1) {
+		return null;
+	}
+
 	@Override
 	public Object conditional(int field) {
 		return null;
 	}
 
 	@Override
-	@Cacheable(value = "testCache", unless = "#result > 10")
+	public Object conditionalSync(int field) {
+		return null;
+	}
+
+	@Override
+	@Cacheable(cacheNames = "testCache", unless = "#result > 10")
 	public Object unless(int arg) {
 		return arg;
 	}
@@ -63,71 +86,71 @@ public class AnnotatedClassCacheableService implements CacheableService<Object>
 	}
 
 	@Override
-	@CacheEvict(value = "testCache", allEntries = true)
+	@CacheEvict(cacheNames = "testCache", allEntries = true)
 	public void evictAll(Object arg1) {
 	}
 
 	@Override
-	@CacheEvict(value = "testCache", beforeInvocation = true)
+	@CacheEvict(cacheNames = "testCache", beforeInvocation = true)
 	public void evictEarly(Object arg1) {
 		throw new RuntimeException("exception thrown - evict should still occur");
 	}
 
 	@Override
-	@CacheEvict(value = "testCache", key = "#p0")
+	@CacheEvict(cacheNames = "testCache", key = "#p0")
 	public void evict(Object arg1, Object arg2) {
 	}
 
 	@Override
-	@CacheEvict(value = "testCache", key = "#p0", beforeInvocation = true)
+	@CacheEvict(cacheNames = "testCache", key = "#p0", beforeInvocation = true)
 	public void invalidateEarly(Object arg1, Object arg2) {
 		throw new RuntimeException("exception thrown - evict should still occur");
 	}
 
 	@Override
-	@Cacheable(value = "testCache", key = "#p0")
+	@Cacheable(cacheNames = "testCache", key = "#p0")
 	public Object key(Object arg1, Object arg2) {
 		return counter.getAndIncrement();
 	}
 
 	@Override
-	@Cacheable(value = "testCache")
+	@Cacheable("testCache")
 	public Object varArgsKey(Object... args) {
 		return counter.getAndIncrement();
 	}
 
 	@Override
-	@Cacheable(value = "testCache", key = "#root.methodName + #root.caches[0].name")
+	@Cacheable(cacheNames = "testCache", key = "#root.methodName + #root.caches[0].name")
 	public Object name(Object arg1) {
 		return counter.getAndIncrement();
 	}
 
 	@Override
-	@Cacheable(value = "testCache", key = "#root.methodName + #root.method.name + #root.targetClass + #root.target")
+	@Cacheable(cacheNames = "testCache", key = "#root.methodName + #root.method.name + #root.targetClass + #root.target")
 	public Object rootVars(Object arg1) {
 		return counter.getAndIncrement();
 	}
 
 	@Override
-	@Cacheable(value = "testCache", keyGenerator = "customKyeGenerator")
+	@Cacheable(cacheNames = "testCache", keyGenerator = "customKyeGenerator")
 	public Object customKeyGenerator(Object arg1) {
 		return counter.getAndIncrement();
 	}
 
 	@Override
-	@Cacheable(value = "testCache", keyGenerator = "unknownBeanName")
+	@Cacheable(cacheNames = "testCache", keyGenerator = "unknownBeanName")
 	public Object unknownCustomKeyGenerator(Object arg1) {
 		return counter.getAndIncrement();
 	}
 
 	@Override
-	@Cacheable(value = "testCache", cacheManager = "customCacheManager")
+	@Cacheable(cacheNames = "testCache", cacheManager = "customCacheManager")
 	public Object customCacheManager(Object arg1) {
 		return counter.getAndIncrement();
 	}
 
 	@Override
-	@Cacheable(value = "testCache", cacheManager = "unknownBeanName")
+	@Cacheable(cacheNames = "testCache", cacheManager = "unknownBeanName")
 	public Object unknownCustomCacheManager(Object arg1) {
 		return counter.getAndIncrement();
 	}
@@ -139,7 +162,7 @@ public class AnnotatedClassCacheableService implements CacheableService<Object>
 	}
 
 	@Override
-	@CachePut(value = "testCache", condition = "#arg.equals(3)")
+	@CachePut(cacheNames = "testCache", condition = "#arg.equals(3)")
 	public Object conditionalUpdate(Object arg) {
 		return arg;
 	}
@@ -157,12 +180,24 @@ public class AnnotatedClassCacheableService implements CacheableService<Object>
 
 	@Override
 	public Long throwChecked(Object arg1) throws Exception {
-		throw new UnsupportedOperationException(arg1.toString());
+		throw new IOException(arg1.toString());
 	}
 
 	@Override
 	public Long throwUnchecked(Object arg1) {
-		throw new UnsupportedOperationException();
+		throw new UnsupportedOperationException(arg1.toString());
+	}
+
+	@Override
+	@Cacheable(cacheNames = "testCache", sync = true)
+	public Object throwCheckedSync(Object arg1) throws Exception {
+		throw new IOException(arg1.toString());
+	}
+
+	@Override
+	@Cacheable(cacheNames = "testCache", sync = true)
+	public Object throwUncheckedSync(Object arg1) {
+		throw new UnsupportedOperationException(arg1.toString());
 	}
 
 	// multi annotations
@@ -174,19 +209,19 @@ public class AnnotatedClassCacheableService implements CacheableService<Object>
 	}
 
 	@Override
-	@Caching(evict = { @CacheEvict("primary"), @CacheEvict(value = "secondary", key = "#a0"),  @CacheEvict(value = "primary", key = "#p0 + 'A'") })
+	@Caching(evict = { @CacheEvict("primary"), @CacheEvict(cacheNames = "secondary", key = "#a0"),  @CacheEvict(cacheNames = "primary", key = "#p0 + 'A'") })
 	public Object multiEvict(Object arg1) {
 		return counter.getAndIncrement();
 	}
 
 	@Override
-	@Caching(cacheable = { @Cacheable(value = "primary", key = "#root.methodName") }, evict = { @CacheEvict("secondary") })
+	@Caching(cacheable = { @Cacheable(cacheNames = "primary", key = "#root.methodName") }, evict = { @CacheEvict("secondary") })
 	public Object multiCacheAndEvict(Object arg1) {
 		return counter.getAndIncrement();
 	}
 
 	@Override
-	@Caching(cacheable = { @Cacheable(value = "primary", condition = "#a0 == 3") }, evict = { @CacheEvict("secondary") })
+	@Caching(cacheable = { @Cacheable(cacheNames = "primary", condition = "#a0 == 3") }, evict = { @CacheEvict("secondary") })
 	public Object multiConditionalCacheAndEvict(Object arg1) {
 		return counter.getAndIncrement();
 	}
@@ -198,7 +233,7 @@ public class AnnotatedClassCacheableService implements CacheableService<Object>
 	}
 
 	@Override
-	@CachePut(value="primary", key="#result.id")
+	@CachePut(cacheNames = "primary", key = "#result.id")
 	public TestEntity putRefersToResult(TestEntity arg1) {
 		arg1.setId(Long.MIN_VALUE);
 		return arg1;
