@@ -1,5 +1,5 @@
 /*
- * Copyright 2002-2014 the original author or authors.
+ * Copyright 2002-2015 the original author or authors.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -16,6 +16,7 @@
 
 package org.springframework.cache.config;
 
+import java.io.IOException;
 import java.util.concurrent.atomic.AtomicLong;
 
 import org.springframework.cache.annotation.CacheEvict;
@@ -41,6 +42,24 @@ public class DefaultCacheableService implements CacheableService<Long> {
 		return counter.getAndIncrement();
 	}
 
+	@Override
+	@Cacheable("testCache")
+	public Long cacheNull(Object arg1) {
+		return null;
+	}
+
+	@Override
+	@Cacheable(cacheNames = "testCache", sync = true)
+	public Long cacheSync(Object arg1) {
+		return counter.getAndIncrement();
+	}
+
+	@Override
+	@Cacheable(cacheNames = "testCache", sync = true)
+	public Long cacheSyncNull(Object arg1) {
+		return null;
+	}
+
 	@Override
 	@CacheEvict("testCache")
 	public void invalidate(Object arg1) {
@@ -53,83 +72,89 @@ public class DefaultCacheableService implements CacheableService<Long> {
 	}
 
 	@Override
-	@CacheEvict(value = "testCache", allEntries = true)
+	@CacheEvict(cacheNames = "testCache", allEntries = true)
 	public void evictAll(Object arg1) {
 	}
 
 	@Override
-	@CacheEvict(value = "testCache", beforeInvocation = true)
+	@CacheEvict(cacheNames = "testCache", beforeInvocation = true)
 	public void evictEarly(Object arg1) {
 		throw new RuntimeException("exception thrown - evict should still occur");
 	}
 
 	@Override
-	@CacheEvict(value = "testCache", key = "#p0")
+	@CacheEvict(cacheNames = "testCache", key = "#p0")
 	public void evict(Object arg1, Object arg2) {
 	}
 
 	@Override
-	@CacheEvict(value = "testCache", key = "#p0", beforeInvocation = true)
+	@CacheEvict(cacheNames = "testCache", key = "#p0", beforeInvocation = true)
 	public void invalidateEarly(Object arg1, Object arg2) {
 		throw new RuntimeException("exception thrown - evict should still occur");
 	}
 
 	@Override
-	@Cacheable(value = "testCache", condition = "#classField == 3")
+	@Cacheable(cacheNames = "testCache", condition = "#p0 == 3")
 	public Long conditional(int classField) {
 		return counter.getAndIncrement();
 	}
 
 	@Override
-	@Cacheable(value = "testCache", unless = "#result > 10")
+	@Cacheable(cacheNames = "testCache", sync = true, condition = "#p0 == 3")
+	public Long conditionalSync(int classField) {
+		return counter.getAndIncrement();
+	}
+
+	@Override
+	@Cacheable(cacheNames = "testCache", unless = "#result > 10")
 	public Long unless(int arg) {
 		return (long) arg;
 	}
 
 	@Override
-	@Cacheable(value = "testCache", key = "#p0")
+	@Cacheable(cacheNames = "testCache", key = "#p0")
 	public Long key(Object arg1, Object arg2) {
 		return counter.getAndIncrement();
 	}
 
 	@Override
-	@Cacheable(value = "testCache")
+	@Cacheable(cacheNames = "testCache")
 	public Long varArgsKey(Object... args) {
 		return counter.getAndIncrement();
 	}
 
 	@Override
-	@Cacheable(value = "testCache", key = "#root.methodName")
+	@Cacheable(cacheNames = "testCache", key = "#root.methodName")
 	public Long name(Object arg1) {
 		return counter.getAndIncrement();
 	}
 
 	@Override
-	@Cacheable(value = "testCache", key = "#root.methodName + #root.method.name + #root.targetClass + #root.target")
+	@Cacheable(cacheNames = "testCache", key = "#root.methodName + #root.method.name + #root.targetClass + #root.target")
 	public Long rootVars(Object arg1) {
 		return counter.getAndIncrement();
 	}
 
 	@Override
-	@Cacheable(value = "testCache", keyGenerator = "customKeyGenerator")
+	@Cacheable(cacheNames = "testCache", keyGenerator = "customKeyGenerator")
 	public Long customKeyGenerator(Object arg1) {
 		return counter.getAndIncrement();
 	}
 
 	@Override
-	@Cacheable(value = "testCache", keyGenerator = "unknownBeanName")
+	@Cacheable(cacheNames = "testCache", keyGenerator = "unknownBeanName")
 	public Long unknownCustomKeyGenerator(Object arg1) {
 		return counter.getAndIncrement();
 	}
 
 	@Override
-	@Cacheable(value = "testCache", cacheManager = "customCacheManager")
+	@Cacheable(cacheNames = "testCache", cacheManager = "customCacheManager")
 	public Long customCacheManager(Object arg1) {
 		return counter.getAndIncrement();
 	}
 
 	@Override
-	@Cacheable(value = "testCache", cacheManager = "unknownBeanName")
+	@Cacheable(cacheNames = "testCache", cacheManager = "unknownBeanName")
 	public Long unknownCustomCacheManager(Object arg1) {
 		return counter.getAndIncrement();
 	}
@@ -141,7 +166,7 @@ public class DefaultCacheableService implements CacheableService<Long> {
 	}
 
 	@Override
-	@CachePut(value = "testCache", condition = "#arg.equals(3)")
+	@CachePut(cacheNames = "testCache", condition = "#arg.equals(3)")
 	public Long conditionalUpdate(Object arg) {
 		return Long.valueOf(arg.toString());
 	}
@@ -161,7 +186,7 @@ public class DefaultCacheableService implements CacheableService<Long> {
 	@Override
 	@Cacheable("testCache")
 	public Long throwChecked(Object arg1) throws Exception {
-		throw new Exception(arg1.toString());
+		throw new IOException(arg1.toString());
 	}
 
 	@Override
@@ -170,6 +195,18 @@ public class DefaultCacheableService implements CacheableService<Long> {
 		throw new UnsupportedOperationException(arg1.toString());
 	}
 
+	@Override
+	@Cacheable(cacheNames = "testCache", sync = true)
+	public Long throwCheckedSync(Object arg1) throws Exception {
+		throw new IOException(arg1.toString());
+	}
+
+	@Override
+	@Cacheable(cacheNames = "testCache", sync = true)
+	public Long throwUncheckedSync(Object arg1) {
+		throw new UnsupportedOperationException(arg1.toString());
+	}
+
 	// multi annotations
 
 	@Override
@@ -179,19 +216,19 @@ public class DefaultCacheableService implements CacheableService<Long> {
 	}
 
 	@Override
-	@Caching(evict = { @CacheEvict("primary"), @CacheEvict(value = "secondary", key = "#p0"), @CacheEvict(value = "primary", key = "#p0 + 'A'") })
+	@Caching(evict = { @CacheEvict("primary"), @CacheEvict(cacheNames = "secondary", key = "#p0"), @CacheEvict(cacheNames = "primary", key = "#p0 + 'A'") })
 	public Long multiEvict(Object arg1) {
 		return counter.getAndIncrement();
 	}
 
 	@Override
-	@Caching(cacheable = { @Cacheable(value = "primary", key = "#root.methodName") }, evict = { @CacheEvict("secondary") })
+	@Caching(cacheable = { @Cacheable(cacheNames = "primary", key = "#root.methodName") }, evict = { @CacheEvict("secondary") })
 	public Long multiCacheAndEvict(Object arg1) {
 		return counter.getAndIncrement();
 	}
 
 	@Override
-	@Caching(cacheable = { @Cacheable(value = "primary", condition = "#p0 == 3") }, evict = { @CacheEvict("secondary") })
+	@Caching(cacheable = { @Cacheable(cacheNames = "primary", condition = "#p0 == 3") }, evict = { @CacheEvict("secondary") })
 	public Long multiConditionalCacheAndEvict(Object arg1) {
 		return counter.getAndIncrement();
 	}
@@ -203,9 +240,10 @@ public class DefaultCacheableService implements CacheableService<Long> {
 	}
 
 	@Override
-	@CachePut(value="primary", key="#result.id")
+	@CachePut(cacheNames = "primary", key = "#result.id")
 	public TestEntity putRefersToResult(TestEntity arg1) {
 		arg1.setId(Long.MIN_VALUE);
 		return arg1;
 	}
+
 }
