@@ -1,5 +1,5 @@
 /*
- * Copyright 2002-2014 the original author or authors.
+ * Copyright 2002-2016 the original author or authors.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -24,7 +24,9 @@ import java.lang.annotation.Target;
 import java.lang.reflect.Method;
 import java.util.Calendar;
 import java.util.Date;
+import java.util.HashMap;
 import java.util.List;
+import java.util.Map;
 import java.util.Properties;
 import java.util.TimeZone;
 
@@ -37,6 +39,7 @@ import org.springframework.beans.factory.config.BeanDefinition;
 import org.springframework.beans.factory.config.PropertyPlaceholderConfigurer;
 import org.springframework.beans.factory.support.RootBeanDefinition;
 import org.springframework.context.support.StaticApplicationContext;
+import org.springframework.core.annotation.AliasFor;
 import org.springframework.scheduling.Trigger;
 import org.springframework.scheduling.TriggerContext;
 import org.springframework.scheduling.config.CronTask;
@@ -77,6 +80,7 @@ public class ScheduledAnnotationBeanPostProcessorTests {
 		context.registerBeanDefinition("postProcessor", processorDefinition);
 		context.registerBeanDefinition("target", targetDefinition);
 		context.refresh();
+
 		Object postProcessor = context.getBean("postProcessor");
 		Object target = context.getBean("target");
 		ScheduledTaskRegistrar registrar = (ScheduledTaskRegistrar)
@@ -98,11 +102,11 @@ public class ScheduledAnnotationBeanPostProcessorTests {
 	@Test
 	public void fixedRateTask() {
 		BeanDefinition processorDefinition = new RootBeanDefinition(ScheduledAnnotationBeanPostProcessor.class);
-		BeanDefinition targetDefinition = new RootBeanDefinition(
-				ScheduledAnnotationBeanPostProcessorTests.FixedRateTestBean.class);
+		BeanDefinition targetDefinition = new RootBeanDefinition(FixedRateTestBean.class);
 		context.registerBeanDefinition("postProcessor", processorDefinition);
 		context.registerBeanDefinition("target", targetDefinition);
 		context.refresh();
+
 		Object postProcessor = context.getBean("postProcessor");
 		Object target = context.getBean("target");
 		ScheduledTaskRegistrar registrar = (ScheduledTaskRegistrar)
@@ -128,6 +132,7 @@ public class ScheduledAnnotationBeanPostProcessorTests {
 		context.registerBeanDefinition("postProcessor", processorDefinition);
 		context.registerBeanDefinition("target", targetDefinition);
 		context.refresh();
+
 		Object postProcessor = context.getBean("postProcessor");
 		Object target = context.getBean("target");
 		ScheduledTaskRegistrar registrar = (ScheduledTaskRegistrar)
@@ -148,19 +153,29 @@ public class ScheduledAnnotationBeanPostProcessorTests {
 
 	@Test
 	public void severalFixedRatesWithRepeatedScheduledAnnotation() {
-		BeanDefinition processorDefinition = new
-		RootBeanDefinition(ScheduledAnnotationBeanPostProcessor.class);
-		BeanDefinition targetDefinition = new RootBeanDefinition(
-				SeveralFixedRatesWithRepeatedScheduledAnnotationTestBean.class);
+		BeanDefinition processorDefinition = new RootBeanDefinition(ScheduledAnnotationBeanPostProcessor.class);
+		BeanDefinition targetDefinition = new RootBeanDefinition(SeveralFixedRatesWithRepeatedScheduledAnnotationTestBean.class);
 		severalFixedRates(context, processorDefinition, targetDefinition);
 	}
 
 	@Test
 	public void severalFixedRatesWithSchedulesContainerAnnotation() {
-		BeanDefinition processorDefinition = new RootBeanDefinition(
-				ScheduledAnnotationBeanPostProcessor.class);
-		BeanDefinition targetDefinition = new RootBeanDefinition(
-				SeveralFixedRatesWithSchedulesContainerAnnotationTestBean.class);
+		BeanDefinition processorDefinition = new RootBeanDefinition(ScheduledAnnotationBeanPostProcessor.class);
+		BeanDefinition targetDefinition = new RootBeanDefinition(SeveralFixedRatesWithSchedulesContainerAnnotationTestBean.class);
+		severalFixedRates(context, processorDefinition, targetDefinition);
+	}
+
+	@Test
+	public void severalFixedRatesOnBaseClass() {
+		BeanDefinition processorDefinition = new RootBeanDefinition(ScheduledAnnotationBeanPostProcessor.class);
+		BeanDefinition targetDefinition = new RootBeanDefinition(FixedRatesSubBean.class);
+		severalFixedRates(context, processorDefinition, targetDefinition);
+	}
+
+	@Test
+	public void severalFixedRatesOnDefaultMethod() {
+		BeanDefinition processorDefinition = new RootBeanDefinition(ScheduledAnnotationBeanPostProcessor.class);
+		BeanDefinition targetDefinition = new RootBeanDefinition(FixedRatesDefaultBean.class);
 		severalFixedRates(context, processorDefinition, targetDefinition);
 	}
 
@@ -170,6 +185,7 @@ public class ScheduledAnnotationBeanPostProcessorTests {
 		context.registerBeanDefinition("postProcessor", processorDefinition);
 		context.registerBeanDefinition("target", targetDefinition);
 		context.refresh();
+
 		Object postProcessor = context.getBean("postProcessor");
 		Object target = context.getBean("target");
 		ScheduledTaskRegistrar registrar = (ScheduledTaskRegistrar)
@@ -201,11 +217,11 @@ public class ScheduledAnnotationBeanPostProcessorTests {
 		Assume.group(TestGroup.LONG_RUNNING);
 
 		BeanDefinition processorDefinition = new RootBeanDefinition(ScheduledAnnotationBeanPostProcessor.class);
-		BeanDefinition targetDefinition = new RootBeanDefinition(
-				ScheduledAnnotationBeanPostProcessorTests.CronTestBean.class);
+		BeanDefinition targetDefinition = new RootBeanDefinition(CronTestBean.class);
 		context.registerBeanDefinition("postProcessor", processorDefinition);
 		context.registerBeanDefinition("target", targetDefinition);
 		context.refresh();
+
 		Object postProcessor = context.getBean("postProcessor");
 		Object target = context.getBean("target");
 		ScheduledTaskRegistrar registrar = (ScheduledTaskRegistrar)
@@ -229,11 +245,11 @@ public class ScheduledAnnotationBeanPostProcessorTests {
 		Assume.group(TestGroup.LONG_RUNNING);
 
 		BeanDefinition processorDefinition = new RootBeanDefinition(ScheduledAnnotationBeanPostProcessor.class);
-		BeanDefinition targetDefinition = new RootBeanDefinition(
-				ScheduledAnnotationBeanPostProcessorTests.CronWithTimezoneTestBean.class);
+		BeanDefinition targetDefinition = new RootBeanDefinition(CronWithTimezoneTestBean.class);
 		context.registerBeanDefinition("postProcessor", processorDefinition);
 		context.registerBeanDefinition("target", targetDefinition);
 		context.refresh();
+
 		Object postProcessor = context.getBean("postProcessor");
 		Object target = context.getBean("target");
 		ScheduledTaskRegistrar registrar = (ScheduledTaskRegistrar)
@@ -274,8 +290,7 @@ public class ScheduledAnnotationBeanPostProcessorTests {
 		Assume.group(TestGroup.LONG_RUNNING);
 
 		BeanDefinition processorDefinition = new RootBeanDefinition(ScheduledAnnotationBeanPostProcessor.class);
-		BeanDefinition targetDefinition = new RootBeanDefinition(
-				ScheduledAnnotationBeanPostProcessorTests.CronWithInvalidTimezoneTestBean.class);
+		BeanDefinition targetDefinition = new RootBeanDefinition(CronWithInvalidTimezoneTestBean.class);
 		context.registerBeanDefinition("postProcessor", processorDefinition);
 		context.registerBeanDefinition("target", targetDefinition);
 		context.refresh();
@@ -286,8 +301,7 @@ public class ScheduledAnnotationBeanPostProcessorTests {
 	public void cronTaskWithMethodValidation() throws InterruptedException {
 		BeanDefinition validationDefinition = new RootBeanDefinition(MethodValidationPostProcessor.class);
 		BeanDefinition processorDefinition = new RootBeanDefinition(ScheduledAnnotationBeanPostProcessor.class);
-		BeanDefinition targetDefinition = new RootBeanDefinition(
-				ScheduledAnnotationBeanPostProcessorTests.CronTestBean.class);
+		BeanDefinition targetDefinition = new RootBeanDefinition(CronTestBean.class);
 		context.registerBeanDefinition("methodValidation", validationDefinition);
 		context.registerBeanDefinition("postProcessor", processorDefinition);
 		context.registerBeanDefinition("target", targetDefinition);
@@ -301,6 +315,7 @@ public class ScheduledAnnotationBeanPostProcessorTests {
 		context.registerBeanDefinition("postProcessor", processorDefinition);
 		context.registerBeanDefinition("target", targetDefinition);
 		context.refresh();
+
 		Object postProcessor = context.getBean("postProcessor");
 		Object target = context.getBean("target");
 		ScheduledTaskRegistrar registrar = (ScheduledTaskRegistrar)
@@ -319,13 +334,39 @@ public class ScheduledAnnotationBeanPostProcessorTests {
 	}
 
 	@Test
-	public void metaAnnotationWithCronExpression() {
+	public void composedAnnotationWithInitialDelayAndFixedRate() {
 		BeanDefinition processorDefinition = new RootBeanDefinition(ScheduledAnnotationBeanPostProcessor.class);
-		BeanDefinition targetDefinition = new RootBeanDefinition(
-				ScheduledAnnotationBeanPostProcessorTests.MetaAnnotationCronTestBean.class);
+		BeanDefinition targetDefinition = new RootBeanDefinition(ComposedAnnotationFixedRateTestBean.class);
 		context.registerBeanDefinition("postProcessor", processorDefinition);
 		context.registerBeanDefinition("target", targetDefinition);
 		context.refresh();
+
+		Object postProcessor = context.getBean("postProcessor");
+		Object target = context.getBean("target");
+		ScheduledTaskRegistrar registrar = (ScheduledTaskRegistrar) new DirectFieldAccessor(
+			postProcessor).getPropertyValue("registrar");
+		@SuppressWarnings("unchecked")
+		List<IntervalTask> fixedRateTasks = (List<IntervalTask>) new DirectFieldAccessor(registrar).getPropertyValue(
+			"fixedRateTasks");
+		assertEquals(1, fixedRateTasks.size());
+		IntervalTask task = fixedRateTasks.get(0);
+		ScheduledMethodRunnable runnable = (ScheduledMethodRunnable) task.getRunnable();
+		Object targetObject = runnable.getTarget();
+		Method targetMethod = runnable.getMethod();
+		assertEquals(target, targetObject);
+		assertEquals("checkForUpdates", targetMethod.getName());
+		assertEquals(5000L, task.getInterval());
+		assertEquals(1000L, task.getInitialDelay());
+	}
+
+	@Test
+	public void metaAnnotationWithCronExpression() {
+		BeanDefinition processorDefinition = new RootBeanDefinition(ScheduledAnnotationBeanPostProcessor.class);
+		BeanDefinition targetDefinition = new RootBeanDefinition(MetaAnnotationCronTestBean.class);
+		context.registerBeanDefinition("postProcessor", processorDefinition);
+		context.registerBeanDefinition("target", targetDefinition);
+		context.refresh();
+
 		Object postProcessor = context.getBean("postProcessor");
 		Object target = context.getBean("target");
 		ScheduledTaskRegistrar registrar = (ScheduledTaskRegistrar)
@@ -352,10 +393,11 @@ public class ScheduledAnnotationBeanPostProcessorTests {
 		properties.setProperty("schedules.businessHours", businessHoursCronExpression);
 		placeholderDefinition.getPropertyValues().addPropertyValue("properties", properties);
 		BeanDefinition targetDefinition = new RootBeanDefinition(PropertyPlaceholderWithCronTestBean.class);
-		context.registerBeanDefinition("placeholder", placeholderDefinition);
 		context.registerBeanDefinition("postProcessor", processorDefinition);
+		context.registerBeanDefinition("placeholder", placeholderDefinition);
 		context.registerBeanDefinition("target", targetDefinition);
 		context.refresh();
+
 		Object postProcessor = context.getBean("postProcessor");
 		Object target = context.getBean("target");
 		ScheduledTaskRegistrar registrar = (ScheduledTaskRegistrar)
@@ -382,10 +424,11 @@ public class ScheduledAnnotationBeanPostProcessorTests {
 		properties.setProperty("initialDelay", "1000");
 		placeholderDefinition.getPropertyValues().addPropertyValue("properties", properties);
 		BeanDefinition targetDefinition = new RootBeanDefinition(PropertyPlaceholderWithFixedDelayTestBean.class);
-		context.registerBeanDefinition("placeholder", placeholderDefinition);
 		context.registerBeanDefinition("postProcessor", processorDefinition);
+		context.registerBeanDefinition("placeholder", placeholderDefinition);
 		context.registerBeanDefinition("target", targetDefinition);
 		context.refresh();
+
 		Object postProcessor = context.getBean("postProcessor");
 		Object target = context.getBean("target");
 		ScheduledTaskRegistrar registrar = (ScheduledTaskRegistrar)
@@ -413,10 +456,11 @@ public class ScheduledAnnotationBeanPostProcessorTests {
 		properties.setProperty("initialDelay", "1000");
 		placeholderDefinition.getPropertyValues().addPropertyValue("properties", properties);
 		BeanDefinition targetDefinition = new RootBeanDefinition(PropertyPlaceholderWithFixedRateTestBean.class);
-		context.registerBeanDefinition("placeholder", placeholderDefinition);
 		context.registerBeanDefinition("postProcessor", processorDefinition);
+		context.registerBeanDefinition("placeholder", placeholderDefinition);
 		context.registerBeanDefinition("target", targetDefinition);
 		context.refresh();
+
 		Object postProcessor = context.getBean("postProcessor");
 		Object target = context.getBean("target");
 		ScheduledTaskRegistrar registrar = (ScheduledTaskRegistrar)
@@ -435,6 +479,35 @@ public class ScheduledAnnotationBeanPostProcessorTests {
 		assertEquals(3000L, task.getInterval());
 	}
 
+	@Test
+	public void expressionWithCron() {
+		String businessHoursCronExpression = "0 0 9-17 * * MON-FRI";
+		BeanDefinition processorDefinition = new RootBeanDefinition(ScheduledAnnotationBeanPostProcessor.class);
+		BeanDefinition targetDefinition = new RootBeanDefinition(ExpressionWithCronTestBean.class);
+		context.registerBeanDefinition("postProcessor", processorDefinition);
+		context.registerBeanDefinition("target", targetDefinition);
+		Map<String, String> schedules = new HashMap<String, String>();
+		schedules.put("businessHours", businessHoursCronExpression);
+		context.getBeanFactory().registerSingleton("schedules", schedules);
+		context.refresh();
+
+		Object postProcessor = context.getBean("postProcessor");
+		Object target = context.getBean("target");
+		ScheduledTaskRegistrar registrar = (ScheduledTaskRegistrar)
+				new DirectFieldAccessor(postProcessor).getPropertyValue("registrar");
+		@SuppressWarnings("unchecked")
+		List<CronTask> cronTasks = (List<CronTask>)
+				new DirectFieldAccessor(registrar).getPropertyValue("cronTasks");
+		assertEquals(1, cronTasks.size());
+		CronTask task = cronTasks.get(0);
+		ScheduledMethodRunnable runnable = (ScheduledMethodRunnable) task.getRunnable();
+		Object targetObject = runnable.getTarget();
+		Method targetMethod = runnable.getMethod();
+		assertEquals(target, targetObject);
+		assertEquals("x", targetMethod.getName());
+		assertEquals(businessHoursCronExpression, task.getExpression());
+	}
+
 	@Test
 	public void propertyPlaceholderForMetaAnnotation() {
 		String businessHoursCronExpression = "0 0 9-17 * * MON-FRI";
@@ -443,12 +516,12 @@ public class ScheduledAnnotationBeanPostProcessorTests {
 		Properties properties = new Properties();
 		properties.setProperty("schedules.businessHours", businessHoursCronExpression);
 		placeholderDefinition.getPropertyValues().addPropertyValue("properties", properties);
-		BeanDefinition targetDefinition = new RootBeanDefinition(
-				ScheduledAnnotationBeanPostProcessorTests.PropertyPlaceholderMetaAnnotationTestBean.class);
+		BeanDefinition targetDefinition = new RootBeanDefinition(PropertyPlaceholderMetaAnnotationTestBean.class);
 		context.registerBeanDefinition("postProcessor", processorDefinition);
 		context.registerBeanDefinition("placeholder", placeholderDefinition);
 		context.registerBeanDefinition("target", targetDefinition);
 		context.refresh();
+
 		Object postProcessor = context.getBean("postProcessor");
 		Object target = context.getBean("target");
 		ScheduledTaskRegistrar registrar = (ScheduledTaskRegistrar)
@@ -466,11 +539,35 @@ public class ScheduledAnnotationBeanPostProcessorTests {
 		assertEquals(businessHoursCronExpression, task.getExpression());
 	}
 
+	@Test
+	public void nonVoidReturnType() {
+		BeanDefinition processorDefinition = new RootBeanDefinition(ScheduledAnnotationBeanPostProcessor.class);
+		BeanDefinition targetDefinition = new RootBeanDefinition(NonVoidReturnTypeTestBean.class);
+		context.registerBeanDefinition("postProcessor", processorDefinition);
+		context.registerBeanDefinition("target", targetDefinition);
+		context.refresh();
+
+		Object postProcessor = context.getBean("postProcessor");
+		Object target = context.getBean("target");
+		ScheduledTaskRegistrar registrar = (ScheduledTaskRegistrar)
+				new DirectFieldAccessor(postProcessor).getPropertyValue("registrar");
+		@SuppressWarnings("unchecked")
+		List<CronTask> cronTasks = (List<CronTask>)
+				new DirectFieldAccessor(registrar).getPropertyValue("cronTasks");
+		assertEquals(1, cronTasks.size());
+		CronTask task = cronTasks.get(0);
+		ScheduledMethodRunnable runnable = (ScheduledMethodRunnable) task.getRunnable();
+		Object targetObject = runnable.getTarget();
+		Method targetMethod = runnable.getMethod();
+		assertEquals(target, targetObject);
+		assertEquals("cron", targetMethod.getName());
+		assertEquals("0 0 9-17 * * MON-FRI", task.getExpression());
+	}
+
 	@Test(expected = BeanCreationException.class)
 	public void emptyAnnotation() {
 		BeanDefinition processorDefinition = new RootBeanDefinition(ScheduledAnnotationBeanPostProcessor.class);
-		BeanDefinition targetDefinition = new RootBeanDefinition(
-				ScheduledAnnotationBeanPostProcessorTests.EmptyAnnotationTestBean.class);
+		BeanDefinition targetDefinition = new RootBeanDefinition(EmptyAnnotationTestBean.class);
 		context.registerBeanDefinition("postProcessor", processorDefinition);
 		context.registerBeanDefinition("target", targetDefinition);
 		context.refresh();
@@ -479,18 +576,7 @@ public class ScheduledAnnotationBeanPostProcessorTests {
 	@Test(expected = BeanCreationException.class)
 	public void invalidCron() throws Throwable {
 		BeanDefinition processorDefinition = new RootBeanDefinition(ScheduledAnnotationBeanPostProcessor.class);
-		BeanDefinition targetDefinition = new RootBeanDefinition(
-				ScheduledAnnotationBeanPostProcessorTests.InvalidCronTestBean.class);
-		context.registerBeanDefinition("postProcessor", processorDefinition);
-		context.registerBeanDefinition("target", targetDefinition);
-		context.refresh();
-	}
-
-	@Test(expected = BeanCreationException.class)
-	public void nonVoidReturnType() {
-		BeanDefinition processorDefinition = new RootBeanDefinition(ScheduledAnnotationBeanPostProcessor.class);
-		BeanDefinition targetDefinition = new RootBeanDefinition(
-				ScheduledAnnotationBeanPostProcessorTests.NonVoidReturnTypeTestBean.class);
+		BeanDefinition targetDefinition = new RootBeanDefinition(InvalidCronTestBean.class);
 		context.registerBeanDefinition("postProcessor", processorDefinition);
 		context.registerBeanDefinition("target", targetDefinition);
 		context.refresh();
@@ -499,8 +585,7 @@ public class ScheduledAnnotationBeanPostProcessorTests {
 	@Test(expected = BeanCreationException.class)
 	public void nonEmptyParamList() {
 		BeanDefinition processorDefinition = new RootBeanDefinition(ScheduledAnnotationBeanPostProcessor.class);
-		BeanDefinition targetDefinition = new RootBeanDefinition(
-				ScheduledAnnotationBeanPostProcessorTests.NonEmptyParamListTestBean.class);
+		BeanDefinition targetDefinition = new RootBeanDefinition(NonEmptyParamListTestBean.class);
 		context.registerBeanDefinition("postProcessor", processorDefinition);
 		context.registerBeanDefinition("target", targetDefinition);
 		context.refresh();
@@ -509,7 +594,7 @@ public class ScheduledAnnotationBeanPostProcessorTests {
 
 	static class FixedDelayTestBean {
 
-		@Scheduled(fixedDelay=5000)
+		@Scheduled(fixedDelay = 5000)
 		public void fixedDelay() {
 		}
 	}
@@ -517,7 +602,7 @@ public class ScheduledAnnotationBeanPostProcessorTests {
 
 	static class FixedRateTestBean {
 
-		@Scheduled(fixedRate=3000)
+		@Scheduled(fixedRate = 3000)
 		public void fixedRate() {
 		}
 	}
@@ -525,7 +610,7 @@ public class ScheduledAnnotationBeanPostProcessorTests {
 
 	static class FixedRateWithInitialDelayTestBean {
 
-		@Scheduled(fixedRate=3000, initialDelay=1000)
+		@Scheduled(fixedRate = 3000, initialDelay = 1000)
 		public void fixedRate() {
 		}
 	}
@@ -542,20 +627,43 @@ public class ScheduledAnnotationBeanPostProcessorTests {
 
 	static class SeveralFixedRatesWithRepeatedScheduledAnnotationTestBean {
 
-		// can use Java 8 repeated @Scheduled once we have Eclipse IDE support for it
-		@Schedules({
-			@Scheduled(fixedRate=4000),
-			@Scheduled(fixedRate=4000, initialDelay=2000)
-		})
+		@Scheduled(fixedRate = 4000)
+		@Scheduled(fixedRate = 4000, initialDelay = 2000)
 		public void fixedRate() {
 		}
 	}
 
 
+	static class FixedRatesBaseBean {
+
+		@Scheduled(fixedRate = 4000)
+		@Scheduled(fixedRate = 4000, initialDelay = 2000)
+		public void fixedRate() {
+		}
+	}
+
+
+	static class FixedRatesSubBean extends FixedRatesBaseBean {
+	}
+
+
+	interface FixedRatesDefaultMethod {
+
+		@Scheduled(fixedRate = 4000)
+		@Scheduled(fixedRate = 4000, initialDelay = 2000)
+		default void fixedRate() {
+		}
+	}
+
+
+	static class FixedRatesDefaultBean implements FixedRatesDefaultMethod {
+	}
+
+
 	@Validated
 	static class CronTestBean {
 
-		@Scheduled(cron="*/7 * * * * ?")
+		@Scheduled(cron = "*/7 * * * * ?")
 		private void cron() throws IOException {
 			throw new IOException("no no no");
 		}
@@ -564,7 +672,7 @@ public class ScheduledAnnotationBeanPostProcessorTests {
 
 	static class CronWithTimezoneTestBean {
 
-		@Scheduled(cron="0 0 0-4,6-23 * * ?", zone = "GMT+10")
+		@Scheduled(cron = "0 0 0-4,6-23 * * ?", zone = "GMT+10")
 		protected void cron() throws IOException {
 			throw new IOException("no no no");
 		}
@@ -573,13 +681,22 @@ public class ScheduledAnnotationBeanPostProcessorTests {
 
 	static class CronWithInvalidTimezoneTestBean {
 
-		@Scheduled(cron="0 0 0-4,6-23 * * ?", zone = "FOO")
+		@Scheduled(cron = "0 0 0-4,6-23 * * ?", zone = "FOO")
 		public void cron() throws IOException {
 			throw new IOException("no no no");
 		}
 	}
 
 
+	static class NonVoidReturnTypeTestBean {
+
+		@Scheduled(cron = "0 0 9-17 * * MON-FRI")
+		public String cron() {
+			return "oops";
+		}
+	}
+
+
 	static class EmptyAnnotationTestBean {
 
 		@Scheduled
@@ -590,39 +707,42 @@ public class ScheduledAnnotationBeanPostProcessorTests {
 
 	static class InvalidCronTestBean {
 
-		@Scheduled(cron="abc")
+		@Scheduled(cron = "abc")
 		public void invalid() {
 		}
 	}
 
 
-	static class NonVoidReturnTypeTestBean {
-
-		@Scheduled(fixedRate=3000)
-		public String invalid() {
-			return "oops";
-		}
-	}
-
-
 	static class NonEmptyParamListTestBean {
 
-		@Scheduled(fixedRate=3000)
+		@Scheduled(fixedRate = 3000)
 		public void invalid(String oops) {
 		}
 	}
 
 
-	@Scheduled(fixedRate=5000)
+	@Scheduled(fixedRate = 5000)
 	@Target(ElementType.METHOD)
 	@Retention(RetentionPolicy.RUNTIME)
-	private static @interface EveryFiveSeconds {}
+	private @interface EveryFiveSeconds {
+	}
 
-
-	@Scheduled(cron="0 0 * * * ?")
+	@Scheduled(cron = "0 0 * * * ?")
 	@Target(ElementType.METHOD)
 	@Retention(RetentionPolicy.RUNTIME)
-	private static @interface Hourly {}
+	private @interface Hourly {
+	}
+
+	@Scheduled(initialDelay = 1000)
+	@Retention(RetentionPolicy.RUNTIME)
+	private @interface WaitASec {
+
+		@AliasFor(annotation = Scheduled.class)
+		long fixedDelay() default -1;
+
+		@AliasFor(annotation = Scheduled.class)
+		long fixedRate() default -1;
+	}
 
 
 	static class MetaAnnotationFixedRateTestBean {
@@ -633,6 +753,14 @@ public class ScheduledAnnotationBeanPostProcessorTests {
 	}
 
 
+	static class ComposedAnnotationFixedRateTestBean {
+
+		@WaitASec(fixedRate = 5000)
+		public void checkForUpdates() {
+		}
+	}
+
+
 	static class MetaAnnotationCronTestBean {
 
 		@Hourly
@@ -640,7 +768,6 @@ public class ScheduledAnnotationBeanPostProcessorTests {
 		}
 	}
 
-
 	static class PropertyPlaceholderWithCronTestBean {
 
 		@Scheduled(cron = "${schedules.businessHours}")
@@ -651,7 +778,7 @@ public class ScheduledAnnotationBeanPostProcessorTests {
 
 	static class PropertyPlaceholderWithFixedDelayTestBean {
 
-		@Scheduled(fixedDelayString="${fixedDelay}", initialDelayString="${initialDelay}")
+		@Scheduled(fixedDelayString = "${fixedDelay}", initialDelayString = "${initialDelay}")
 		public void fixedDelay() {
 		}
 	}
@@ -659,16 +786,24 @@ public class ScheduledAnnotationBeanPostProcessorTests {
 
 	static class PropertyPlaceholderWithFixedRateTestBean {
 
-		@Scheduled(fixedRateString="${fixedRate}", initialDelayString="${initialDelay}")
+		@Scheduled(fixedRateString = "${fixedRate}", initialDelayString = "${initialDelay}")
 		public void fixedRate() {
 		}
 	}
 
 
-	@Scheduled(cron="${schedules.businessHours}")
+	static class ExpressionWithCronTestBean {
+
+		@Scheduled(cron = "#{schedules.businessHours}")
+		public void x() {
+		}
+	}
+
+
+	@Scheduled(cron = "${schedules.businessHours}")
 	@Target(ElementType.METHOD)
 	@Retention(RetentionPolicy.RUNTIME)
-	private static @interface BusinessHours {
+	private @interface BusinessHours {
 	}
 
 
