@@ -1,5 +1,5 @@
 /*
- * Copyright 2002-2014 the original author or authors.
+ * Copyright 2002-2016 the original author or authors.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -29,6 +29,7 @@ import java.util.List;
 import java.util.Map;
 import java.util.NavigableMap;
 import java.util.NavigableSet;
+import java.util.Properties;
 import java.util.Set;
 import java.util.SortedMap;
 import java.util.SortedSet;
@@ -40,11 +41,9 @@ import org.springframework.util.LinkedMultiValueMap;
 import org.springframework.util.MultiValueMap;
 
 /**
- * Factory for collections that is aware of Java 5 and Java 6 collection types.
+ * Factory for collections that is aware of Java 5, Java 6, and Spring collection types.
+ *
  * <p>Mainly for internal use within the framework.
- * <p>The goal of this class is to avoid runtime dependencies on a specific
- * Java version, while nevertheless using the best collection implementation
- * that is available at runtime.
  *
  * @author Juergen Hoeller
  * @author Arjen Poutsma
@@ -96,9 +95,17 @@ public abstract class CollectionFactory {
 
 	/**
 	 * Create the most approximate collection for the given collection.
-	 * @param collection the original collection object
+	 * <p><strong>Warning</strong>: Since the parameterized type {@code E} is
+	 * not bound to the type of elements contained in the supplied
+	 * {@code collection}, type safety cannot be guaranteed if the supplied
+	 * {@code collection} is an {@link EnumSet}. In such scenarios, the caller
+	 * is responsible for ensuring that the element type for the supplied
+	 * {@code collection} is an enum type matching type {@code E}. As an
+	 * alternative, the caller may wish to treat the return value as a raw
+	 * collection or collection of {@link Object}.
+	 * @param collection the original collection object, potentially {@code null}
 	 * @param capacity the initial capacity
-	 * @return the new, empty collection instance
+	 * @return a new, empty collection instance
 	 * @see #isApproximableCollectionType
 	 * @see java.util.LinkedList
 	 * @see java.util.ArrayList
@@ -132,9 +139,11 @@ public abstract class CollectionFactory {
 	 * Create the most appropriate collection for the given collection type.
 	 * <p>Delegates to {@link #createCollection(Class, Class, int)} with a
 	 * {@code null} element type.
-	 * @param collectionType the desired type of the target collection
+	 * @param collectionType the desired type of the target collection; never {@code null}
 	 * @param capacity the initial capacity
-	 * @return the new collection instance
+	 * @return a new collection instance
+	 * @throws IllegalArgumentException if the supplied {@code collectionType}
+	 * is {@code null} or of type {@link EnumSet}
 	 */
 	public static <E> Collection<E> createCollection(Class<?> collectionType, int capacity) {
 		return createCollection(collectionType, null, capacity);
@@ -142,35 +151,45 @@ public abstract class CollectionFactory {
 
 	/**
 	 * Create the most appropriate collection for the given collection type.
+	 * <p><strong>Warning</strong>: Since the parameterized type {@code E} is
+	 * not bound to the supplied {@code elementType}, type safety cannot be
+	 * guaranteed if the desired {@code collectionType} is {@link EnumSet}.
+	 * In such scenarios, the caller is responsible for ensuring that the
+	 * supplied {@code elementType} is an enum type matching type {@code E}.
+	 * As an alternative, the caller may wish to treat the return value as a
+	 * raw collection or collection of {@link Object}.
 	 * @param collectionType the desired type of the target collection; never {@code null}
-	 * @param elementType the collection's element type, or {@code null} if not known
+	 * @param elementType the collection's element type, or {@code null} if unknown
 	 * (note: only relevant for {@link EnumSet} creation)
 	 * @param capacity the initial capacity
-	 * @return the new collection instance
+	 * @return a new collection instance
 	 * @since 4.1.3
 	 * @see java.util.LinkedHashSet
 	 * @see java.util.ArrayList
 	 * @see java.util.TreeSet
 	 * @see java.util.EnumSet
+	 * @throws IllegalArgumentException if the supplied {@code collectionType} is
+	 * {@code null}; or if the desired {@code collectionType} is {@link EnumSet} and
+	 * the supplied {@code elementType} is not a subtype of {@link Enum}
 	 */
 	@SuppressWarnings({ "unchecked", "cast" })
 	public static <E> Collection<E> createCollection(Class<?> collectionType, Class<?> elementType, int capacity) {
 		Assert.notNull(collectionType, "Collection type must not be null");
 		if (collectionType.isInterface()) {
-			if (Set.class.equals(collectionType) || Collection.class.equals(collectionType)) {
+			if (Set.class == collectionType || Collection.class == collectionType) {
 				return new LinkedHashSet<E>(capacity);
 			}
-			else if (List.class.equals(collectionType)) {
+			else if (List.class == collectionType) {
 				return new ArrayList<E>(capacity);
 			}
-			else if (SortedSet.class.equals(collectionType) || NavigableSet.class.equals(collectionType)) {
+			else if (SortedSet.class == collectionType || NavigableSet.class == collectionType) {
 				return new TreeSet<E>();
 			}
 			else {
 				throw new IllegalArgumentException("Unsupported Collection interface: " + collectionType.getName());
 			}
 		}
-		else if (EnumSet.class.equals(collectionType)) {
+		else if (EnumSet.class == collectionType) {
 			Assert.notNull(elementType, "Cannot create EnumSet for unknown element type");
 			// Cast is necessary for compilation in Eclipse 4.4.1.
 			return (Collection<E>) EnumSet.noneOf(asEnumType(elementType));
@@ -182,7 +201,7 @@ public abstract class CollectionFactory {
 			try {
 				return (Collection<E>) collectionType.newInstance();
 			}
-			catch (Exception ex) {
+			catch (Throwable ex) {
 				throw new IllegalArgumentException(
 					"Could not instantiate Collection type: " + collectionType.getName(), ex);
 			}
@@ -201,9 +220,16 @@ public abstract class CollectionFactory {
 
 	/**
 	 * Create the most approximate map for the given map.
-	 * @param map the original Map object
+	 * <p><strong>Warning</strong>: Since the parameterized type {@code K} is
+	 * not bound to the type of keys contained in the supplied {@code map},
+	 * type safety cannot be guaranteed if the supplied {@code map} is an
+	 * {@link EnumMap}. In such scenarios, the caller is responsible for
+	 * ensuring that the key type in the supplied {@code map} is an enum type
+	 * matching type {@code K}. As an alternative, the caller may wish to
+	 * treat the return value as a raw map or map keyed by {@link Object}.
+	 * @param map the original map object, potentially {@code null}
 	 * @param capacity the initial capacity
-	 * @return the new, empty Map instance
+	 * @return a new, empty map instance
 	 * @see #isApproximableMapType
 	 * @see java.util.EnumMap
 	 * @see java.util.TreeMap
@@ -228,9 +254,11 @@ public abstract class CollectionFactory {
 	 * Create the most appropriate map for the given map type.
 	 * <p>Delegates to {@link #createMap(Class, Class, int)} with a
 	 * {@code null} key type.
-	 * @param mapType the desired type of the target Map
+	 * @param mapType the desired type of the target map
 	 * @param capacity the initial capacity
-	 * @return the new Map instance
+	 * @return a new map instance
+	 * @throws IllegalArgumentException if the supplied {@code mapType} is
+	 * {@code null} or of type {@link EnumMap}
 	 */
 	public static <K, V> Map<K, V> createMap(Class<?> mapType, int capacity) {
 		return createMap(mapType, null, capacity);
@@ -238,35 +266,46 @@ public abstract class CollectionFactory {
 
 	/**
 	 * Create the most appropriate map for the given map type.
-	 * @param mapType the desired type of the target Map
-	 * @param keyType the map's key type, or {@code null} if not known
+	 * <p><strong>Warning</strong>: Since the parameterized type {@code K}
+	 * is not bound to the supplied {@code keyType}, type safety cannot be
+	 * guaranteed if the desired {@code mapType} is {@link EnumMap}. In such
+	 * scenarios, the caller is responsible for ensuring that the {@code keyType}
+	 * is an enum type matching type {@code K}. As an alternative, the caller
+	 * may wish to treat the return value as a raw map or map keyed by
+	 * {@link Object}. Similarly, type safety cannot be enforced if the
+	 * desired {@code mapType} is {@link MultiValueMap}.
+	 * @param mapType the desired type of the target map; never {@code null}
+	 * @param keyType the map's key type, or {@code null} if unknown
 	 * (note: only relevant for {@link EnumMap} creation)
 	 * @param capacity the initial capacity
-	 * @return the new Map instance
+	 * @return a new map instance
 	 * @since 4.1.3
 	 * @see java.util.LinkedHashMap
 	 * @see java.util.TreeMap
 	 * @see org.springframework.util.LinkedMultiValueMap
 	 * @see java.util.EnumMap
+	 * @throws IllegalArgumentException if the supplied {@code mapType} is
+	 * {@code null}; or if the desired {@code mapType} is {@link EnumMap} and
+	 * the supplied {@code keyType} is not a subtype of {@link Enum}
 	 */
 	@SuppressWarnings({"unchecked", "rawtypes"})
 	public static <K, V> Map<K, V> createMap(Class<?> mapType, Class<?> keyType, int capacity) {
 		Assert.notNull(mapType, "Map type must not be null");
 		if (mapType.isInterface()) {
-			if (Map.class.equals(mapType)) {
+			if (Map.class == mapType) {
 				return new LinkedHashMap<K, V>(capacity);
 			}
-			else if (SortedMap.class.equals(mapType) || NavigableMap.class.equals(mapType)) {
+			else if (SortedMap.class == mapType || NavigableMap.class == mapType) {
 				return new TreeMap<K, V>();
 			}
-			else if (MultiValueMap.class.equals(mapType)) {
+			else if (MultiValueMap.class == mapType) {
 				return new LinkedMultiValueMap();
 			}
 			else {
 				throw new IllegalArgumentException("Unsupported Map interface: " + mapType.getName());
 			}
 		}
-		else if (EnumMap.class.equals(mapType)) {
+		else if (EnumMap.class == mapType) {
 			Assert.notNull(keyType, "Cannot create EnumMap for unknown key type");
 			return new EnumMap(asEnumType(keyType));
 		}
@@ -277,12 +316,29 @@ public abstract class CollectionFactory {
 			try {
 				return (Map<K, V>) mapType.newInstance();
 			}
-			catch (Exception ex) {
+			catch (Throwable ex) {
 				throw new IllegalArgumentException("Could not instantiate Map type: " + mapType.getName(), ex);
 			}
 		}
 	}
 
+	/**
+	 * Create a variant of {@code java.util.Properties} that automatically adapts
+	 * non-String values to String representations on {@link Properties#getProperty}.
+	 * @return a new {@code Properties} instance
+	 * @since 4.3.4
+	 */
+	@SuppressWarnings("serial")
+	public static Properties createStringAdaptingProperties() {
+		return new Properties() {
+			@Override
+			public String getProperty(String key) {
+				Object value = get(key);
+				return (value != null ? value.toString() : null);
+			}
+		};
+	}
+
 	/**
 	 * Cast the given type to a subtype of {@link Enum}.
 	 * @param enumType the enum type, never {@code null}
