@@ -1,5 +1,5 @@
 /*
- * Copyright 2002-2014 the original author or authors.
+ * Copyright 2002-2016 the original author or authors.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -51,10 +51,10 @@ import java.util.concurrent.locks.ReentrantLock;
  * <p>If not explicitly specified, this implementation will use
  * {@linkplain SoftReference soft entry references}.
  *
- * @param <K> The key type
- * @param <V> The value type
  * @author Phillip Webb
  * @since 3.2
+ * @param <K> the key type
+ * @param <V> the value type
  */
 public class ConcurrentReferenceHashMap<K, V> extends AbstractMap<K, V> implements ConcurrentMap<K, V> {
 
@@ -252,6 +252,7 @@ public class ConcurrentReferenceHashMap<K, V> extends AbstractMap<K, V> implemen
 		return put(key, value, true);
 	}
 
+	@Override
 	public V putIfAbsent(K key, V value) {
 		return put(key, value, false);
 	}
@@ -287,6 +288,7 @@ public class ConcurrentReferenceHashMap<K, V> extends AbstractMap<K, V> implemen
 		});
 	}
 
+	@Override
 	public boolean remove(Object key, final Object value) {
 		return doTask(key, new Task<Boolean>(TaskOption.RESTRUCTURE_AFTER, TaskOption.SKIP_IF_EMPTY) {
 			@Override
@@ -300,6 +302,7 @@ public class ConcurrentReferenceHashMap<K, V> extends AbstractMap<K, V> implemen
 		});
 	}
 
+	@Override
 	public boolean replace(K key, final V oldValue, final V newValue) {
 		return doTask(key, new Task<Boolean>(TaskOption.RESTRUCTURE_BEFORE, TaskOption.SKIP_IF_EMPTY) {
 			@Override
@@ -313,6 +316,7 @@ public class ConcurrentReferenceHashMap<K, V> extends AbstractMap<K, V> implemen
 		});
 	}
 
+	@Override
 	public V replace(K key, final V value) {
 		return doTask(key, new Task<V>(TaskOption.RESTRUCTURE_BEFORE, TaskOption.SKIP_IF_EMPTY) {
 			@Override
@@ -394,7 +398,7 @@ public class ConcurrentReferenceHashMap<K, V> extends AbstractMap<K, V> implemen
 	/**
 	 * Various reference types supported by this map.
 	 */
-	public static enum ReferenceType {
+	public enum ReferenceType {
 
 		/** Use {@link SoftReference}s */
 		SOFT,
@@ -632,7 +636,7 @@ public class ConcurrentReferenceHashMap<K, V> extends AbstractMap<K, V> implemen
 	 * A reference to an {@link Entry} contained in the map. Implementations are usually
 	 * wrappers around specific Java reference implementations (e.g., {@link SoftReference}).
 	 */
-	protected static interface Reference<K, V> {
+	protected interface Reference<K, V> {
 
 		/**
 		 * Returns the referenced entry or {@code null} if the entry is no longer
@@ -675,14 +679,17 @@ public class ConcurrentReferenceHashMap<K, V> extends AbstractMap<K, V> implemen
 			this.value = value;
 		}
 
+		@Override
 		public K getKey() {
 			return this.key;
 		}
 
+		@Override
 		public V getValue() {
 			return this.value;
 		}
 
+		@Override
 		public V setValue(V value) {
 			V previous = this.value;
 			this.value = value;
@@ -758,7 +765,7 @@ public class ConcurrentReferenceHashMap<K, V> extends AbstractMap<K, V> implemen
 	/**
 	 * Various options supported by a {@code Task}.
 	 */
-	private static enum TaskOption {
+	private enum TaskOption {
 
 		RESTRUCTURE_BEFORE, RESTRUCTURE_AFTER, SKIP_IF_EMPTY, RESIZE
 	}
@@ -842,11 +849,13 @@ public class ConcurrentReferenceHashMap<K, V> extends AbstractMap<K, V> implemen
 			moveToNextSegment();
 		}
 
+		@Override
 		public boolean hasNext() {
 			getNextIfNecessary();
 			return (this.next != null);
 		}
 
+		@Override
 		public Entry<K, V> next() {
 			getNextIfNecessary();
 			if (this.next == null) {
@@ -892,6 +901,7 @@ public class ConcurrentReferenceHashMap<K, V> extends AbstractMap<K, V> implemen
 			}
 		}
 
+		@Override
 		public void remove() {
 			Assert.state(this.last != null);
 			ConcurrentReferenceHashMap.this.remove(this.last.getKey());
@@ -902,7 +912,7 @@ public class ConcurrentReferenceHashMap<K, V> extends AbstractMap<K, V> implemen
 	/**
 	 * The types of restructuring that can be performed.
 	 */
-	protected static enum Restructure {
+	protected enum Restructure {
 
 		WHEN_NECESSARY, NEVER
 	}
@@ -959,14 +969,17 @@ public class ConcurrentReferenceHashMap<K, V> extends AbstractMap<K, V> implemen
 			this.nextReference = next;
 		}
 
+		@Override
 		public int getHash() {
 			return this.hash;
 		}
 
+		@Override
 		public Reference<K, V> getNext() {
 			return this.nextReference;
 		}
 
+		@Override
 		public void release() {
 			enqueue();
 			clear();
@@ -989,14 +1002,17 @@ public class ConcurrentReferenceHashMap<K, V> extends AbstractMap<K, V> implemen
 			this.nextReference = next;
 		}
 
+		@Override
 		public int getHash() {
 			return this.hash;
 		}
 
+		@Override
 		public Reference<K, V> getNext() {
 			return this.nextReference;
 		}
 
+		@Override
 		public void release() {
 			enqueue();
 			clear();
