@@ -1,5 +1,5 @@
 /*
- * Copyright 2002-2013 the original author or authors.
+ * Copyright 2002-2016 the original author or authors.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -18,10 +18,13 @@ package org.springframework.util;
 
 import java.lang.reflect.Array;
 import java.util.Arrays;
+import java.util.Collection;
+import java.util.Map;
 
 /**
  * Miscellaneous object utility methods.
- * Mainly for internal use within the framework.
+ *
+ * <p>Mainly for internal use within the framework.
  *
  * <p>Thanks to Alex Ruiz for contributing several enhancements to this class!
  *
@@ -30,7 +33,11 @@ import java.util.Arrays;
  * @author Rod Johnson
  * @author Rob Harrop
  * @author Chris Beams
+ * @author Sam Brannen
  * @since 19.03.2004
+ * @see ClassUtils
+ * @see CollectionUtils
+ * @see StringUtils
  */
 public abstract class ObjectUtils {
 
@@ -92,11 +99,55 @@ public abstract class ObjectUtils {
 	 * Determine whether the given array is empty:
 	 * i.e. {@code null} or of zero length.
 	 * @param array the array to check
+	 * @see #isEmpty(Object)
 	 */
 	public static boolean isEmpty(Object[] array) {
 		return (array == null || array.length == 0);
 	}
 
+	/**
+	 * Determine whether the given object is empty.
+	 * <p>This method supports the following object types.
+	 * <ul>
+	 * <li>{@code Array}: considered empty if its length is zero</li>
+	 * <li>{@link CharSequence}: considered empty if its length is zero</li>
+	 * <li>{@link Collection}: delegates to {@link Collection#isEmpty()}</li>
+	 * <li>{@link Map}: delegates to {@link Map#isEmpty()}</li>
+	 * </ul>
+	 * <p>If the given object is non-null and not one of the aforementioned
+	 * supported types, this method returns {@code false}.
+	 * @param obj the object to check
+	 * @return {@code true} if the object is {@code null} or <em>empty</em>
+	 * @since 4.2
+	 * @see ObjectUtils#isEmpty(Object[])
+	 * @see StringUtils#hasLength(CharSequence)
+	 * @see StringUtils#isEmpty(Object)
+	 * @see CollectionUtils#isEmpty(java.util.Collection)
+	 * @see CollectionUtils#isEmpty(java.util.Map)
+	 */
+	@SuppressWarnings("rawtypes")
+	public static boolean isEmpty(Object obj) {
+		if (obj == null) {
+			return true;
+		}
+
+		if (obj.getClass().isArray()) {
+			return Array.getLength(obj) == 0;
+		}
+		if (obj instanceof CharSequence) {
+			return ((CharSequence) obj).length() == 0;
+		}
+		if (obj instanceof Collection) {
+			return ((Collection) obj).isEmpty();
+		}
+		if (obj instanceof Map) {
+			return ((Map) obj).isEmpty();
+		}
+
+		// else
+		return false;
+	}
+
 	/**
 	 * Check whether the given array contains the given element.
 	 * @param array the array to check (may be {@code null},
@@ -226,14 +277,14 @@ public abstract class ObjectUtils {
 	//---------------------------------------------------------------------
 
 	/**
-	 * Determine if the given objects are equal, returning {@code true}
-	 * if both are {@code null} or {@code false} if only one is
-	 * {@code null}.
+	 * Determine if the given objects are equal, returning {@code true} if
+	 * both are {@code null} or {@code false} if only one is {@code null}.
 	 * <p>Compares arrays with {@code Arrays.equals}, performing an equality
 	 * check based on the array elements rather than the array reference.
 	 * @param o1 first Object to compare
 	 * @param o2 second Object to compare
 	 * @return whether the given objects are equal
+	 * @see Object#equals(Object)
 	 * @see java.util.Arrays#equals
 	 */
 	public static boolean nullSafeEquals(Object o1, Object o2) {
@@ -247,33 +298,47 @@ public abstract class ObjectUtils {
 			return true;
 		}
 		if (o1.getClass().isArray() && o2.getClass().isArray()) {
-			if (o1 instanceof Object[] && o2 instanceof Object[]) {
-				return Arrays.equals((Object[]) o1, (Object[]) o2);
-			}
-			if (o1 instanceof boolean[] && o2 instanceof boolean[]) {
-				return Arrays.equals((boolean[]) o1, (boolean[]) o2);
-			}
-			if (o1 instanceof byte[] && o2 instanceof byte[]) {
-				return Arrays.equals((byte[]) o1, (byte[]) o2);
-			}
-			if (o1 instanceof char[] && o2 instanceof char[]) {
-				return Arrays.equals((char[]) o1, (char[]) o2);
-			}
-			if (o1 instanceof double[] && o2 instanceof double[]) {
-				return Arrays.equals((double[]) o1, (double[]) o2);
-			}
-			if (o1 instanceof float[] && o2 instanceof float[]) {
-				return Arrays.equals((float[]) o1, (float[]) o2);
-			}
-			if (o1 instanceof int[] && o2 instanceof int[]) {
-				return Arrays.equals((int[]) o1, (int[]) o2);
-			}
-			if (o1 instanceof long[] && o2 instanceof long[]) {
-				return Arrays.equals((long[]) o1, (long[]) o2);
-			}
-			if (o1 instanceof short[] && o2 instanceof short[]) {
-				return Arrays.equals((short[]) o1, (short[]) o2);
-			}
+			return arrayEquals(o1, o2);
+		}
+		return false;
+	}
+
+	/**
+	 * Compare the given arrays with {@code Arrays.equals}, performing an equality
+	 * check based on the array elements rather than the array reference.
+	 * @param o1 first array to compare
+	 * @param o2 second array to compare
+	 * @return whether the given objects are equal
+	 * @see #nullSafeEquals(Object, Object)
+	 * @see java.util.Arrays#equals
+	 */
+	private static boolean arrayEquals(Object o1, Object o2) {
+		if (o1 instanceof Object[] && o2 instanceof Object[]) {
+			return Arrays.equals((Object[]) o1, (Object[]) o2);
+		}
+		if (o1 instanceof boolean[] && o2 instanceof boolean[]) {
+			return Arrays.equals((boolean[]) o1, (boolean[]) o2);
+		}
+		if (o1 instanceof byte[] && o2 instanceof byte[]) {
+			return Arrays.equals((byte[]) o1, (byte[]) o2);
+		}
+		if (o1 instanceof char[] && o2 instanceof char[]) {
+			return Arrays.equals((char[]) o1, (char[]) o2);
+		}
+		if (o1 instanceof double[] && o2 instanceof double[]) {
+			return Arrays.equals((double[]) o1, (double[]) o2);
+		}
+		if (o1 instanceof float[] && o2 instanceof float[]) {
+			return Arrays.equals((float[]) o1, (float[]) o2);
+		}
+		if (o1 instanceof int[] && o2 instanceof int[]) {
+			return Arrays.equals((int[]) o1, (int[]) o2);
+		}
+		if (o1 instanceof long[] && o2 instanceof long[]) {
+			return Arrays.equals((long[]) o1, (long[]) o2);
+		}
+		if (o1 instanceof short[] && o2 instanceof short[]) {
+			return Arrays.equals((short[]) o1, (short[]) o2);
 		}
 		return false;
 	}
@@ -284,6 +349,7 @@ public abstract class ObjectUtils {
 	 * this method will delegate to any of the {@code nullSafeHashCode}
 	 * methods for arrays in this class. If the object is {@code null},
 	 * this method returns 0.
+	 * @see Object#hashCode()
 	 * @see #nullSafeHashCode(Object[])
 	 * @see #nullSafeHashCode(boolean[])
 	 * @see #nullSafeHashCode(byte[])
