@@ -1,5 +1,5 @@
 /*
- * Copyright 2002-2013 the original author or authors.
+ * Copyright 2002-2016 the original author or authors.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -24,6 +24,7 @@ import org.junit.Before;
 import org.junit.Test;
 
 import org.springframework.core.convert.ConversionException;
+import org.springframework.core.convert.ConverterNotFoundException;
 import org.springframework.mock.env.MockPropertySource;
 
 import static org.hamcrest.Matchers.*;
@@ -114,9 +115,9 @@ public class PropertySourcesPropertyResolverTests {
 
 		try {
 			propertyResolver.getProperty("foo", TestType.class);
-			fail("Expected IllegalArgumentException due to non-convertible types");
+			fail("Expected ConverterNotFoundException due to non-convertible types");
 		}
-		catch (IllegalArgumentException ex) {
+		catch (ConverterNotFoundException ex) {
 			// expected
 		}
 	}
@@ -257,6 +258,7 @@ public class PropertySourcesPropertyResolverTests {
 	}
 
 	@Test
+	@SuppressWarnings("deprecation")
 	public void getPropertyAsClass() throws ClassNotFoundException, LinkageError {
 		MutablePropertySources propertySources = new MutablePropertySources();
 		propertySources.addFirst(new MockPropertySource().withProperty("some.class", SpecificType.class.getName()));
@@ -265,6 +267,7 @@ public class PropertySourcesPropertyResolverTests {
 	}
 
 	@Test
+	@SuppressWarnings("deprecation")
 	public void getPropertyAsClass_withInterfaceAsTarget() throws ClassNotFoundException, LinkageError {
 		MutablePropertySources propertySources = new MutablePropertySources();
 		propertySources.addFirst(new MockPropertySource().withProperty("some.class", SomeType.class.getName()));
@@ -272,7 +275,8 @@ public class PropertySourcesPropertyResolverTests {
 		assertTrue(resolver.getPropertyAsClass("some.class", SomeType.class).equals(SomeType.class));
 	}
 
-	@Test(expected=ConversionException.class)
+	@Test(expected = ConversionException.class)
+	@SuppressWarnings("deprecation")
 	public void getPropertyAsClass_withMismatchedTypeForValue() {
 		MutablePropertySources propertySources = new MutablePropertySources();
 		propertySources.addFirst(new MockPropertySource().withProperty("some.class", "java.lang.String"));
@@ -280,7 +284,8 @@ public class PropertySourcesPropertyResolverTests {
 		resolver.getPropertyAsClass("some.class", SomeType.class);
 	}
 
-	@Test(expected=ConversionException.class)
+	@Test(expected = ConversionException.class)
+	@SuppressWarnings("deprecation")
 	public void getPropertyAsClass_withNonExistentClassForValue() {
 		MutablePropertySources propertySources = new MutablePropertySources();
 		propertySources.addFirst(new MockPropertySource().withProperty("some.class", "some.bogus.Class"));
@@ -289,6 +294,7 @@ public class PropertySourcesPropertyResolverTests {
 	}
 
 	@Test
+	@SuppressWarnings("deprecation")
 	public void getPropertyAsClass_withObjectForValue() {
 		MutablePropertySources propertySources = new MutablePropertySources();
 		propertySources.addFirst(new MockPropertySource().withProperty("some.class", new SpecificType()));
@@ -296,7 +302,8 @@ public class PropertySourcesPropertyResolverTests {
 		assertTrue(resolver.getPropertyAsClass("some.class", SomeType.class).equals(SpecificType.class));
 	}
 
-	@Test(expected=ConversionException.class)
+	@Test(expected = ConversionException.class)
+	@SuppressWarnings("deprecation")
 	public void getPropertyAsClass_withMismatchedObjectForValue() {
 		MutablePropertySources propertySources = new MutablePropertySources();
 		propertySources.addFirst(new MockPropertySource().withProperty("some.class", new Integer(42)));
@@ -305,6 +312,7 @@ public class PropertySourcesPropertyResolverTests {
 	}
 
 	@Test
+	@SuppressWarnings("deprecation")
 	public void getPropertyAsClass_withRealClassForValue() {
 		MutablePropertySources propertySources = new MutablePropertySources();
 		propertySources.addFirst(new MockPropertySource().withProperty("some.class", SpecificType.class));
@@ -312,7 +320,8 @@ public class PropertySourcesPropertyResolverTests {
 		assertTrue(resolver.getPropertyAsClass("some.class", SomeType.class).equals(SpecificType.class));
 	}
 
-	@Test(expected=ConversionException.class)
+	@Test(expected = ConversionException.class)
+	@SuppressWarnings("deprecation")
 	public void getPropertyAsClass_withMismatchedRealClassForValue() {
 		MutablePropertySources propertySources = new MutablePropertySources();
 		propertySources.addFirst(new MockPropertySource().withProperty("some.class", Integer.class));
