@@ -462,7 +462,8 @@ public class ParsingTests {
 				fail("Parsed exception was null");
 			}
 			assertEquals("String form of AST does not match expected output", expectedStringFormOfAST, e.toStringAST());
-		} catch (ParseException ee) {
+		}
+		catch (ParseException ee) {
 			ee.printStackTrace();
 			fail("Unexpected Exception: " + ee.getMessage());
 		}
