@@ -78,15 +78,17 @@ public class PropertyAccessTests extends AbstractExpressionTests {
 		try {
 			expr.getValue(context);
 			fail("Should have failed - default property resolver cannot resolve on null");
-		} catch (Exception e) {
-			checkException(e,SpelMessage.PROPERTY_OR_FIELD_NOT_READABLE_ON_NULL);
+		}
+		catch (Exception ex) {
+			checkException(ex, SpelMessage.PROPERTY_OR_FIELD_NOT_READABLE_ON_NULL);
 		}
 		assertFalse(expr.isWritable(context));
 		try {
 			expr.setValue(context,"abc");
 			fail("Should have failed - default property resolver cannot resolve on null");
-		} catch (Exception e) {
-			checkException(e,SpelMessage.PROPERTY_OR_FIELD_NOT_WRITABLE_ON_NULL);
+		}
+		catch (Exception ex) {
+			checkException(ex, SpelMessage.PROPERTY_OR_FIELD_NOT_WRITABLE_ON_NULL);
 		}
 	}
 
@@ -94,7 +96,8 @@ public class PropertyAccessTests extends AbstractExpressionTests {
 		if (e instanceof SpelEvaluationException) {
 			SpelMessage sm = ((SpelEvaluationException)e).getMessageCode();
 			assertEquals("Expected exception type did not occur",expectedMessage,sm);
-		} else {
+		}
+		else {
 			fail("Should be a SpelException "+e);
 		}
 	}
@@ -127,7 +130,8 @@ public class PropertyAccessTests extends AbstractExpressionTests {
 		try {
 			expr.setValue(ctx, "not allowed");
 			fail("Should not have been allowed");
-		} catch (EvaluationException e) {
+		}
+		catch (EvaluationException ex) {
 			// success - message will be: EL1063E:(pos 20): A problem occurred whilst attempting to set the property
 			// 'flibbles': 'Cannot set flibbles to an object of type 'class java.lang.String''
 			// System.out.println(e.getMessage());
@@ -171,38 +175,42 @@ public class PropertyAccessTests extends AbstractExpressionTests {
 
 		@Override
 		public Class<?>[] getSpecificTargetClasses() {
-			return new Class[] { String.class };
+			return new Class[] {String.class};
 		}
 
 		@Override
 		public boolean canRead(EvaluationContext context, Object target, String name) throws AccessException {
-			if (!(target instanceof String))
+			if (!(target instanceof String)) {
 				throw new RuntimeException("Assertion Failed! target should be String");
+			}
 			return (name.equals("flibbles"));
 		}
 
 		@Override
 		public boolean canWrite(EvaluationContext context, Object target, String name) throws AccessException {
-			if (!(target instanceof String))
+			if (!(target instanceof String)) {
 				throw new RuntimeException("Assertion Failed! target should be String");
+			}
 			return (name.equals("flibbles"));
 		}
 
 		@Override
 		public TypedValue read(EvaluationContext context, Object target, String name) throws AccessException {
-			if (!name.equals("flibbles"))
+			if (!name.equals("flibbles")) {
 				throw new RuntimeException("Assertion Failed! name should be flibbles");
+			}
 			return new TypedValue(flibbles);
 		}
 
 		@Override
-		public void write(EvaluationContext context, Object target, String name, Object newValue)
-				throws AccessException {
-			if (!name.equals("flibbles"))
+		public void write(EvaluationContext context, Object target, String name, Object newValue) throws AccessException {
+			if (!name.equals("flibbles")) {
 				throw new RuntimeException("Assertion Failed! name should be flibbles");
+			}
 			try {
 				flibbles = (Integer) context.getTypeConverter().convertValue(newValue, TypeDescriptor.forObject(newValue), TypeDescriptor.valueOf(Integer.class));
-			}catch (EvaluationException e) {
+			}
+			catch (EvaluationException ex) {
 				throw new AccessException("Cannot set flibbles to an object of type '" + newValue.getClass() + "'");
 			}
 		}
