@@ -1,5 +1,5 @@
 /*
- * Copyright 2002-2013 the original author or authors.
+ * Copyright 2002-2014 the original author or authors.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -16,6 +16,7 @@
 
 package org.springframework.jdbc.core;
 
+import java.sql.BatchUpdateException;
 import java.sql.CallableStatement;
 import java.sql.Connection;
 import java.sql.DatabaseMetaData;
@@ -30,13 +31,13 @@ import java.util.Arrays;
 import java.util.LinkedList;
 import java.util.List;
 import java.util.Map;
-
 import javax.sql.DataSource;
 
 import org.junit.Before;
 import org.junit.Rule;
 import org.junit.Test;
 import org.junit.rules.ExpectedException;
+
 import org.springframework.dao.DataAccessException;
 import org.springframework.dao.InvalidDataAccessApiUsageException;
 import org.springframework.jdbc.BadSqlGrammarException;
@@ -458,6 +459,24 @@ public class JdbcTemplateTests {
 		verify(this.connection, atLeastOnce()).close();
 	}
 
+	@Test
+	public void testBatchUpdateWithBatchFailure() throws Exception {
+		final String[] sql = {"A", "B", "C", "D"};
+		given(this.statement.executeBatch()).willThrow(
+				new BatchUpdateException(new int[] { 1, Statement.EXECUTE_FAILED, 1,
+					Statement.EXECUTE_FAILED }));
+		mockDatabaseMetaData(true);
+		given(this.connection.createStatement()).willReturn(this.statement);
+
+		JdbcTemplate template = new JdbcTemplate(this.dataSource, false);
+		try {
+			template.batchUpdate(sql);
+		}
+		catch (UncategorizedSQLException ex) {
+			assertThat(ex.getSql(), equalTo("B; D"));
+		}
+	}
+
 	@Test
 	public void testBatchUpdateWithNoBatchSupport() throws Exception {
 		final String[] sql = {"UPDATE NOSUCHTABLE SET DATE_DISPATCHED = SYSDATE WHERE ID = 1",
@@ -1270,7 +1289,7 @@ public class JdbcTemplateTests {
 		List<SqlParameter> params = new ArrayList<SqlParameter>();
 		params.add(new SqlOutParameter("a", 12));
 
-		Map out = this.template.call(new CallableStatementCreator() {
+		Map<String, Object> out = this.template.call(new CallableStatementCreator() {
 			@Override
 			public CallableStatement createCallableStatement(Connection conn)
 					throws SQLException {
