@@ -1,5 +1,5 @@
 /*
- * Copyright 2002-2014 the original author or authors.
+ * Copyright 2002-2015 the original author or authors.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -16,34 +16,40 @@
 
 package org.springframework.jms.listener;
 
+import java.lang.reflect.InvocationTargetException;
+import java.lang.reflect.Method;
 import javax.jms.Connection;
 import javax.jms.Destination;
 import javax.jms.ExceptionListener;
 import javax.jms.JMSException;
 import javax.jms.Message;
+import javax.jms.MessageConsumer;
 import javax.jms.MessageListener;
 import javax.jms.Queue;
 import javax.jms.Session;
 import javax.jms.Topic;
 
 import org.springframework.jms.support.JmsUtils;
+import org.springframework.jms.support.converter.MessageConverter;
 import org.springframework.util.Assert;
+import org.springframework.util.ClassUtils;
 import org.springframework.util.ErrorHandler;
+import org.springframework.util.ReflectionUtils;
 
 /**
- * Abstract base class for message listener containers. Can either host
- * a standard JMS {@link javax.jms.MessageListener} or a Spring-specific
- * {@link SessionAwareMessageListener}.
+ * Abstract base class for Spring message listener container implementations.
+ * Can either host a standard JMS {@link javax.jms.MessageListener} or Spring's
+ * {@link SessionAwareMessageListener} for actual message processing.
  *
- * <p>Usually holds a single JMS {@link Connection} that all listeners are
- * supposed to be registered on, which is the standard JMS way of managing
- * listeners. Can alternatively also be used with a fresh Connection per
- * listener, for J2EE-style XA-aware JMS messaging. The actual registration
- * process is up to concrete subclasses.
+ * <p>Usually holds a single JMS {@link Connection} that all listeners are supposed
+ * to be registered on, which is the standard JMS way of managing listener sessions.
+ * Can alternatively also be used with a fresh Connection per listener, for Java EE
+ * style XA-aware JMS messaging. The actual registration process is up to concrete
+ * subclasses.
  *
- * <p><b>NOTE:</b> The default behavior of this message listener container
- * is to <b>never</b> propagate an exception thrown by a message listener up to
- * the JMS provider. Instead, it will log any such exception at the error level.
+ * <p><b>NOTE:</b> The default behavior of this message listener container is to
+ * <b>never</b> propagate an exception thrown by a message listener up to the JMS
+ * provider. Instead, it will log any such exception at the error level.
  * This means that from the perspective of the attendant JMS provider no such
  * listener will ever fail. However, if error handling is necessary, then
  * any implementation of the {@link ErrorHandler} strategy may be provided to
@@ -54,34 +60,50 @@ import org.springframework.util.ErrorHandler;
  * <p>The listener container offers the following message acknowledgment options:
  * <ul>
  * <li>"sessionAcknowledgeMode" set to "AUTO_ACKNOWLEDGE" (default):
- * Automatic message acknowledgment <i>before</i> listener execution;
- * no redelivery in case of exception thrown.
+ * This mode is container-dependent: For {@link DefaultMessageListenerContainer},
+ * it means automatic message acknowledgment <i>before</i> listener execution, with
+ * no redelivery in case of an exception and no redelivery in case of other listener
+ * execution interruptions either. For {@link SimpleMessageListenerContainer},
+ * it means automatic message acknowledgment <i>after</i> listener execution, with
+ * no redelivery in case of a user exception thrown but potential redelivery in case
+ * of the JVM dying during listener execution. In order to consistently arrange for
+ * redelivery with any container variant, consider "CLIENT_ACKNOWLEDGE" mode or -
+ * preferably - setting "sessionTransacted" to "true" instead.
+ * <li>"sessionAcknowledgeMode" set to "DUPS_OK_ACKNOWLEDGE":
+ * <i>Lazy</i> message acknowledgment during ({@link DefaultMessageListenerContainer})
+ * or shortly after ({@link SimpleMessageListenerContainer}) listener execution;
+ * no redelivery in case of a user exception thrown but potential redelivery in case
+ * of the JVM dying during listener execution. In order to consistently arrange for
+ * redelivery with any container variant, consider "CLIENT_ACKNOWLEDGE" mode or -
+ * preferably - setting "sessionTransacted" to "true" instead.
  * <li>"sessionAcknowledgeMode" set to "CLIENT_ACKNOWLEDGE":
  * Automatic message acknowledgment <i>after</i> successful listener execution;
- * no redelivery in case of exception thrown.
- * <li>"sessionAcknowledgeMode" set to "DUPS_OK_ACKNOWLEDGE":
- * <i>Lazy</i> message acknowledgment during or after listener execution;
- * <i>potential redelivery</i> in case of exception thrown.
+ * best-effort redelivery in case of a user exception thrown as well as in case
+ * of other listener execution interruptions (such as the JVM dying).
  * <li>"sessionTransacted" set to "true":
  * Transactional acknowledgment after successful listener execution;
- * <i>guaranteed redelivery</i> in case of exception thrown.
+ * <i>guaranteed redelivery</i> in case of a user exception thrown as well as
+ * in case of other listener execution interruptions (such as the JVM dying).
  * </ul>
- * The exact behavior might vary according to the concrete listener container
- * and JMS provider used.
  *
- * <p>There are two solutions to the duplicate processing problem:
+ * <p>There are two solutions to the duplicate message processing problem:
  * <ul>
  * <li>Either add <i>duplicate message detection</i> to your listener, in the
  * form of a business entity existence check or a protocol table check. This
  * usually just needs to be done in case of the JMSRedelivered flag being
- * set on the incoming message (else just process straightforwardly).
- * <li>Or wrap the <i>entire processing with an XA transaction</i>, covering the
- * reception of the message as well as the execution of the message listener.
- * This is only supported by {@link DefaultMessageListenerContainer}, through
- * specifying a "transactionManager" (typically a
+ * set on the incoming message (otherwise just process straightforwardly).
+ * Note that with "sessionTransacted" set to "true", duplicate messages will
+ * only appear in case of the JVM dying at the most unfortunate point possible
+ * (i.e. after your business logic executed but before the JMS part got committed),
+ * so duplicate message detection is just there to cover a corner case.
+ * <li>Or wrap your <i>entire processing with an XA transaction</i>, covering the
+ * reception of the JMS message as well as the execution of the business logic in
+ * your message listener (including database operations etc). This is only
+ * supported by {@link DefaultMessageListenerContainer}, through specifying
+ * an external "transactionManager" (typically a
  * {@link org.springframework.transaction.jta.JtaTransactionManager}, with
- * a corresponding XA-aware JMS {@link javax.jms.ConnectionFactory} passed in as
- * "connectionFactory").
+ * a corresponding XA-aware JMS {@link javax.jms.ConnectionFactory} passed in
+ * as "connectionFactory").
  * </ul>
  * Note that XA transaction coordination adds significant runtime overhead,
  * so it might be feasible to avoid it unless absolutely necessary.
@@ -97,23 +119,20 @@ import org.springframework.util.ErrorHandler;
  * <li>Alternatively, specify a
  * {@link org.springframework.transaction.jta.JtaTransactionManager} as
  * "transactionManager" for a fully XA-aware JMS provider - typically when
- * running on a J2EE server, but also for other environments with a JTA
+ * running on a Java EE server, but also for other environments with a JTA
  * transaction manager present. This will give full "exactly-once" guarantees
  * without custom duplicate message checks, at the price of additional
  * runtime processing overhead.
  * </ul>
  *
- * <p>Note that it is also possible to specify a
- * {@link org.springframework.jms.connection.JmsTransactionManager} as external
- * "transactionManager", providing fully synchronized Spring transactions based
- * on local JMS transactions. The effect is similar to "sessionTransacted" set
- * to "true", the difference being that this external transaction management
- * will also affect independent JMS access code within the service layer
- * (e.g. based on {@link org.springframework.jms.core.JmsTemplate} or
- * {@link org.springframework.jms.connection.TransactionAwareConnectionFactoryProxy}),
- * not just direct JMS Session usage in a {@link SessionAwareMessageListener}.
+ * <p>Note that the "sessionTransacted" flag is strongly recommended over
+ * {@link org.springframework.jms.connection.JmsTransactionManager}, provided
+ * that transactions do not need to be managed externally. As a consequence,
+ * set the transaction manager only if you are using JTA or if you need to
+ * synchronize with custom external transaction arrangements.
  *
  * @author Juergen Hoeller
+ * @author Stephane Nicoll
  * @since 2.0
  * @see #setMessageListener
  * @see javax.jms.MessageListener
@@ -123,7 +142,17 @@ import org.springframework.util.ErrorHandler;
  * @see SimpleMessageListenerContainer
  * @see org.springframework.jms.listener.endpoint.JmsMessageEndpointManager
  */
-public abstract class AbstractMessageListenerContainer extends AbstractJmsListeningContainer {
+public abstract class AbstractMessageListenerContainer extends AbstractJmsListeningContainer
+		implements MessageListenerContainer {
+
+	/** The JMS 2.0 Session.createSharedConsumer method, if available */
+	private static final Method createSharedConsumerMethod = ClassUtils.getMethodIfAvailable(
+			Session.class, "createSharedConsumer", Topic.class, String.class, String.class);
+
+	/** The JMS 2.0 Session.createSharedDurableConsumer method, if available */
+	private static final Method createSharedDurableConsumerMethod = ClassUtils.getMethodIfAvailable(
+			Session.class, "createSharedDurableConsumer", Topic.class, String.class, String.class);
+
 
 	private volatile Object destination;
 
@@ -133,7 +162,15 @@ public abstract class AbstractMessageListenerContainer extends AbstractJmsListen
 
 	private boolean subscriptionDurable = false;
 
-	private String durableSubscriptionName;
+	private boolean subscriptionShared = false;
+
+	private String subscriptionName;
+
+	private Boolean replyPubSubDomain;
+
+	private boolean pubSubNoLocal = false;
+
+	private MessageConverter messageConverter;
 
 	private ExceptionListener exceptionListener;
 
@@ -144,6 +181,11 @@ public abstract class AbstractMessageListenerContainer extends AbstractJmsListen
 	private boolean acceptMessagesWhileStopping = false;
 
 
+	/**
+	 * Specify concurrency limits.
+	 */
+	public abstract void setConcurrency(String concurrency);
+
 	/**
 	 * Set the destination to receive messages from.
 	 * <p>Alternatively, specify a "destinationName", to be dynamically
@@ -244,8 +286,8 @@ public abstract class AbstractMessageListenerContainer extends AbstractJmsListen
 	public void setMessageListener(Object messageListener) {
 		checkMessageListener(messageListener);
 		this.messageListener = messageListener;
-		if (this.durableSubscriptionName == null) {
-			this.durableSubscriptionName = getDefaultSubscriptionName(messageListener);
+		if (this.subscriptionName == null) {
+			this.subscriptionName = getDefaultSubscriptionName(messageListener);
 		}
 	}
 
@@ -293,15 +335,20 @@ public abstract class AbstractMessageListenerContainer extends AbstractJmsListen
 
 	/**
 	 * Set whether to make the subscription durable. The durable subscription name
-	 * to be used can be specified through the "durableSubscriptionName" property.
+	 * to be used can be specified through the "subscriptionName" property.
 	 * <p>Default is "false". Set this to "true" to register a durable subscription,
-	 * typically in combination with a "durableSubscriptionName" value (unless
+	 * typically in combination with a "subscriptionName" value (unless
 	 * your message listener class name is good enough as subscription name).
-	 * <p>Only makes sense when listening to a topic (pub-sub domain).
-	 * @see #setDurableSubscriptionName
+	 * <p>Only makes sense when listening to a topic (pub-sub domain),
+	 * therefore this method switches the "pubSubDomain" flag as well.
+	 * @see #setSubscriptionName
+	 * @see #setPubSubDomain
 	 */
 	public void setSubscriptionDurable(boolean subscriptionDurable) {
 		this.subscriptionDurable = subscriptionDurable;
+		if (subscriptionDurable) {
+			setPubSubDomain(true);
+		}
 	}
 
 	/**
@@ -312,25 +359,148 @@ public abstract class AbstractMessageListenerContainer extends AbstractJmsListen
 	}
 
 	/**
-	 * Set the name of a durable subscription to create. To be applied in case
-	 * of a topic (pub-sub domain) with subscription durability activated.
+	 * Set whether to make the subscription shared. The shared subscription name
+	 * to be used can be specified through the "subscriptionName" property.
+	 * <p>Default is "false". Set this to "true" to register a shared subscription,
+	 * typically in combination with a "subscriptionName" value (unless
+	 * your message listener class name is good enough as subscription name).
+	 * Note that shared subscriptions may also be durable, so this flag can
+	 * (and often will) be combined with "subscriptionDurable" as well.
+	 * <p>Only makes sense when listening to a topic (pub-sub domain),
+	 * therefore this method switches the "pubSubDomain" flag as well.
+	 * <p><b>Requires a JMS 2.0 compatible message broker.</b>
+	 * @since 4.1
+	 * @see #setSubscriptionName
+	 * @see #setSubscriptionDurable
+	 * @see #setPubSubDomain
+	 */
+	public void setSubscriptionShared(boolean subscriptionShared) {
+		this.subscriptionShared = subscriptionShared;
+		if (subscriptionShared) {
+			setPubSubDomain(true);
+		}
+	}
+
+	/**
+	 * Return whether to make the subscription shared.
+	 * @since 4.1
+	 */
+	public boolean isSubscriptionShared() {
+		return this.subscriptionShared;
+	}
+
+	/**
+	 * Set the name of a subscription to create. To be applied in case
+	 * of a topic (pub-sub domain) with a shared or durable subscription.
+	 * <p>The subscription name needs to be unique within this client's
+	 * JMS client id. Default is the class name of the specified message listener.
+	 * <p>Note: Only 1 concurrent consumer (which is the default of this
+	 * message listener container) is allowed for each subscription,
+	 * except for a shared subscription (which requires JMS 2.0).
+	 * @since 4.1
+	 * @see #setPubSubDomain
+	 * @see #setSubscriptionDurable
+	 * @see #setSubscriptionShared
+	 * @see #setClientId
+	 * @see #setMessageListener
+	 */
+	public void setSubscriptionName(String subscriptionName) {
+		this.subscriptionName = subscriptionName;
+	}
+
+	/**
+	 * Return the name of a subscription to create, if any.
+	 * @since 4.1
+	 */
+	public String getSubscriptionName() {
+		return this.subscriptionName;
+	}
+
+	/**
+	 * Set the name of a durable subscription to create. This method switches
+	 * to pub-sub domain mode and activates subscription durability as well.
 	 * <p>The durable subscription name needs to be unique within this client's
 	 * JMS client id. Default is the class name of the specified message listener.
 	 * <p>Note: Only 1 concurrent consumer (which is the default of this
-	 * message listener container) is allowed for each durable subscription.
+	 * message listener container) is allowed for each durable subscription,
+	 * except for a shared durable subscription (which requires JMS 2.0).
+	 * @see #setPubSubDomain
 	 * @see #setSubscriptionDurable
+	 * @see #setSubscriptionShared
 	 * @see #setClientId
 	 * @see #setMessageListener
 	 */
 	public void setDurableSubscriptionName(String durableSubscriptionName) {
-		this.durableSubscriptionName = durableSubscriptionName;
+		this.subscriptionName = durableSubscriptionName;
+		this.subscriptionDurable = true;
 	}
 
 	/**
 	 * Return the name of a durable subscription to create, if any.
 	 */
 	public String getDurableSubscriptionName() {
-		return this.durableSubscriptionName;
+		return (this.subscriptionDurable ? this.subscriptionName : null);
+	}
+
+	/**
+	 * Set whether to inhibit the delivery of messages published by its own connection.
+	 * Default is "false".
+	 * @since 4.1
+	 * @see javax.jms.Session#createConsumer(javax.jms.Destination, String, boolean)
+	 */
+	public void setPubSubNoLocal(boolean pubSubNoLocal) {
+		this.pubSubNoLocal = pubSubNoLocal;
+	}
+
+	/**
+	 * Return whether to inhibit the delivery of messages published by its own connection.
+	 * @since 4.1
+	 */
+	public boolean isPubSubNoLocal() {
+		return this.pubSubNoLocal;
+	}
+
+	/**
+	 * Configure the reply destination type. By default, the configured {@code pubSubDomain}
+	 * value is used (see {@link #isPubSubDomain()}.
+	 * <p>This setting primarily indicates what type of destination to resolve if dynamic
+	 * destinations are enabled.
+	 * @param replyPubSubDomain "true" for the Publish/Subscribe domain ({@link Topic Topics}),
+	 * "false" for the Point-to-Point domain ({@link Queue Queues})
+	 * @since 4.2
+	 * @see #setDestinationResolver
+	 */
+	public void setReplyPubSubDomain(boolean replyPubSubDomain) {
+		this.replyPubSubDomain = replyPubSubDomain;
+	}
+
+	/**
+	 * Return whether the Publish/Subscribe domain ({@link javax.jms.Topic Topics}) is used
+	 * for replies. Otherwise, the Point-to-Point domain ({@link javax.jms.Queue Queues})
+	 * is used.
+	 * @since 4.2
+	 */
+	@Override
+	public boolean isReplyPubSubDomain() {
+		if (this.replyPubSubDomain != null) {
+			return replyPubSubDomain;
+		}
+		else {
+			return isPubSubDomain();
+		}
+	}
+
+	/**
+	 * Set the {@link MessageConverter} strategy for converting JMS Messages.
+	 * @since 4.1
+	 */
+	public void setMessageConverter(MessageConverter messageConverter) {
+		this.messageConverter = messageConverter;
+	}
+
+	@Override
+	public MessageConverter getMessageConverter() {
+		return this.messageConverter;
 	}
 
 	/**
@@ -350,14 +520,24 @@ public abstract class AbstractMessageListenerContainer extends AbstractJmsListen
 	}
 
 	/**
-	 * Set an ErrorHandler to be invoked in case of any uncaught exceptions thrown
-	 * while processing a Message. By default there will be <b>no</b> ErrorHandler
-	 * so that error-level logging is the only result.
+	 * Set the ErrorHandler to be invoked in case of any uncaught exceptions thrown
+	 * while processing a Message.
+	 * <p>By default, there will be <b>no</b> ErrorHandler so that error-level
+	 * logging is the only result.
 	 */
 	public void setErrorHandler(ErrorHandler errorHandler) {
 		this.errorHandler = errorHandler;
 	}
 
+	/**
+	 * Return the ErrorHandler to be invoked in case of any uncaught exceptions thrown
+	 * while processing a Message.
+	 * @since 4.1
+	 */
+	public ErrorHandler getErrorHandler() {
+		return this.errorHandler;
+	}
+
 	/**
 	 * Set whether to expose the listener JMS Session to a registered
 	 * {@link SessionAwareMessageListener} as well as to
@@ -415,9 +595,11 @@ public abstract class AbstractMessageListenerContainer extends AbstractJmsListen
 		if (this.destination == null) {
 			throw new IllegalArgumentException("Property 'destination' or 'destinationName' is required");
 		}
-		if (isSubscriptionDurable() && !isPubSubDomain()) {
-			throw new IllegalArgumentException("A durable subscription requires a topic (pub-sub domain)");
-		}
+	}
+
+	@Override
+	public void setupMessageListener(Object messageListener) {
+		setMessageListener(messageListener);
 	}
 
 
@@ -464,6 +646,7 @@ public abstract class AbstractMessageListenerContainer extends AbstractJmsListen
 			rollbackIfNecessary(session);
 			throw new MessageRejectedWhileStoppingException();
 		}
+
 		try {
 			invokeListener(session, message);
 		}
@@ -493,6 +676,7 @@ public abstract class AbstractMessageListenerContainer extends AbstractJmsListen
 	@SuppressWarnings("rawtypes")
 	protected void invokeListener(Session session, Message message) throws JMSException {
 		Object listener = getMessageListener();
+
 		if (listener instanceof SessionAwareMessageListener) {
 			doInvokeListener((SessionAwareMessageListener) listener, session, message);
 		}
@@ -594,7 +778,7 @@ public abstract class AbstractMessageListenerContainer extends AbstractJmsListen
 				JmsUtils.rollbackIfNecessary(session);
 			}
 		}
-		else {
+		else if (isClientAcknowledge(session)) {
 			session.recover();
 		}
 	}
@@ -616,7 +800,7 @@ public abstract class AbstractMessageListenerContainer extends AbstractJmsListen
 					JmsUtils.rollbackIfNecessary(session);
 				}
 			}
-			else {
+			else if (isClientAcknowledge(session)) {
 				session.recover();
 			}
 		}
@@ -653,6 +837,51 @@ public abstract class AbstractMessageListenerContainer extends AbstractJmsListen
 		return isSessionTransacted();
 	}
 
+	/**
+	 * Create a JMS MessageConsumer for the given Session and Destination.
+	 * <p>This implementation uses JMS 1.1 API.
+	 * @param session the JMS Session to create a MessageConsumer for
+	 * @param destination the JMS Destination to create a MessageConsumer for
+	 * @return the new JMS MessageConsumer
+	 * @throws javax.jms.JMSException if thrown by JMS API methods
+	 */
+	protected MessageConsumer createConsumer(Session session, Destination destination) throws JMSException {
+		if (isPubSubDomain() && destination instanceof Topic) {
+			if (isSubscriptionShared()) {
+				// createSharedConsumer((Topic) dest, subscription, selector);
+				// createSharedDurableConsumer((Topic) dest, subscription, selector);
+				Method method = (isSubscriptionDurable() ?
+						createSharedDurableConsumerMethod : createSharedConsumerMethod);
+				try {
+					return (MessageConsumer) method.invoke(session, destination, getSubscriptionName(), getMessageSelector());
+				}
+				catch (InvocationTargetException ex) {
+					if (ex.getTargetException() instanceof JMSException) {
+						throw (JMSException) ex.getTargetException();
+					}
+					ReflectionUtils.handleInvocationTargetException(ex);
+					return null;
+				}
+				catch (IllegalAccessException ex) {
+					throw new IllegalStateException("Could not access JMS 2.0 API method: " + ex.getMessage());
+				}
+			}
+			else if (isSubscriptionDurable()) {
+				return session.createDurableSubscriber(
+						(Topic) destination, getSubscriptionName(), getMessageSelector(), isPubSubNoLocal());
+			}
+			else {
+				// Only pass in the NoLocal flag in case of a Topic (pub-sub mode):
+				// Some JMS providers, such as WebSphere MQ 6.0, throw IllegalStateException
+				// in case of the NoLocal flag being specified for a Queue.
+				return session.createConsumer(destination, getMessageSelector(), isPubSubNoLocal());
+			}
+		}
+		else {
+			return session.createConsumer(destination, getMessageSelector());
+		}
+	}
+
 	/**
 	 * Handle the given exception that arose during listener execution.
 	 * <p>The default implementation logs the exception at warn level,
@@ -699,10 +928,11 @@ public abstract class AbstractMessageListenerContainer extends AbstractJmsListen
 	 * @see #setErrorHandler
 	 */
 	protected void invokeErrorHandler(Throwable ex) {
-		if (this.errorHandler != null) {
-			this.errorHandler.handleError(ex);
+		ErrorHandler errorHandler = getErrorHandler();
+		if (errorHandler != null) {
+			errorHandler.handleError(ex);
 		}
-		else if (logger.isWarnEnabled()) {
+		else {
 			logger.warn("Execution of JMS message listener failed, and no ErrorHandler has been set.", ex);
 		}
 	}
@@ -714,7 +944,6 @@ public abstract class AbstractMessageListenerContainer extends AbstractJmsListen
 	 */
 	@SuppressWarnings("serial")
 	private static class MessageRejectedWhileStoppingException extends RuntimeException {
-
 	}
 
 }
