@@ -1,5 +1,5 @@
 /*
- * Copyright 2002-2014 the original author or authors.
+ * Copyright 2002-2016 the original author or authors.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -57,26 +57,24 @@ import org.springframework.messaging.handler.annotation.Headers;
 import org.springframework.messaging.handler.annotation.Payload;
 import org.springframework.messaging.handler.annotation.SendTo;
 import org.springframework.messaging.handler.annotation.support.DefaultMessageHandlerMethodFactory;
-import org.springframework.messaging.handler.annotation.support.MethodArgumentTypeMismatchException;
 import org.springframework.util.ReflectionUtils;
 import org.springframework.validation.Errors;
 import org.springframework.validation.Validator;
 import org.springframework.validation.annotation.Validated;
 
-import static org.junit.Assert.*;
-import static org.mockito.BDDMockito.*;
+import static org.junit.Assert.assertEquals;
+import static org.junit.Assert.assertNotNull;
+import static org.junit.Assert.assertSame;
+import static org.junit.Assert.assertTrue;
+import static org.mockito.BDDMockito.given;
+import static org.mockito.BDDMockito.mock;
+import static org.mockito.BDDMockito.verify;
 
 /**
  * @author Stephane Nicoll
  */
 public class MethodJmsListenerEndpointTests {
 
-	@Rule
-	public final TestName name = new TestName();
-
-	@Rule
-	public final ExpectedException thrown = ExpectedException.none();
-
 	private final DefaultMessageHandlerMethodFactory factory = new DefaultMessageHandlerMethodFactory();
 
 	private final DefaultMessageListenerContainer container = new DefaultMessageListenerContainer();
@@ -84,11 +82,19 @@ public class MethodJmsListenerEndpointTests {
 	private final JmsEndpointSampleBean sample = new JmsEndpointSampleBean();
 
 
+	@Rule
+	public final TestName name = new TestName();
+
+	@Rule
+	public final ExpectedException thrown = ExpectedException.none();
+
+
 	@Before
 	public void setup() {
 		initializeFactory(factory);
 	}
 
+
 	@Test
 	public void createMessageListenerNoFactory() {
 		MethodJmsListenerEndpoint endpoint = new MethodJmsListenerEndpoint();
@@ -163,6 +169,17 @@ public class MethodJmsListenerEndpointTests {
 		assertDefaultListenerMethodInvocation();
 	}
 
+	@Test
+	public void resolveCustomHeaderNameAndPayloadWithHeaderNameSet() throws JMSException {
+		MessagingMessageListenerAdapter listener = createDefaultInstance(String.class, int.class);
+
+		Session session = mock(Session.class);
+		StubTextMessage message = createSimpleJmsTextMessage("my payload");
+		message.setIntProperty("myCounter", 24);
+		listener.onMessage(message, session);
+		assertDefaultListenerMethodInvocation();
+	}
+
 	@Test
 	public void resolveHeaders() throws JMSException {
 		MessagingMessageListenerAdapter listener = createDefaultInstance(String.class, Map.class);
@@ -253,7 +270,19 @@ public class MethodJmsListenerEndpointTests {
 		SimpleMessageListenerContainer container = new SimpleMessageListenerContainer();
 		MessagingMessageListenerAdapter listener = createInstance(this.factory,
 				getListenerMethod(methodName, String.class), container);
-		processAndReplyWithSendTo(listener, false);
+		processAndReplyWithSendTo(listener, "replyDestination", false);
+		assertListenerMethodInvocation(sample, methodName);
+	}
+
+	@Test
+	public void processFromTopicAndReplyWithSendToQueue() throws JMSException {
+		String methodName = "processAndReplyWithSendTo";
+		SimpleMessageListenerContainer container = new SimpleMessageListenerContainer();
+		container.setPubSubDomain(true);
+		container.setReplyPubSubDomain(false);
+		MessagingMessageListenerAdapter listener = createInstance(this.factory,
+				getListenerMethod(methodName, String.class), container);
+		processAndReplyWithSendTo(listener, "replyDestination", false);
 		assertListenerMethodInvocation(sample, methodName);
 	}
 
@@ -264,11 +293,30 @@ public class MethodJmsListenerEndpointTests {
 		container.setPubSubDomain(true);
 		MessagingMessageListenerAdapter listener = createInstance(this.factory,
 				getListenerMethod(methodName, String.class), container);
-		processAndReplyWithSendTo(listener, true);
+		processAndReplyWithSendTo(listener, "replyDestination", true);
 		assertListenerMethodInvocation(sample, methodName);
 	}
 
-	private void processAndReplyWithSendTo(MessagingMessageListenerAdapter listener, boolean pubSubDomain) throws JMSException {
+	@Test
+	public void processFromQueueAndReplyWithSendToTopic() throws JMSException {
+		String methodName = "processAndReplyWithSendTo";
+		SimpleMessageListenerContainer container = new SimpleMessageListenerContainer();
+		container.setReplyPubSubDomain(true);
+		MessagingMessageListenerAdapter listener = createInstance(this.factory,
+				getListenerMethod(methodName, String.class), container);
+		processAndReplyWithSendTo(listener, "replyDestination", true);
+		assertListenerMethodInvocation(sample, methodName);
+	}
+
+	@Test
+	public void processAndReplyWithDefaultSendTo() throws JMSException {
+		MessagingMessageListenerAdapter listener = createDefaultInstance(String.class);
+		processAndReplyWithSendTo(listener, "defaultReply", false);
+		assertDefaultListenerMethodInvocation();
+	}
+
+	private void processAndReplyWithSendTo(MessagingMessageListenerAdapter listener,
+			String replyDestinationName, boolean pubSubDomain) throws JMSException {
 		String body = "echo text";
 		String correlationId = "link-1234";
 		Destination replyDestination = new Destination() {};
@@ -278,7 +326,7 @@ public class MethodJmsListenerEndpointTests {
 		QueueSender queueSender = mock(QueueSender.class);
 		Session session = mock(Session.class);
 
-		given(destinationResolver.resolveDestinationName(session, "replyDestination", pubSubDomain))
+		given(destinationResolver.resolveDestinationName(session, replyDestinationName, pubSubDomain))
 				.willReturn(replyDestination);
 		given(session.createTextMessage(body)).willReturn(reply);
 		given(session.createProducer(replyDestination)).willReturn(queueSender);
@@ -288,7 +336,7 @@ public class MethodJmsListenerEndpointTests {
 		inputMessage.setJMSCorrelationID(correlationId);
 		listener.onMessage(inputMessage, session);
 
-		verify(destinationResolver).resolveDestinationName(session, "replyDestination", pubSubDomain);
+		verify(destinationResolver).resolveDestinationName(session, replyDestinationName, pubSubDomain);
 		verify(reply).setJMSCorrelationID(correlationId);
 		verify(queueSender).send(reply);
 		verify(queueSender).close();
@@ -363,12 +411,14 @@ public class MethodJmsListenerEndpointTests {
 		Session session = mock(Session.class);
 
 		thrown.expect(ListenerExecutionFailedException.class);
-		thrown.expectCause(Matchers.isA(MethodArgumentTypeMismatchException.class));
+		thrown.expectCause(Matchers.isA(MessageConversionException.class));
 		listener.onMessage(createSimpleJmsTextMessage("test"), session);  // Message<String> as Message<Integer>
 	}
 
+
 	private MessagingMessageListenerAdapter createInstance(
 			DefaultMessageHandlerMethodFactory factory, Method method, MessageListenerContainer container) {
+
 		MethodJmsListenerEndpoint endpoint = new MethodJmsListenerEndpoint();
 		endpoint.setBean(sample);
 		endpoint.setMethod(method);
@@ -376,8 +426,7 @@ public class MethodJmsListenerEndpointTests {
 		return endpoint.createMessageListener(container);
 	}
 
-	private MessagingMessageListenerAdapter createInstance(
-			DefaultMessageHandlerMethodFactory factory, Method method) {
+	private MessagingMessageListenerAdapter createInstance(DefaultMessageHandlerMethodFactory factory, Method method) {
 		return createInstance(factory, method, new SimpleMessageListenerContainer());
 	}
 
@@ -433,9 +482,10 @@ public class MethodJmsListenerEndpointTests {
 	}
 
 
+	@SendTo("defaultReply") @SuppressWarnings("unused")
 	static class JmsEndpointSampleBean {
 
-		private final Map<String, Boolean> invocations = new HashMap<String, Boolean>();
+		private final Map<String, Boolean> invocations = new HashMap<>();
 
 		public void resolveMessageAndSession(javax.jms.Message message, Session session) {
 			invocations.put("resolveMessageAndSession", true);
@@ -461,6 +511,12 @@ public class MethodJmsListenerEndpointTests {
 			assertEquals("Wrong @Header resolution", 24, counter);
 		}
 
+		public void resolveCustomHeaderNameAndPayloadWithHeaderNameSet(@Payload String content, @Header(name = "myCounter") int counter) {
+			invocations.put("resolveCustomHeaderNameAndPayloadWithHeaderNameSet", true);
+			assertEquals("Wrong @Payload resolution", "my payload", content);
+			assertEquals("Wrong @Header resolution", 24, counter);
+		}
+
 		public void resolveHeaders(String content, @Headers Map<String, Object> headers) {
 			invocations.put("resolveHeaders", true);
 			assertEquals("Wrong payload resolution", "my payload", content);
@@ -506,6 +562,11 @@ public class MethodJmsListenerEndpointTests {
 			return content;
 		}
 
+		public String processAndReplyWithDefaultSendTo(String content) {
+			invocations.put("processAndReplyWithDefaultSendTo", true);
+			return content;
+		}
+
 		@SendTo("")
 		public String emptySendTo(String content) {
 			invocations.put("emptySendTo", true);
@@ -535,8 +596,8 @@ public class MethodJmsListenerEndpointTests {
 
 	@SuppressWarnings("serial")
 	static class MyBean implements Serializable {
-		private String name;
 
+		private String name;
 	}
 
 }
