@@ -1,5 +1,5 @@
 /*
- * Copyright 2002-2014 the original author or authors.
+ * Copyright 2002-2016 the original author or authors.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -58,13 +58,14 @@ import static org.junit.Assert.*;
  */
 public class StompBrokerRelayMessageHandlerIntegrationTests {
 
+	private static final Charset UTF_8 = Charset.forName("UTF-8");
+
+
 	@Rule
 	public final TestName testName = new TestName();
 
 	private static final Log logger = LogFactory.getLog(StompBrokerRelayMessageHandlerIntegrationTests.class);
 
-	private static final Charset UTF_8 = Charset.forName("UTF-8");
-
 	private StompBrokerRelayMessageHandler relay;
 
 	private BrokerService activeMQBroker;
@@ -141,9 +142,9 @@ public class StompBrokerRelayMessageHandlerIntegrationTests {
 		logger.debug("Broker stopped");
 	}
 
+
 	@Test
 	public void publishSubscribe() throws Exception {
-
 		logger.debug("Starting test publishSubscribe()");
 
 		String sess1 = "sess1";
@@ -166,7 +167,7 @@ public class StompBrokerRelayMessageHandlerIntegrationTests {
 		this.responseHandler.expectMessages(send);
 	}
 
-	@Test(expected=MessageDeliveryException.class)
+	@Test(expected = MessageDeliveryException.class)
 	public void messageDeliveryExceptionIfSystemSessionForwardFails() throws Exception {
 
 		logger.debug("Starting test messageDeliveryExceptionIfSystemSessionForwardFails()");
@@ -180,7 +181,6 @@ public class StompBrokerRelayMessageHandlerIntegrationTests {
 
 	@Test
 	public void brokerBecomingUnvailableTriggersErrorFrame() throws Exception {
-
 		logger.debug("Starting test brokerBecomingUnvailableTriggersErrorFrame()");
 
 		String sess1 = "sess1";
@@ -196,7 +196,6 @@ public class StompBrokerRelayMessageHandlerIntegrationTests {
 
 	@Test
 	public void brokerAvailabilityEventWhenStopped() throws Exception {
-
 		logger.debug("Starting test brokerAvailabilityEventWhenStopped()");
 
 		stopActiveMqBrokerAndAwait();
@@ -205,7 +204,6 @@ public class StompBrokerRelayMessageHandlerIntegrationTests {
 
 	@Test
 	public void relayReconnectsIfBrokerComesBackUp() throws Exception {
-
 		logger.debug("Starting test relayReconnectsIfBrokerComesBackUp()");
 
 		String sess1 = "sess1";
@@ -229,28 +227,8 @@ public class StompBrokerRelayMessageHandlerIntegrationTests {
 		this.eventPublisher.expectBrokerAvailabilityEvent(true);
 	}
 
-	@Test
-	public void disconnectClosesRelaySessionCleanly() throws Exception {
-
-		logger.debug("Starting test disconnectClosesRelaySessionCleanly()");
-
-		MessageExchange connect = MessageExchangeBuilder.connect("sess1").build();
-		this.relay.handleMessage(connect.message);
-		this.responseHandler.expectMessages(connect);
-
-		StompHeaderAccessor headers = StompHeaderAccessor.create(StompCommand.DISCONNECT);
-		headers.setSessionId("sess1");
-		this.relay.handleMessage(MessageBuilder.createMessage(new byte[0], headers.getMessageHeaders()));
-
-		Thread.sleep(2000);
-
-		// Check that we have not received an ERROR as a result of the connection closing
-		assertTrue("Unexpected messages: " + this.responseHandler.queue, this.responseHandler.queue.isEmpty());
-	}
-
 	@Test
 	public void disconnectWithReceipt() throws Exception {
-
 		logger.debug("Starting test disconnectWithReceipt()");
 
 		MessageExchange connect = MessageExchangeBuilder.connect("sess1").build();
@@ -270,6 +248,11 @@ public class StompBrokerRelayMessageHandlerIntegrationTests {
 
 		@Override
 		public void publishEvent(ApplicationEvent event) {
+			publishEvent((Object) event);
+		}
+
+		@Override
+		public void publishEvent(Object event) {
 			logger.debug("Processing ApplicationEvent " + event);
 			if (event instanceof BrokerAvailabilityEvent) {
 				this.eventQueue.add((BrokerAvailabilityEvent) event);
@@ -283,6 +266,7 @@ public class StompBrokerRelayMessageHandlerIntegrationTests {
 		}
 	}
 
+
 	private static class TestMessageHandler implements MessageHandler {
 
 		private final BlockingQueue<Message<?>> queue = new LinkedBlockingQueue<>();
@@ -296,17 +280,13 @@ public class StompBrokerRelayMessageHandlerIntegrationTests {
 		}
 
 		public void expectMessages(MessageExchange... messageExchanges) throws InterruptedException {
-
 			List<MessageExchange> expectedMessages =
 					new ArrayList<MessageExchange>(Arrays.<MessageExchange>asList(messageExchanges));
-
 			while (expectedMessages.size() > 0) {
 				Message<?> message = this.queue.poll(10000, TimeUnit.MILLISECONDS);
 				assertNotNull("Timed out waiting for messages, expected [" + expectedMessages + "]", message);
-
 				MessageExchange match = findMatch(expectedMessages, message);
 				assertNotNull("Unexpected message=" + message + ", expected [" + expectedMessages + "]", match);
-
 				expectedMessages.remove(match);
 			}
 		}
@@ -321,6 +301,7 @@ public class StompBrokerRelayMessageHandlerIntegrationTests {
 		}
 	}
 
+
 	/**
 	 * Holds a message as well as expected and actual messages matched against expectations.
 	 */
@@ -339,7 +320,7 @@ public class StompBrokerRelayMessageHandlerIntegrationTests {
 		}
 
 		public boolean matchMessage(Message<?> message) {
-			for (int i=0 ; i < this.expected.length; i++) {
+			for (int i = 0 ; i < this.expected.length; i++) {
 				if (this.expected[i].match(message)) {
 					this.actual[i] = message;
 					return true;
@@ -356,6 +337,7 @@ public class StompBrokerRelayMessageHandlerIntegrationTests {
 		}
 	}
 
+
 	private static class MessageExchangeBuilder {
 
 		private final Message<?> message;
@@ -364,8 +346,7 @@ public class StompBrokerRelayMessageHandlerIntegrationTests {
 
 		private final List<MessageMatcher> expected = new ArrayList<>();
 
-
-		private MessageExchangeBuilder(Message<?> message) {
+		public MessageExchangeBuilder(Message<?> message) {
 			this.message = message;
 			this.headers = StompHeaderAccessor.wrap(message);
 		}
@@ -455,25 +436,24 @@ public class StompBrokerRelayMessageHandlerIntegrationTests {
 		}
 	}
 
-	private static interface MessageMatcher {
+
+	private interface MessageMatcher {
 
 		boolean match(Message<?> message);
-
 	}
 
+
 	private static class StompFrameMessageMatcher implements MessageMatcher {
 
 		private final StompCommand command;
 
 		private final String sessionId;
 
-
 		public StompFrameMessageMatcher(StompCommand command, String sessionId) {
 			this.command = command;
 			this.sessionId = sessionId;
 		}
 
-
 		@Override
 		public final boolean match(Message<?> message) {
 			StompHeaderAccessor headers = StompHeaderAccessor.wrap(message);
@@ -493,6 +473,7 @@ public class StompBrokerRelayMessageHandlerIntegrationTests {
 		}
 	}
 
+
 	private static class StompReceiptFrameMessageMatcher extends StompFrameMessageMatcher {
 
 		private final String receiptId;
@@ -513,6 +494,7 @@ public class StompBrokerRelayMessageHandlerIntegrationTests {
 		}
 	}
 
+
 	private static class StompMessageFrameMessageMatcher extends StompFrameMessageMatcher {
 
 		private final String subscriptionId;
@@ -521,7 +503,6 @@ public class StompBrokerRelayMessageHandlerIntegrationTests {
 
 		private final Object payload;
 
-
 		public StompMessageFrameMessageMatcher(String sessionId, String subscriptionId, String destination, Object payload) {
 			super(StompCommand.MESSAGE, sessionId);
 			this.subscriptionId = subscriptionId;
@@ -549,18 +530,17 @@ public class StompBrokerRelayMessageHandlerIntegrationTests {
 		}
 
 		protected String getPayloadAsText() {
-			return (this.payload instanceof byte[])
-					? new String((byte[]) this.payload, UTF_8) : payload.toString();
+			return (this.payload instanceof byte[]) ?
+					new String((byte[]) this.payload, UTF_8) : this.payload.toString();
 		}
 	}
 
-	private static class StompConnectedFrameMessageMatcher extends StompFrameMessageMatcher {
 
+	private static class StompConnectedFrameMessageMatcher extends StompFrameMessageMatcher {
 
 		public StompConnectedFrameMessageMatcher(String sessionId) {
 			super(StompCommand.CONNECTED, sessionId);
 		}
-
 	}
 
 }
