@@ -68,7 +68,9 @@ import static org.mockito.BDDMockito.*;
  * @author Rossen Stoyanchev
  * @author Phillip Webb
  * @since 05.03.2005
+ * @deprecated as of Spring 4.3, in favor of Hibernate 4.x/5.x
  */
+@Deprecated
 public class OpenSessionInViewTests {
 
 	private MockServletContext sc;
