@@ -25,26 +25,23 @@ import javax.xml.transform.stream.StreamSource;
 
 import org.junit.Ignore;
 import org.junit.Test;
-import org.xml.sax.InputSource;
-import org.xml.sax.XMLReader;
 
 import org.springframework.core.io.ClassPathResource;
 import org.springframework.oxm.AbstractUnmarshallerTests;
 import org.springframework.oxm.MarshallingException;
-import org.springframework.oxm.Unmarshaller;
 
-import static junit.framework.Assert.assertEquals;
+import org.xml.sax.InputSource;
+import org.xml.sax.XMLReader;
+
 import static org.hamcrest.CoreMatchers.*;
-import static org.junit.Assert.assertNotNull;
-import static org.junit.Assert.assertNull;
-import static org.junit.Assert.assertSame;
 import static org.junit.Assert.*;
 
 /**
  * @author Arjen Poutsma
  * @author Jakub Narloch
+ * @author Sam Brannen
  */
-public class CastorUnmarshallerTests extends AbstractUnmarshallerTests {
+public class CastorUnmarshallerTests extends AbstractUnmarshallerTests<CastorMarshaller> {
 
 	/**
 	 * Represents the xml with additional attribute that is not mapped in Castor config.
@@ -62,6 +59,15 @@ public class CastorUnmarshallerTests extends AbstractUnmarshallerTests {
 			"</tns:flight></tns:flights>";
 
 
+	@Override
+	protected CastorMarshaller createUnmarshaller() throws Exception {
+		CastorMarshaller marshaller = new CastorMarshaller();
+		ClassPathResource mappingLocation = new ClassPathResource("mapping.xml", CastorMarshaller.class);
+		marshaller.setMappingLocation(mappingLocation);
+		marshaller.afterPropertiesSet();
+		return marshaller;
+	}
+
 	@Override
 	protected void testFlights(Object o) {
 		Flights flights = (Flights) o;
@@ -77,15 +83,6 @@ public class CastorUnmarshallerTests extends AbstractUnmarshallerTests {
 		assertThat("Number is invalid", flight.getNumber(), equalTo(42L));
 	}
 
-	@Override
-	protected Unmarshaller createUnmarshaller() throws Exception {
-		CastorMarshaller marshaller = new CastorMarshaller();
-		ClassPathResource mappingLocation = new ClassPathResource("mapping.xml", CastorMarshaller.class);
-		marshaller.setMappingLocation(mappingLocation);
-		marshaller.afterPropertiesSet();
-		return marshaller;
-	}
-
 
 	@Test
 	public void unmarshalTargetClass() throws Exception {
@@ -98,7 +95,7 @@ public class CastorUnmarshallerTests extends AbstractUnmarshallerTests {
 	}
 
 	@Test
-	public void testSetBothTargetClassesAndMapping() throws IOException {
+	public void setBothTargetClassesAndMapping() throws IOException {
 		CastorMarshaller unmarshaller = new CastorMarshaller();
 		unmarshaller.setMappingLocation(new ClassPathResource("order-mapping.xml", CastorMarshaller.class));
 		unmarshaller.setTargetClasses(new Class[] {Order.class});
@@ -120,62 +117,62 @@ public class CastorUnmarshallerTests extends AbstractUnmarshallerTests {
 	}
 
 	@Test
-	public void testWhitespacePreserveTrue() throws Exception {
-		getCastorUnmarshaller().setWhitespacePreserve(true);
+	public void whitespacePreserveTrue() throws Exception {
+		unmarshaller.setWhitespacePreserve(true);
 		Object result = unmarshalFlights();
 		testFlights(result);
 	}
 
 	@Test
-	public void testWhitespacePreserveFalse() throws Exception {
-		getCastorUnmarshaller().setWhitespacePreserve(false);
+	public void whitespacePreserveFalse() throws Exception {
+		unmarshaller.setWhitespacePreserve(false);
 		Object result = unmarshalFlights();
 		testFlights(result);
 	}
 
 	@Test
-	public void testIgnoreExtraAttributesTrue() throws Exception {
-		getCastorUnmarshaller().setIgnoreExtraAttributes(true);
+	public void ignoreExtraAttributesTrue() throws Exception {
+		unmarshaller.setIgnoreExtraAttributes(true);
 		Object result = unmarshal(EXTRA_ATTRIBUTES_STRING);
 		testFlights(result);
 	}
 
 	@Test(expected = MarshallingException.class)
-	public void testIgnoreExtraAttributesFalse() throws Exception {
-		getCastorUnmarshaller().setIgnoreExtraAttributes(false);
+	public void ignoreExtraAttributesFalse() throws Exception {
+		unmarshaller.setIgnoreExtraAttributes(false);
 		unmarshal(EXTRA_ATTRIBUTES_STRING);
 	}
 
 	@Test
 	@Ignore("Not working yet")
-	public void testIgnoreExtraElementsTrue() throws Exception {
-		getCastorUnmarshaller().setIgnoreExtraElements(true);
-		getCastorUnmarshaller().setValidating(false);
+	public void ignoreExtraElementsTrue() throws Exception {
+		unmarshaller.setIgnoreExtraElements(true);
+		unmarshaller.setValidating(false);
 		Object result = unmarshal(EXTRA_ELEMENTS_STRING);
 		testFlights(result);
 	}
 
 	@Test(expected = MarshallingException.class)
-	public void testIgnoreExtraElementsFalse() throws Exception {
-		getCastorUnmarshaller().setIgnoreExtraElements(false);
+	public void ignoreExtraElementsFalse() throws Exception {
+		unmarshaller.setIgnoreExtraElements(false);
 		unmarshal(EXTRA_ELEMENTS_STRING);
 	}
 
 	@Test
-	public void testRootObject() throws Exception {
+	public void rootObject() throws Exception {
 		Flights flights = new Flights();
-		getCastorUnmarshaller().setRootObject(flights);
+		unmarshaller.setRootObject(flights);
 		Object result = unmarshalFlights();
 		testFlights(result);
 		assertSame("Result Flights is different object.", flights, result);
 	}
 
 	@Test
-	public void testClearCollectionsTrue() throws Exception {
+	public void clearCollectionsTrue() throws Exception {
 		Flights flights = new Flights();
 		flights.setFlight(new Flight[]{new Flight()});
-		getCastorUnmarshaller().setRootObject(flights);
-		getCastorUnmarshaller().setClearCollections(true);
+		unmarshaller.setRootObject(flights);
+		unmarshaller.setClearCollections(true);
 		Object result = unmarshalFlights();
 
 		assertSame("Result Flights is different object.", flights, result);
@@ -185,11 +182,11 @@ public class CastorUnmarshallerTests extends AbstractUnmarshallerTests {
 
 	@Test
 	@Ignore("Fails on the build server for some reason")
-	public void testClearCollectionsFalse() throws Exception {
+	public void clearCollectionsFalse() throws Exception {
 		Flights flights = new Flights();
 		flights.setFlight(new Flight[] {new Flight(), null});
-		getCastorUnmarshaller().setRootObject(flights);
-		getCastorUnmarshaller().setClearCollections(false);
+		unmarshaller.setRootObject(flights);
+		unmarshaller.setClearCollections(false);
 		Object result = unmarshalFlights();
 
 		assertSame("Result Flights is different object.", flights, result);
@@ -253,11 +250,6 @@ public class CastorUnmarshallerTests extends AbstractUnmarshallerTests {
 		assertEquals(true, result.get().getFeature("http://xml.org/sax/features/external-general-entities"));
 	}
 
-
-	private CastorMarshaller getCastorUnmarshaller() {
-		return (CastorMarshaller) unmarshaller;
-	}
-
 	private Object unmarshalFlights() throws Exception {
 		return unmarshal(INPUT_STRING);
 	}
