@@ -1,5 +1,5 @@
 /*
- * Copyright 2002-2014 the original author or authors.
+ * Copyright 2002-2015 the original author or authors.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -21,12 +21,15 @@ import java.nio.charset.Charset;
 import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.Collections;
+import java.util.Date;
 import java.util.Enumeration;
 import java.util.HashMap;
 import java.util.List;
 import java.util.Locale;
 import java.util.Map;
 
+import javax.servlet.http.Cookie;
+
 import org.junit.Test;
 
 import org.springframework.util.StreamUtils;
@@ -40,11 +43,17 @@ import static org.junit.Assert.*;
  * @author Mark Fisher
  * @author Rossen Stoyanchev
  * @author Sam Brannen
+ * @author Brian Clozel
+ * @author Jakub Narloch
  */
 public class MockHttpServletRequestTests {
 
 	private static final String HOST = "Host";
 
+	private static final String CONTENT_TYPE = "Content-Type";
+
+	private static final String IF_MODIFIED_SINCE = "If-Modified-Since";
+
 	private MockHttpServletRequest request = new MockHttpServletRequest();
 
 
@@ -69,7 +78,7 @@ public class MockHttpServletRequestTests {
 		String contentType = "test/plain";
 		request.setContentType(contentType);
 		assertEquals(contentType, request.getContentType());
-		assertEquals(contentType, request.getHeader("Content-Type"));
+		assertEquals(contentType, request.getHeader(CONTENT_TYPE));
 		assertNull(request.getCharacterEncoding());
 	}
 
@@ -78,7 +87,7 @@ public class MockHttpServletRequestTests {
 		String contentType = "test/plain;charset=UTF-8";
 		request.setContentType(contentType);
 		assertEquals(contentType, request.getContentType());
-		assertEquals(contentType, request.getHeader("Content-Type"));
+		assertEquals(contentType, request.getHeader(CONTENT_TYPE));
 		assertEquals("UTF-8", request.getCharacterEncoding());
 	}
 
@@ -87,7 +96,7 @@ public class MockHttpServletRequestTests {
 		String contentType = "test/plain";
 		request.addHeader("Content-Type", contentType);
 		assertEquals(contentType, request.getContentType());
-		assertEquals(contentType, request.getHeader("Content-Type"));
+		assertEquals(contentType, request.getHeader(CONTENT_TYPE));
 		assertNull(request.getCharacterEncoding());
 	}
 
@@ -96,7 +105,18 @@ public class MockHttpServletRequestTests {
 		String contentType = "test/plain;charset=UTF-8";
 		request.addHeader("Content-Type", contentType);
 		assertEquals(contentType, request.getContentType());
-		assertEquals(contentType, request.getHeader("Content-Type"));
+		assertEquals(contentType, request.getHeader(CONTENT_TYPE));
+		assertEquals("UTF-8", request.getCharacterEncoding());
+	}
+
+	// SPR-12677
+
+	@Test
+	public void setContentTypeHeaderWithMoreComplexCharsetSyntax() {
+		String contentType = "test/plain;charset=\"utf-8\";foo=\"charset=bar\";foocharset=bar;foo=bar";
+		request.addHeader("Content-Type", contentType);
+		assertEquals(contentType, request.getContentType());
+		assertEquals(contentType, request.getHeader(CONTENT_TYPE));
 		assertEquals("UTF-8", request.getCharacterEncoding());
 	}
 
@@ -105,7 +125,7 @@ public class MockHttpServletRequestTests {
 		request.setContentType("test/plain");
 		request.setCharacterEncoding("UTF-8");
 		assertEquals("test/plain", request.getContentType());
-		assertEquals("test/plain;charset=UTF-8", request.getHeader("Content-Type"));
+		assertEquals("test/plain;charset=UTF-8", request.getHeader(CONTENT_TYPE));
 		assertEquals("UTF-8", request.getCharacterEncoding());
 	}
 
@@ -114,7 +134,7 @@ public class MockHttpServletRequestTests {
 		request.setCharacterEncoding("UTF-8");
 		request.setContentType("test/plain");
 		assertEquals("test/plain", request.getContentType());
-		assertEquals("test/plain;charset=UTF-8", request.getHeader("Content-Type"));
+		assertEquals("test/plain;charset=UTF-8", request.getHeader(CONTENT_TYPE));
 		assertEquals("UTF-8", request.getCharacterEncoding());
 	}
 
@@ -182,6 +202,26 @@ public class MockHttpServletRequestTests {
 		assertEquals(0, request.getParameterMap().size());
 	}
 
+	@Test
+	public void cookies() {
+		Cookie cookie1 = new Cookie("foo", "bar");
+		Cookie cookie2 = new Cookie("baz", "qux");
+		request.setCookies(cookie1, cookie2);
+
+		Cookie[] cookies = request.getCookies();
+
+		assertEquals(2, cookies.length);
+		assertEquals("foo", cookies[0].getName());
+		assertEquals("bar", cookies[0].getValue());
+		assertEquals("baz", cookies[1].getName());
+		assertEquals("qux", cookies[1].getValue());
+	}
+
+	@Test
+	public void noCookies() {
+		assertNull(request.getCookies());
+	}
+
 	@Test
 	public void defaultLocale() {
 		Locale originalDefaultLocale = Locale.getDefault();
@@ -367,6 +407,44 @@ public class MockHttpServletRequestTests {
 		assertTrue(request.isSecure());
 	}
 
+	@Test
+	public void httpHeaderDate() throws Exception {
+		Date date = new Date();
+		request.addHeader(IF_MODIFIED_SINCE, date);
+		assertEquals(date.getTime(), request.getDateHeader(IF_MODIFIED_SINCE));
+	}
+
+	@Test
+	public void httpHeaderTimestamp() throws Exception {
+		long timestamp = new Date().getTime();
+		request.addHeader(IF_MODIFIED_SINCE, timestamp);
+		assertEquals(timestamp, request.getDateHeader(IF_MODIFIED_SINCE));
+	}
+
+	@Test
+	public void httpHeaderRfcFormatedDate() throws Exception {
+		request.addHeader(IF_MODIFIED_SINCE, "Tue, 21 Jul 2015 10:00:00 GMT");
+		assertEquals(1437472800000L, request.getDateHeader(IF_MODIFIED_SINCE));
+	}
+
+	@Test
+	public void httpHeaderFirstVariantFormatedDate() throws Exception {
+		request.addHeader(IF_MODIFIED_SINCE, "Tue, 21-Jul-15 10:00:00 GMT");
+		assertEquals(1437472800000L, request.getDateHeader(IF_MODIFIED_SINCE));
+	}
+
+	@Test
+	public void httpHeaderSecondVariantFormatedDate() throws Exception {
+		request.addHeader(IF_MODIFIED_SINCE, "Tue Jul 21 10:00:00 2015");
+		assertEquals(1437472800000L, request.getDateHeader(IF_MODIFIED_SINCE));
+	}
+
+	@Test(expected = IllegalArgumentException.class)
+	public void httpHeaderFormatedDateError() throws Exception {
+		request.addHeader(IF_MODIFIED_SINCE, "This is not a date");
+		request.getDateHeader(IF_MODIFIED_SINCE);
+	}
+
 	private void assertEqualEnumerations(Enumeration<?> enum1, Enumeration<?> enum2) {
 		assertNotNull(enum1);
 		assertNotNull(enum2);
