@@ -35,6 +35,7 @@ import static org.junit.Assert.*;
  * @author Rossen Stoyanchev
  * @author Rob Winch
  * @author Sam Brannen
+ * @author Brian Clozel
  * @since 19.02.2006
  */
 public class MockHttpServletResponseTests {
@@ -90,6 +91,21 @@ public class MockHttpServletResponseTests {
 		assertEquals("UTF-8", response.getCharacterEncoding());
 	}
 
+	@Test  // SPR-12677
+	public void contentTypeHeaderWithMoreComplexCharsetSyntax() {
+		String contentType = "test/plain;charset=\"utf-8\";foo=\"charset=bar\";foocharset=bar;foo=bar";
+		response.setHeader("Content-Type", contentType);
+		assertEquals(contentType, response.getContentType());
+		assertEquals(contentType, response.getHeader("Content-Type"));
+		assertEquals("UTF-8", response.getCharacterEncoding());
+
+		response = new MockHttpServletResponse();
+		response.addHeader("Content-Type", contentType);
+		assertEquals(contentType, response.getContentType());
+		assertEquals(contentType, response.getHeader("Content-Type"));
+		assertEquals("UTF-8", response.getCharacterEncoding());
+	}
+
 	@Test
 	public void setContentTypeThenCharacterEncoding() {
 		response.setContentType("test/plain");
@@ -228,6 +244,35 @@ public class MockHttpServletResponseTests {
 		assertEquals(redirectUrl, response.getRedirectedUrl());
 	}
 
+	@Test
+	public void setDateHeader() {
+		response.setDateHeader("Last-Modified", 1437472800000L);
+		assertEquals("Tue, 21 Jul 2015 10:00:00 GMT", response.getHeader("Last-Modified"));
+	}
+
+	@Test
+	public void addDateHeader() {
+		response.addDateHeader("Last-Modified", 1437472800000L);
+		response.addDateHeader("Last-Modified", 1437472801000L);
+		assertEquals("Tue, 21 Jul 2015 10:00:00 GMT", response.getHeaders("Last-Modified").get(0));
+		assertEquals("Tue, 21 Jul 2015 10:00:01 GMT", response.getHeaders("Last-Modified").get(1));
+	}
+
+	@Test
+	public void getDateHeader() {
+		long time = 1437472800000L;
+		response.setDateHeader("Last-Modified", time);
+		assertEquals("Tue, 21 Jul 2015 10:00:00 GMT", response.getHeader("Last-Modified"));
+		assertEquals(time, response.getDateHeader("Last-Modified"));
+	}
+
+	@Test(expected = IllegalArgumentException.class)
+	public void getInvalidDateHeader() {
+		response.setHeader("Last-Modified", "invalid");
+		assertEquals("invalid", response.getHeader("Last-Modified"));
+		response.getDateHeader("Last-Modified");
+	}
+
 	@Test  // SPR-10414
 	public void modifyStatusAfterSendError() throws IOException {
 		response.sendError(HttpServletResponse.SC_NOT_FOUND);
