@@ -1,5 +1,5 @@
 /*
- * Copyright 2002-2014 the original author or authors.
+ * Copyright 2002-2016 the original author or authors.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -57,7 +57,7 @@ public class ServletTestExecutionListenerTestNGIntegrationTests extends Abstract
 	 * @see #ensureMocksAreReinjectedBetweenTests_2
 	 */
 	@Test
-	public void ensureMocksAreReinjectedBetweenTests_1() {
+	void ensureMocksAreReinjectedBetweenTests_1() {
 		assertInjectedServletRequestEqualsRequestInRequestContextHolder();
 	}
 
@@ -67,7 +67,7 @@ public class ServletTestExecutionListenerTestNGIntegrationTests extends Abstract
 	 * @see #ensureMocksAreReinjectedBetweenTests_1
 	 */
 	@Test
-	public void ensureMocksAreReinjectedBetweenTests_2() {
+	void ensureMocksAreReinjectedBetweenTests_2() {
 		assertInjectedServletRequestEqualsRequestInRequestContextHolder();
 	}
 
