@@ -1,5 +1,5 @@
 /*
- * Copyright 2002-2013 the original author or authors.
+ * Copyright 2002-2015 the original author or authors.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -18,7 +18,8 @@ package org.springframework.transaction;
 
 import java.lang.reflect.Method;
 
-import junit.framework.TestCase;
+import org.junit.Before;
+import org.junit.Test;
 
 import org.springframework.aop.support.AopUtils;
 import org.springframework.context.ApplicationContext;
@@ -29,11 +30,13 @@ import org.springframework.transaction.interceptor.TransactionAttribute;
 import org.springframework.transaction.interceptor.TransactionAttributeSource;
 import org.springframework.transaction.interceptor.TransactionInterceptor;
 
+import static org.junit.Assert.*;
+
 /**
  * @author Rob Harrop
  * @author Adrian Colyer
  */
-public class TxNamespaceHandlerTests extends TestCase {
+public class TxNamespaceHandlerTests {
 
 	private ApplicationContext context;
 
@@ -41,19 +44,22 @@ public class TxNamespaceHandlerTests extends TestCase {
 
 	private Method setAgeMethod;
 
-	@Override
+
+	@Before
 	public void setUp() throws Exception {
 		this.context = new ClassPathXmlApplicationContext("txNamespaceHandlerTests.xml", getClass());
 		this.getAgeMethod = ITestBean.class.getMethod("getAge", new Class[0]);
 		this.setAgeMethod = ITestBean.class.getMethod("setAge", new Class[] {int.class});
 	}
 
-	public void testIsProxy() throws Exception {
+	@Test
+	public void isProxy() throws Exception {
 		ITestBean bean = getTestBean();
 		assertTrue("testBean is not a proxy", AopUtils.isAopProxy(bean));
 	}
 
-	public void testInvokeTransactional() throws Exception {
+	@Test
+	public void invokeTransactional() throws Exception {
 		ITestBean testBean = getTestBean();
 		CallCountingTransactionManager ptm = (CallCountingTransactionManager) context.getBean("transactionManager");
 
@@ -79,7 +85,8 @@ public class TxNamespaceHandlerTests extends TestCase {
 		}
 	}
 
-	public void testRollbackRules() {
+	@Test
+	public void rollbackRules() {
 		TransactionInterceptor txInterceptor = (TransactionInterceptor) context.getBean("txRollbackAdvice");
 		TransactionAttributeSource txAttrSource = txInterceptor.getTransactionAttributeSource();
 		TransactionAttribute txAttr = txAttrSource.getTransactionAttribute(getAgeMethod,ITestBean.class);
