@@ -1,5 +1,5 @@
 /*
- * Copyright 2002-2014 the original author or authors.
+ * Copyright 2002-2016 the original author or authors.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -39,6 +39,7 @@ import org.springframework.util.StringUtils;
  * Extension of {@link UriComponents} for hierarchical URIs.
  *
  * @author Arjen Poutsma
+ * @author Rossen Stoyanchev
  * @author Phillip Webb
  * @since 3.1.3
  * @see <a href="http://tools.ietf.org/html/rfc3986#section-1.2.3">Hierarchical URIs</a>
@@ -52,7 +53,7 @@ final class HierarchicalUriComponents extends UriComponents {
 
 	private final String host;
 
-	private final int port;
+	private final String port;
 
 	private final PathComponent path;
 
@@ -73,8 +74,9 @@ final class HierarchicalUriComponents extends UriComponents {
 	 * @param encoded whether the components are already encoded
 	 * @param verify whether the components need to be checked for illegal characters
 	 */
-	HierarchicalUriComponents(String scheme, String userInfo, String host, int port, PathComponent path,
-			MultiValueMap<String, String> queryParams, String fragment, boolean encoded, boolean verify) {
+	HierarchicalUriComponents(String scheme, String userInfo, String host, String port,
+			PathComponent path, MultiValueMap<String, String> queryParams,
+			String fragment, boolean encoded, boolean verify) {
 
 		super(scheme, fragment);
 		this.userInfo = userInfo;
@@ -109,7 +111,14 @@ final class HierarchicalUriComponents extends UriComponents {
 
 	@Override
 	public int getPort() {
-		return this.port;
+		if (this.port == null) {
+			return -1;
+		}
+		else if (this.port.contains("{")) {
+			throw new IllegalStateException(
+					"The port contains a URI variable but has not been expanded yet: " + this.port);
+		}
+		return Integer.parseInt(this.port);
 	}
 
 	@Override
@@ -168,41 +177,44 @@ final class HierarchicalUriComponents extends UriComponents {
 	// encoding
 
 	/**
-	 * Encodes all URI components using their specific encoding rules, and returns the result as a new
-	 * {@code UriComponents} instance.
+	 * Encode all URI components using their specific encoding rules and return
+	 * the result as a new {@code UriComponents} instance.
 	 * @param encoding the encoding of the values contained in this map
 	 * @return the encoded uri components
 	 * @throws UnsupportedEncodingException if the given encoding is not supported
 	 */
 	@Override
 	public HierarchicalUriComponents encode(String encoding) throws UnsupportedEncodingException {
-		Assert.hasLength(encoding, "Encoding must not be empty");
 		if (this.encoded) {
 			return this;
 		}
-		String encodedScheme = encodeUriComponent(getScheme(), encoding, Type.SCHEME);
-		String encodedUserInfo = encodeUriComponent(this.userInfo, encoding, Type.USER_INFO);
-		String encodedHost = encodeUriComponent(this.host, encoding, Type.HOST);
+		Assert.hasLength(encoding, "Encoding must not be empty");
+		String schemeTo = encodeUriComponent(getScheme(), encoding, Type.SCHEME);
+		String userInfoTo = encodeUriComponent(this.userInfo, encoding, Type.USER_INFO);
+		String hostTo = encodeUriComponent(this.host, encoding, getHostType());
+		PathComponent pathTo = this.path.encode(encoding);
+		MultiValueMap<String, String> paramsTo = encodeQueryParams(encoding);
+		String fragmentTo = encodeUriComponent(getFragment(), encoding, Type.FRAGMENT);
+		return new HierarchicalUriComponents(schemeTo, userInfoTo, hostTo, this.port,
+				pathTo, paramsTo, fragmentTo, true, false);
+	}
 
-		PathComponent encodedPath = this.path.encode(encoding);
-		MultiValueMap<String, String> encodedQueryParams =
-				new LinkedMultiValueMap<String, String>(this.queryParams.size());
+	private MultiValueMap<String, String> encodeQueryParams(String encoding) throws UnsupportedEncodingException {
+		int size = this.queryParams.size();
+		MultiValueMap<String, String> result = new LinkedMultiValueMap<String, String>(size);
 		for (Map.Entry<String, List<String>> entry : this.queryParams.entrySet()) {
-			String encodedName = encodeUriComponent(entry.getKey(), encoding, Type.QUERY_PARAM);
-			List<String> encodedValues = new ArrayList<String>(entry.getValue().size());
+			String name = encodeUriComponent(entry.getKey(), encoding, Type.QUERY_PARAM);
+			List<String> values = new ArrayList<String>(entry.getValue().size());
 			for (String value : entry.getValue()) {
-				String encodedValue = encodeUriComponent(value, encoding, Type.QUERY_PARAM);
-				encodedValues.add(encodedValue);
+				values.add(encodeUriComponent(value, encoding, Type.QUERY_PARAM));
 			}
-			encodedQueryParams.put(encodedName, encodedValues);
+			result.put(name, values);
 		}
-		String encodedFragment = encodeUriComponent(this.getFragment(), encoding, Type.FRAGMENT);
-		return new HierarchicalUriComponents(encodedScheme, encodedUserInfo, encodedHost, this.port, encodedPath,
-				encodedQueryParams, encodedFragment, true, false);
+		return result;
 	}
 
 	/**
-	 * Encodes the given source into an encoded String using the rules specified
+	 * Encode the given source into an encoded String using the rules specified
 	 * by the given component and with the given options.
 	 * @param source the source string
 	 * @param encoding the encoding of the source string
@@ -210,7 +222,9 @@ final class HierarchicalUriComponents extends UriComponents {
 	 * @return the encoded URI
 	 * @throws IllegalArgumentException when the given uri parameter is not a valid URI
 	 */
-	static String encodeUriComponent(String source, String encoding, Type type) throws UnsupportedEncodingException {
+	static String encodeUriComponent(String source, String encoding, Type type)
+			throws UnsupportedEncodingException {
+
 		if (source == null) {
 			return null;
 		}
@@ -241,6 +255,10 @@ final class HierarchicalUriComponents extends UriComponents {
 		return bos.toByteArray();
 	}
 
+	private Type getHostType() {
+		return (this.host != null && this.host.startsWith("[")) ? Type.HOST_IPV6 : Type.HOST_IPV4;
+	}
+
 
 	// verifying
 
@@ -255,7 +273,7 @@ final class HierarchicalUriComponents extends UriComponents {
 		}
 		verifyUriComponent(getScheme(), Type.SCHEME);
 		verifyUriComponent(this.userInfo, Type.USER_INFO);
-		verifyUriComponent(this.host, Type.HOST);
+		verifyUriComponent(this.host, getHostType());
 		this.path.verify();
 		for (Map.Entry<String, List<String>> entry : queryParams.entrySet()) {
 			verifyUriComponent(entry.getKey(), Type.QUERY_PARAM);
@@ -271,7 +289,7 @@ final class HierarchicalUriComponents extends UriComponents {
 			return;
 		}
 		int length = source.length();
-		for (int i=0; i < length; i++) {
+		for (int i = 0; i < length; i++) {
 			char ch = source.charAt(i);
 			if (ch == '%') {
 				if ((i + 2) < length) {
@@ -280,17 +298,19 @@ final class HierarchicalUriComponents extends UriComponents {
 					int u = Character.digit(hex1, 16);
 					int l = Character.digit(hex2, 16);
 					if (u == -1 || l == -1) {
-						throw new IllegalArgumentException("Invalid encoded sequence \"" + source.substring(i) + "\"");
+						throw new IllegalArgumentException("Invalid encoded sequence \"" +
+								source.substring(i) + "\"");
 					}
 					i += 2;
 				}
 				else {
-					throw new IllegalArgumentException("Invalid encoded sequence \"" + source.substring(i) + "\"");
+					throw new IllegalArgumentException("Invalid encoded sequence \"" +
+							source.substring(i) + "\"");
 				}
 			}
 			else if (!type.isAllowed(ch)) {
-				throw new IllegalArgumentException(
-						"Invalid character '" + ch + "' for " + type.name() + " in \"" + source + "\"");
+				throw new IllegalArgumentException("Invalid character '" + ch + "' for " +
+						type.name() + " in \"" + source + "\"");
 			}
 		}
 	}
@@ -301,24 +321,32 @@ final class HierarchicalUriComponents extends UriComponents {
 	@Override
 	protected HierarchicalUriComponents expandInternal(UriTemplateVariables uriVariables) {
 		Assert.state(!this.encoded, "Cannot expand an already encoded UriComponents object");
-		String expandedScheme = expandUriComponent(getScheme(), uriVariables);
-		String expandedUserInfo = expandUriComponent(this.userInfo, uriVariables);
-		String expandedHost = expandUriComponent(this.host, uriVariables);
-		PathComponent expandedPath = this.path.expand(uriVariables);
-		MultiValueMap<String, String> expandedQueryParams =
-				new LinkedMultiValueMap<String, String>(this.queryParams.size());
+
+		String schemeTo = expandUriComponent(getScheme(), uriVariables);
+		String userInfoTo = expandUriComponent(this.userInfo, uriVariables);
+		String hostTo = expandUriComponent(this.host, uriVariables);
+		String portTo = expandUriComponent(this.port, uriVariables);
+		PathComponent pathTo = this.path.expand(uriVariables);
+		MultiValueMap<String, String> paramsTo = expandQueryParams(uriVariables);
+		String fragmentTo = expandUriComponent(getFragment(), uriVariables);
+
+		return new HierarchicalUriComponents(schemeTo, userInfoTo, hostTo, portTo,
+				pathTo, paramsTo, fragmentTo, false, false);
+	}
+
+	private MultiValueMap<String, String> expandQueryParams(UriTemplateVariables variables) {
+		int size = this.queryParams.size();
+		MultiValueMap<String, String> result = new LinkedMultiValueMap<String, String>(size);
+		variables = new QueryUriTemplateVariables(variables);
 		for (Map.Entry<String, List<String>> entry : this.queryParams.entrySet()) {
-			String expandedName = expandUriComponent(entry.getKey(), uriVariables);
-			List<String> expandedValues = new ArrayList<String>(entry.getValue().size());
+			String name = expandUriComponent(entry.getKey(), variables);
+			List<String> values = new ArrayList<String>(entry.getValue().size());
 			for (String value : entry.getValue()) {
-				String expandedValue = expandUriComponent(value, uriVariables);
-				expandedValues.add(expandedValue);
+				values.add(expandUriComponent(value, variables));
 			}
-			expandedQueryParams.put(expandedName, expandedValues);
+			result.put(name, values);
 		}
-		String expandedFragment = expandUriComponent(this.getFragment(), uriVariables);
-		return new HierarchicalUriComponents(expandedScheme, expandedUserInfo, expandedHost, this.port, expandedPath,
-				expandedQueryParams, expandedFragment, false, false);
+		return result;
 	}
 
 	/**
@@ -355,7 +383,7 @@ final class HierarchicalUriComponents extends UriComponents {
 			if (this.host != null) {
 				uriBuilder.append(host);
 			}
-			if (this.port != -1) {
+			if (getPort() != -1) {
 				uriBuilder.append(':');
 				uriBuilder.append(port);
 			}
@@ -405,6 +433,19 @@ final class HierarchicalUriComponents extends UriComponents {
 		}
 	}
 
+	@Override
+	protected void copyToUriComponentsBuilder(UriComponentsBuilder builder) {
+		builder.scheme(getScheme());
+		builder.userInfo(getUserInfo());
+		builder.host(getHost());
+		builder.port(getPort());
+		builder.replacePath("");
+		this.path.copyToUriComponentsBuilder(builder);
+		builder.replaceQueryParams(getQueryParams());
+		builder.fragment(getFragment());
+	}
+
+
 	@Override
 	public boolean equals(Object obj) {
 		if (this == obj) {
@@ -428,7 +469,7 @@ final class HierarchicalUriComponents extends UriComponents {
 		int result = ObjectUtils.nullSafeHashCode(getScheme());
 		result = 31 * result + ObjectUtils.nullSafeHashCode(this.userInfo);
 		result = 31 * result + ObjectUtils.nullSafeHashCode(this.host);
-		result = 31 * result + this.port;
+		result = 31 * result + ObjectUtils.nullSafeHashCode(this.port);
 		result = 31 * result + this.path.hashCode();
 		result = 31 * result + this.queryParams.hashCode();
 		result = 31 * result + ObjectUtils.nullSafeHashCode(getFragment());
@@ -439,11 +480,11 @@ final class HierarchicalUriComponents extends UriComponents {
 	// inner types
 
 	/**
-	 * Enumeration used to identify the parts of a URI.
+	 * Enumeration used to identify the allowed characters per URI component.
 	 * <p>Contains methods to indicate whether a given character is valid in a specific URI component.
 	 * @see <a href="http://www.ietf.org/rfc/rfc3986.txt">RFC 3986</a>
 	 */
-	static enum Type {
+	enum Type {
 
 		SCHEME {
 			@Override
@@ -463,12 +504,18 @@ final class HierarchicalUriComponents extends UriComponents {
 				return isUnreserved(c) || isSubDelimiter(c) || ':' == c;
 			}
 		},
-		HOST {
+		HOST_IPV4 {
 			@Override
 			public boolean isAllowed(int c) {
 				return isUnreserved(c) || isSubDelimiter(c);
 			}
 		},
+		HOST_IPV6 {
+			@Override
+			public boolean isAllowed(int c) {
+				return isUnreserved(c) || isSubDelimiter(c) || '[' == c || ']' == c || ':' == c;
+			}
+		},
 		PORT {
 			@Override
 			public boolean isAllowed(int c) {
@@ -509,6 +556,12 @@ final class HierarchicalUriComponents extends UriComponents {
 			public boolean isAllowed(int c) {
 				return isPchar(c) || '/' == c || '?' == c;
 			}
+		},
+		URI {
+			@Override
+			public boolean isAllowed(int c) {
+				return isUnreserved(c);
+			}
 		};
 
 		/**
@@ -554,8 +607,8 @@ final class HierarchicalUriComponents extends UriComponents {
 		 * Indicates whether the given character is in the {@code reserved} set.
 		 * @see <a href="http://www.ietf.org/rfc/rfc3986.txt">RFC 3986, appendix A</a>
 		 */
-		protected boolean isReserved(char c) {
-			return isGenericDelimiter(c) || isReserved(c);
+		protected boolean isReserved(int c) {
+			return isGenericDelimiter(c) || isSubDelimiter(c);
 		}
 
 		/**
@@ -590,6 +643,8 @@ final class HierarchicalUriComponents extends UriComponents {
 		void verify();
 
 		PathComponent expand(UriTemplateVariables uriVariables);
+
+		void copyToUriComponentsBuilder(UriComponentsBuilder builder);
 	}
 
 
@@ -600,34 +655,44 @@ final class HierarchicalUriComponents extends UriComponents {
 
 		private final String path;
 
+
 		public FullPathComponent(String path) {
 			this.path = path;
 		}
 
+		@Override
 		public String getPath() {
 			return this.path;
 		}
 
+		@Override
 		public List<String> getPathSegments() {
 			String delimiter = new String(new char[]{PATH_DELIMITER});
 			String[] pathSegments = StringUtils.tokenizeToStringArray(path, delimiter);
 			return Collections.unmodifiableList(Arrays.asList(pathSegments));
 		}
 
+		@Override
 		public PathComponent encode(String encoding) throws UnsupportedEncodingException {
 			String encodedPath = encodeUriComponent(getPath(),encoding, Type.PATH);
-			return new FullPathComponent(encodedPath);
-		}
+			return new FullPathComponent(encodedPath);		}
 
+		@Override
 		public void verify() {
 			verifyUriComponent(this.path, Type.PATH);
 		}
 
+		@Override
 		public PathComponent expand(UriTemplateVariables uriVariables) {
 			String expandedPath = expandUriComponent(getPath(), uriVariables);
 			return new FullPathComponent(expandedPath);
 		}
 
+		@Override
+		public void copyToUriComponentsBuilder(UriComponentsBuilder builder) {
+			builder.path(getPath());
+		}
+
 		@Override
 		public boolean equals(Object obj) {
 			return (this == obj || (obj instanceof FullPathComponent &&
@@ -649,9 +714,11 @@ final class HierarchicalUriComponents extends UriComponents {
 		private final List<String> pathSegments;
 
 		public PathSegmentComponent(List<String> pathSegments) {
+			Assert.notNull(pathSegments);
 			this.pathSegments = Collections.unmodifiableList(new ArrayList<String>(pathSegments));
 		}
 
+		@Override
 		public String getPath() {
 			StringBuilder pathBuilder = new StringBuilder();
 			pathBuilder.append(PATH_DELIMITER);
@@ -665,10 +732,12 @@ final class HierarchicalUriComponents extends UriComponents {
 			return pathBuilder.toString();
 		}
 
+		@Override
 		public List<String> getPathSegments() {
 			return this.pathSegments;
 		}
 
+		@Override
 		public PathComponent encode(String encoding) throws UnsupportedEncodingException {
 			List<String> pathSegments = getPathSegments();
 			List<String> encodedPathSegments = new ArrayList<String>(pathSegments.size());
@@ -679,12 +748,14 @@ final class HierarchicalUriComponents extends UriComponents {
 			return new PathSegmentComponent(encodedPathSegments);
 		}
 
+		@Override
 		public void verify() {
 			for (String pathSegment : getPathSegments()) {
 				verifyUriComponent(pathSegment, Type.PATH_SEGMENT);
 			}
 		}
 
+		@Override
 		public PathComponent expand(UriTemplateVariables uriVariables) {
 			List<String> pathSegments = getPathSegments();
 			List<String> expandedPathSegments = new ArrayList<String>(pathSegments.size());
@@ -695,6 +766,11 @@ final class HierarchicalUriComponents extends UriComponents {
 			return new PathSegmentComponent(expandedPathSegments);
 		}
 
+		@Override
+		public void copyToUriComponentsBuilder(UriComponentsBuilder builder) {
+			builder.pathSegment(getPathSegments().toArray(new String[getPathSegments().size()]));
+		}
+
 		@Override
 		public boolean equals(Object obj) {
 			return (this == obj || (obj instanceof PathSegmentComponent &&
@@ -716,9 +792,11 @@ final class HierarchicalUriComponents extends UriComponents {
 		private final List<PathComponent> pathComponents;
 
 		public PathComponentComposite(List<PathComponent> pathComponents) {
+			Assert.notNull(pathComponents);
 			this.pathComponents = pathComponents;
 		}
 
+		@Override
 		public String getPath() {
 			StringBuilder pathBuilder = new StringBuilder();
 			for (PathComponent pathComponent : this.pathComponents) {
@@ -727,6 +805,7 @@ final class HierarchicalUriComponents extends UriComponents {
 			return pathBuilder.toString();
 		}
 
+		@Override
 		public List<String> getPathSegments() {
 			List<String> result = new ArrayList<String>();
 			for (PathComponent pathComponent : this.pathComponents) {
@@ -735,6 +814,7 @@ final class HierarchicalUriComponents extends UriComponents {
 			return result;
 		}
 
+		@Override
 		public PathComponent encode(String encoding) throws UnsupportedEncodingException {
 			List<PathComponent> encodedComponents = new ArrayList<PathComponent>(this.pathComponents.size());
 			for (PathComponent pathComponent : this.pathComponents) {
@@ -743,12 +823,14 @@ final class HierarchicalUriComponents extends UriComponents {
 			return new PathComponentComposite(encodedComponents);
 		}
 
+		@Override
 		public void verify() {
 			for (PathComponent pathComponent : this.pathComponents) {
 				pathComponent.verify();
 			}
 		}
 
+		@Override
 		public PathComponent expand(UriTemplateVariables uriVariables) {
 			List<PathComponent> expandedComponents = new ArrayList<PathComponent>(this.pathComponents.size());
 			for (PathComponent pathComponent : this.pathComponents) {
@@ -756,6 +838,13 @@ final class HierarchicalUriComponents extends UriComponents {
 			}
 			return new PathComponentComposite(expandedComponents);
 		}
+
+		@Override
+		public void copyToUriComponentsBuilder(UriComponentsBuilder builder) {
+			for (PathComponent pathComponent : this.pathComponents) {
+				pathComponent.copyToUriComponentsBuilder(builder);
+			}
+		}
 	}
 
 
@@ -763,21 +852,29 @@ final class HierarchicalUriComponents extends UriComponents {
 	 * Represents an empty path.
 	 */
 	static final PathComponent NULL_PATH_COMPONENT = new PathComponent() {
+		@Override
 		public String getPath() {
 			return null;
 		}
+		@Override
 		public List<String> getPathSegments() {
 			return Collections.emptyList();
 		}
+		@Override
 		public PathComponent encode(String encoding) throws UnsupportedEncodingException {
 			return this;
 		}
+		@Override
 		public void verify() {
 		}
+		@Override
 		public PathComponent expand(UriTemplateVariables uriVariables) {
 			return this;
 		}
 		@Override
+		public void copyToUriComponentsBuilder(UriComponentsBuilder builder) {
+		}
+		@Override
 		public boolean equals(Object obj) {
 			return (this == obj);
 		}
@@ -787,4 +884,23 @@ final class HierarchicalUriComponents extends UriComponents {
 		}
 	};
 
+
+	private static class QueryUriTemplateVariables implements UriTemplateVariables {
+
+		private final UriTemplateVariables delegate;
+
+		public QueryUriTemplateVariables(UriTemplateVariables delegate) {
+			this.delegate = delegate;
+		}
+
+		@Override
+		public Object getValue(String name) {
+			Object value = this.delegate.getValue(name);
+			if (ObjectUtils.isArray(value)) {
+				value = StringUtils.arrayToCommaDelimitedString(ObjectUtils.toObjectArray(value));
+			}
+			return value;
+		}
+	}
+
 }
