@@ -1,5 +1,5 @@
 /*
- * Copyright 2002-2013 the original author or authors.
+ * Copyright 2002-2015 the original author or authors.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -13,6 +13,7 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
+
 package org.springframework.mock.web.test;
 
 import java.io.IOException;
@@ -29,6 +30,7 @@ import javax.servlet.http.HttpServletRequest;
 import javax.servlet.http.HttpServletResponse;
 
 import org.springframework.beans.BeanUtils;
+import org.springframework.util.Assert;
 import org.springframework.web.util.WebUtils;
 
 /**
@@ -49,6 +51,8 @@ public class MockAsyncContext implements AsyncContext {
 
 	private long timeout = 10 * 1000L;	// 10 seconds is Tomcat's default
 
+	private final List<Runnable> dispatchHandlers = new ArrayList<Runnable>();
+
 
 	public MockAsyncContext(ServletRequest request, ServletResponse response) {
 		this.request = (HttpServletRequest) request;
@@ -56,6 +60,11 @@ public class MockAsyncContext implements AsyncContext {
 	}
 
 
+	public void addDispatchHandler(Runnable handler) {
+		Assert.notNull(handler);
+		this.dispatchHandlers.add(handler);
+	}
+
 	@Override
 	public ServletRequest getRequest() {
 		return this.request;
@@ -84,6 +93,9 @@ public class MockAsyncContext implements AsyncContext {
 	@Override
 	public void dispatch(ServletContext context, String path) {
 		this.dispatchedPath = path;
+		for (Runnable r : this.dispatchHandlers) {
+			r.run();
+		}
 	}
 
 	public String getDispatchedPath() {
