@@ -23,16 +23,21 @@ import java.io.OutputStreamWriter;
 import java.io.PrintWriter;
 import java.io.UnsupportedEncodingException;
 import java.io.Writer;
+import java.text.ParseException;
+import java.text.SimpleDateFormat;
 import java.util.ArrayList;
 import java.util.Collection;
 import java.util.Collections;
+import java.util.Date;
 import java.util.List;
 import java.util.Locale;
 import java.util.Map;
+import java.util.TimeZone;
 import javax.servlet.ServletOutputStream;
 import javax.servlet.http.Cookie;
 import javax.servlet.http.HttpServletResponse;
 
+import org.springframework.http.MediaType;
 import org.springframework.util.Assert;
 import org.springframework.util.LinkedCaseInsensitiveMap;
 import org.springframework.web.util.WebUtils;
@@ -46,6 +51,7 @@ import org.springframework.web.util.WebUtils;
  *
  * @author Juergen Hoeller
  * @author Rod Johnson
+ * @author Brian Clozel
  * @since 1.0.2
  */
 public class MockHttpServletResponse implements HttpServletResponse {
@@ -58,6 +64,10 @@ public class MockHttpServletResponse implements HttpServletResponse {
 
 	private static final String LOCATION_HEADER = "Location";
 
+	private static final String DATE_FORMAT = "EEE, dd MMM yyyy HH:mm:ss zzz";
+
+	private static final TimeZone GMT = TimeZone.getTimeZone("GMT");
+
 
 	//---------------------------------------------------------------------
 	// ServletResponse properties
@@ -139,6 +149,14 @@ public class MockHttpServletResponse implements HttpServletResponse {
 		return this.writerAccessAllowed;
 	}
 
+	/**
+	 * Return whether the character encoding has been set.
+	 * <p>If {@code false}, {@link #getCharacterEncoding()} will return a default encoding value.
+	 */
+	public boolean isCharset() {
+		return charset;
+	}
+
 	@Override
 	public void setCharacterEncoding(String characterEncoding) {
 		this.characterEncoding = characterEncoding;
@@ -216,10 +234,20 @@ public class MockHttpServletResponse implements HttpServletResponse {
 	public void setContentType(String contentType) {
 		this.contentType = contentType;
 		if (contentType != null) {
-			int charsetIndex = contentType.toLowerCase().indexOf(CHARSET_PREFIX);
-			if (charsetIndex != -1) {
-				this.characterEncoding = contentType.substring(charsetIndex + CHARSET_PREFIX.length());
-				this.charset = true;
+			try {
+				MediaType mediaType = MediaType.parseMediaType(contentType);
+				if (mediaType.getCharset() != null) {
+					this.characterEncoding = mediaType.getCharset().name();
+					this.charset = true;
+				}
+			}
+			catch (Exception ex) {
+				// Try to get charset value anyway
+				int charsetIndex = contentType.toLowerCase().indexOf(CHARSET_PREFIX);
+				if (charsetIndex != -1) {
+					this.characterEncoding = contentType.substring(charsetIndex + CHARSET_PREFIX.length());
+					this.charset = true;
+				}
 			}
 			updateContentTypeHeader();
 		}
@@ -462,12 +490,30 @@ public class MockHttpServletResponse implements HttpServletResponse {
 
 	@Override
 	public void setDateHeader(String name, long value) {
-		setHeaderValue(name, value);
+		setHeaderValue(name, formatDate(value));
+	}
+
+	public long getDateHeader(String name) {
+		SimpleDateFormat dateFormat = new SimpleDateFormat(DATE_FORMAT, Locale.US);
+		dateFormat.setTimeZone(GMT);
+		try {
+			return dateFormat.parse(getHeader(name)).getTime();
+		}
+		catch (ParseException ex) {
+			throw new IllegalArgumentException(
+					"Value for header '" + name + "' is not a valid Date: " + getHeader(name));
+		}
 	}
 
 	@Override
 	public void addDateHeader(String name, long value) {
-		addHeaderValue(name, value);
+		addHeaderValue(name, formatDate(value));
+	}
+
+	private String formatDate(long date) {
+		SimpleDateFormat dateFormat = new SimpleDateFormat(DATE_FORMAT, Locale.US);
+		dateFormat.setTimeZone(GMT);
+		return dateFormat.format(new Date(date));
 	}
 
 	@Override
