@@ -1,5 +1,5 @@
 /*
- * Copyright 2002-2013 the original author or authors.
+ * Copyright 2002-2015 the original author or authors.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -16,7 +16,9 @@
 
 package org.springframework.web.context.request;
 
-import junit.framework.TestCase;
+import org.junit.After;
+import org.junit.Before;
+import org.junit.Test;
 
 import org.springframework.beans.factory.BeanCreationException;
 import org.springframework.beans.factory.BeanCurrentlyInCreationException;
@@ -29,18 +31,22 @@ import org.springframework.mock.web.test.MockHttpServletRequest;
 import org.springframework.tests.sample.beans.DerivedTestBean;
 import org.springframework.tests.sample.beans.TestBean;
 
+import static org.junit.Assert.*;
+
 /**
  * @author Rob Harrop
  * @author Juergen Hoeller
  * @author Mark Fisher
+ * @author Sam Brannen
+ * @see SessionScopeTests
  */
-public class RequestScopeTests extends TestCase {
+public class RequestScopeTests {
 
-	private DefaultListableBeanFactory beanFactory;
+	private final DefaultListableBeanFactory beanFactory = new DefaultListableBeanFactory();
 
-	@Override
-	protected void setUp() throws Exception {
-		this.beanFactory = new DefaultListableBeanFactory();
+
+	@Before
+	public void setUp() throws Exception {
 		this.beanFactory.registerScope("request", new RequestScope());
 		this.beanFactory.setBeanExpressionResolver(new StandardBeanExpressionResolver());
 		XmlBeanDefinitionReader reader = new XmlBeanDefinitionReader(this.beanFactory);
@@ -48,63 +54,57 @@ public class RequestScopeTests extends TestCase {
 		this.beanFactory.preInstantiateSingletons();
 	}
 
-	public void testGetFromScope() throws Exception {
+	@After
+	public void resetRequestAttributes() {
+		RequestContextHolder.setRequestAttributes(null);
+	}
+
+	@Test
+	public void getFromScope() throws Exception {
 		MockHttpServletRequest request = new MockHttpServletRequest();
 		request.setContextPath("/path");
 		RequestAttributes requestAttributes = new ServletRequestAttributes(request);
 		RequestContextHolder.setRequestAttributes(requestAttributes);
 
-		try {
-			String name = "requestScopedObject";
-			assertNull(request.getAttribute(name));
-			TestBean bean = (TestBean) this.beanFactory.getBean(name);
-			assertEquals("/path", bean.getName());
-			assertSame(bean, request.getAttribute(name));
-			assertSame(bean, this.beanFactory.getBean(name));
-		}
-		finally {
-			RequestContextHolder.setRequestAttributes(null);
-		}
+		String name = "requestScopedObject";
+		assertNull(request.getAttribute(name));
+		TestBean bean = (TestBean) this.beanFactory.getBean(name);
+		assertEquals("/path", bean.getName());
+		assertSame(bean, request.getAttribute(name));
+		assertSame(bean, this.beanFactory.getBean(name));
 	}
 
-	public void testDestructionAtRequestCompletion() throws Exception {
+	@Test
+	public void destructionAtRequestCompletion() throws Exception {
 		MockHttpServletRequest request = new MockHttpServletRequest();
 		ServletRequestAttributes requestAttributes = new ServletRequestAttributes(request);
 		RequestContextHolder.setRequestAttributes(requestAttributes);
 
-		try {
-			String name = "requestScopedDisposableObject";
-			assertNull(request.getAttribute(name));
-			DerivedTestBean bean = (DerivedTestBean) this.beanFactory.getBean(name);
-			assertSame(bean, request.getAttribute(name));
-			assertSame(bean, this.beanFactory.getBean(name));
+		String name = "requestScopedDisposableObject";
+		assertNull(request.getAttribute(name));
+		DerivedTestBean bean = (DerivedTestBean) this.beanFactory.getBean(name);
+		assertSame(bean, request.getAttribute(name));
+		assertSame(bean, this.beanFactory.getBean(name));
 
-			requestAttributes.requestCompleted();
-			assertTrue(bean.wasDestroyed());
-		}
-		finally {
-			RequestContextHolder.setRequestAttributes(null);
-		}
+		requestAttributes.requestCompleted();
+		assertTrue(bean.wasDestroyed());
 	}
 
-	public void testGetFromFactoryBeanInScope() throws Exception {
+	@Test
+	public void getFromFactoryBeanInScope() throws Exception {
 		MockHttpServletRequest request = new MockHttpServletRequest();
 		RequestAttributes requestAttributes = new ServletRequestAttributes(request);
 		RequestContextHolder.setRequestAttributes(requestAttributes);
 
-		try {
-			String name = "requestScopedFactoryBean";
-			assertNull(request.getAttribute(name));
-			TestBean bean = (TestBean) this.beanFactory.getBean(name);
-			assertTrue(request.getAttribute(name) instanceof FactoryBean);
-			assertSame(bean, this.beanFactory.getBean(name));
-		}
-		finally {
-			RequestContextHolder.setRequestAttributes(null);
-		}
+		String name = "requestScopedFactoryBean";
+		assertNull(request.getAttribute(name));
+		TestBean bean = (TestBean) this.beanFactory.getBean(name);
+		assertTrue(request.getAttribute(name) instanceof FactoryBean);
+		assertSame(bean, this.beanFactory.getBean(name));
 	}
 
-	public void testCircleLeadsToException() throws Exception {
+	@Test
+	public void circleLeadsToException() throws Exception {
 		MockHttpServletRequest request = new MockHttpServletRequest();
 		RequestAttributes requestAttributes = new ServletRequestAttributes(request);
 		RequestContextHolder.setRequestAttributes(requestAttributes);
@@ -112,65 +112,54 @@ public class RequestScopeTests extends TestCase {
 		try {
 			String name = "requestScopedObjectCircle1";
 			assertNull(request.getAttribute(name));
+
 			this.beanFactory.getBean(name);
 			fail("Should have thrown BeanCreationException");
 		}
 		catch (BeanCreationException ex) {
-			// expected
 			assertTrue(ex.contains(BeanCurrentlyInCreationException.class));
 		}
-		finally {
-			RequestContextHolder.setRequestAttributes(null);
-		}
 	}
 
-	public void testInnerBeanInheritsContainingBeanScopeByDefault() {
+	@Test
+	public void innerBeanInheritsContainingBeanScopeByDefault() {
 		MockHttpServletRequest request = new MockHttpServletRequest();
 		ServletRequestAttributes requestAttributes = new ServletRequestAttributes(request);
 		RequestContextHolder.setRequestAttributes(requestAttributes);
 
-		try {
-			String outerBeanName = "requestScopedOuterBean";
-			assertNull(request.getAttribute(outerBeanName));
-			TestBean outer1 = (TestBean) this.beanFactory.getBean(outerBeanName);
-			assertNotNull(request.getAttribute(outerBeanName));
-			TestBean inner1 = (TestBean) outer1.getSpouse();
-			assertSame(outer1, this.beanFactory.getBean(outerBeanName));
-			requestAttributes.requestCompleted();
-			assertTrue(outer1.wasDestroyed());
-			assertTrue(inner1.wasDestroyed());
-			request = new MockHttpServletRequest();
-			requestAttributes = new ServletRequestAttributes(request);
-			RequestContextHolder.setRequestAttributes(requestAttributes);
-			TestBean outer2 = (TestBean) this.beanFactory.getBean(outerBeanName);
-			assertNotSame(outer1, outer2);
-			assertNotSame(inner1, outer2.getSpouse());
-		}
-		finally {
-			RequestContextHolder.setRequestAttributes(null);
-		}
+		String outerBeanName = "requestScopedOuterBean";
+		assertNull(request.getAttribute(outerBeanName));
+		TestBean outer1 = (TestBean) this.beanFactory.getBean(outerBeanName);
+		assertNotNull(request.getAttribute(outerBeanName));
+		TestBean inner1 = (TestBean) outer1.getSpouse();
+		assertSame(outer1, this.beanFactory.getBean(outerBeanName));
+		requestAttributes.requestCompleted();
+		assertTrue(outer1.wasDestroyed());
+		assertTrue(inner1.wasDestroyed());
+		request = new MockHttpServletRequest();
+		requestAttributes = new ServletRequestAttributes(request);
+		RequestContextHolder.setRequestAttributes(requestAttributes);
+		TestBean outer2 = (TestBean) this.beanFactory.getBean(outerBeanName);
+		assertNotSame(outer1, outer2);
+		assertNotSame(inner1, outer2.getSpouse());
 	}
 
-	public void testRequestScopedInnerBeanDestroyedWhileContainedBySingleton() throws Exception {
+	@Test
+	public void requestScopedInnerBeanDestroyedWhileContainedBySingleton() throws Exception {
 		MockHttpServletRequest request = new MockHttpServletRequest();
 		ServletRequestAttributes requestAttributes = new ServletRequestAttributes(request);
 		RequestContextHolder.setRequestAttributes(requestAttributes);
 
-		try {
-			String outerBeanName = "singletonOuterBean";
-			TestBean outer1 = (TestBean) this.beanFactory.getBean(outerBeanName);
-			assertNull(request.getAttribute(outerBeanName));
-			TestBean inner1 = (TestBean) outer1.getSpouse();
-			TestBean outer2 = (TestBean) this.beanFactory.getBean(outerBeanName);
-			assertSame(outer1, outer2);
-			assertSame(inner1, outer2.getSpouse());
-			requestAttributes.requestCompleted();
-			assertTrue(inner1.wasDestroyed());
-			assertFalse(outer1.wasDestroyed());
-		}
-		finally {
-			RequestContextHolder.setRequestAttributes(null);
-		}
+		String outerBeanName = "singletonOuterBean";
+		TestBean outer1 = (TestBean) this.beanFactory.getBean(outerBeanName);
+		assertNull(request.getAttribute(outerBeanName));
+		TestBean inner1 = (TestBean) outer1.getSpouse();
+		TestBean outer2 = (TestBean) this.beanFactory.getBean(outerBeanName);
+		assertSame(outer1, outer2);
+		assertSame(inner1, outer2.getSpouse());
+		requestAttributes.requestCompleted();
+		assertTrue(inner1.wasDestroyed());
+		assertFalse(outer1.wasDestroyed());
 	}
 
 }
