@@ -1,5 +1,5 @@
 /*
- * Copyright 2002-2014 the original author or authors.
+ * Copyright 2002-2016 the original author or authors.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -16,19 +16,12 @@
 
 package org.springframework.web.method.annotation;
 
-import static org.junit.Assert.assertEquals;
-import static org.junit.Assert.assertNull;
-import static org.junit.Assert.assertSame;
-import static org.junit.Assert.assertTrue;
-import static org.junit.Assert.fail;
-import static org.mockito.BDDMockito.given;
-import static org.mockito.BDDMockito.mock;
-
 import java.lang.reflect.Method;
-import java.util.Arrays;
+import java.util.Collections;
 
 import org.junit.Before;
 import org.junit.Test;
+
 import org.springframework.core.LocalVariableTableParameterNameDiscoverer;
 import org.springframework.mock.web.test.MockHttpServletRequest;
 import org.springframework.ui.Model;
@@ -43,10 +36,19 @@ import org.springframework.web.bind.support.SessionAttributeStore;
 import org.springframework.web.bind.support.WebDataBinderFactory;
 import org.springframework.web.context.request.NativeWebRequest;
 import org.springframework.web.context.request.ServletWebRequest;
+import org.springframework.web.method.HandlerMethod;
 import org.springframework.web.method.support.HandlerMethodArgumentResolverComposite;
 import org.springframework.web.method.support.InvocableHandlerMethod;
 import org.springframework.web.method.support.ModelAndViewContainer;
 
+import static org.junit.Assert.assertEquals;
+import static org.junit.Assert.assertNull;
+import static org.junit.Assert.assertSame;
+import static org.junit.Assert.assertTrue;
+import static org.junit.Assert.fail;
+import static org.mockito.BDDMockito.given;
+import static org.mockito.BDDMockito.mock;
+
 
 /**
  * Text fixture for {@link ModelFactory} tests.
@@ -55,103 +57,116 @@ import org.springframework.web.method.support.ModelAndViewContainer;
  */
 public class ModelFactoryTests {
 
+	private NativeWebRequest webRequest;
+
+	private SessionAttributesHandler attributeHandler;
+
+	private SessionAttributeStore attributeStore;
+
 	private TestController controller = new TestController();
 
-	private InvocableHandlerMethod handleMethod;
-
-	private InvocableHandlerMethod handleSessionAttrMethod;
-
-	private SessionAttributesHandler sessionAttrsHandler;
-
-	private SessionAttributeStore sessionAttributeStore;
-
-	private NativeWebRequest webRequest;
+	private ModelAndViewContainer mavContainer;
 
 
 	@Before
 	public void setUp() throws Exception {
-		this.controller = new TestController();
-
-		Method method = TestController.class.getDeclaredMethod("handle");
-		this.handleMethod = new InvocableHandlerMethod(this.controller, method);
-
-		method = TestController.class.getDeclaredMethod("handleSessionAttr", String.class);
-		this.handleSessionAttrMethod = new InvocableHandlerMethod(this.controller, method);
-
-		this.sessionAttributeStore = new DefaultSessionAttributeStore();
-		this.sessionAttrsHandler = new SessionAttributesHandler(TestController.class, this.sessionAttributeStore);
 		this.webRequest = new ServletWebRequest(new MockHttpServletRequest());
+		this.attributeStore = new DefaultSessionAttributeStore();
+		this.attributeHandler = new SessionAttributesHandler(TestController.class, this.attributeStore);
+		this.controller = new TestController();
+		this.mavContainer = new ModelAndViewContainer();
 	}
 
 
 	@Test
 	public void modelAttributeMethod() throws Exception {
 		ModelFactory modelFactory = createModelFactory("modelAttr", Model.class);
-		ModelAndViewContainer mavContainer = new ModelAndViewContainer();
-		modelFactory.initModel(this.webRequest, mavContainer, this.handleMethod);
+		HandlerMethod handlerMethod = createHandlerMethod("handle");
+		modelFactory.initModel(this.webRequest, this.mavContainer, handlerMethod);
 
-		assertEquals(Boolean.TRUE, mavContainer.getModel().get("modelAttr"));
+		assertEquals(Boolean.TRUE, this.mavContainer.getModel().get("modelAttr"));
 	}
 
 	@Test
 	public void modelAttributeMethodWithExplicitName() throws Exception {
 		ModelFactory modelFactory = createModelFactory("modelAttrWithName");
-		ModelAndViewContainer mavContainer = new ModelAndViewContainer();
-		modelFactory.initModel(this.webRequest, mavContainer, this.handleMethod);
+		HandlerMethod handlerMethod = createHandlerMethod("handle");
+		modelFactory.initModel(this.webRequest, this.mavContainer, handlerMethod);
 
-		assertEquals(Boolean.TRUE, mavContainer.getModel().get("name"));
+		assertEquals(Boolean.TRUE, this.mavContainer.getModel().get("name"));
 	}
 
 	@Test
 	public void modelAttributeMethodWithNameByConvention() throws Exception {
 		ModelFactory modelFactory = createModelFactory("modelAttrConvention");
-		ModelAndViewContainer mavContainer = new ModelAndViewContainer();
-		modelFactory.initModel(this.webRequest, mavContainer, this.handleMethod);
+		HandlerMethod handlerMethod = createHandlerMethod("handle");
+		modelFactory.initModel(this.webRequest, this.mavContainer, handlerMethod);
 
-		assertEquals(Boolean.TRUE, mavContainer.getModel().get("boolean"));
+		assertEquals(Boolean.TRUE, this.mavContainer.getModel().get("boolean"));
 	}
 
 	@Test
 	public void modelAttributeMethodWithNullReturnValue() throws Exception {
 		ModelFactory modelFactory = createModelFactory("nullModelAttr");
-		ModelAndViewContainer mavContainer = new ModelAndViewContainer();
-		modelFactory.initModel(this.webRequest, mavContainer, this.handleMethod);
+		HandlerMethod handlerMethod = createHandlerMethod("handle");
+		modelFactory.initModel(this.webRequest, this.mavContainer, handlerMethod);
 
-		assertTrue(mavContainer.containsAttribute("name"));
-		assertNull(mavContainer.getModel().get("name"));
+		assertTrue(this.mavContainer.containsAttribute("name"));
+		assertNull(this.mavContainer.getModel().get("name"));
+	}
+
+	@Test
+	public void modelAttributeWithBindingDisabled() throws Exception {
+		ModelFactory modelFactory = createModelFactory("modelAttrWithBindingDisabled");
+		HandlerMethod handlerMethod = createHandlerMethod("handle");
+		modelFactory.initModel(this.webRequest, this.mavContainer, handlerMethod);
+
+		assertTrue(this.mavContainer.containsAttribute("foo"));
+		assertTrue(this.mavContainer.isBindingDisabled("foo"));
+	}
+
+	@Test
+	public void modelAttributeFromSessionWithBindingDisabled() throws Exception {
+		Foo foo = new Foo();
+		this.attributeStore.storeAttribute(this.webRequest, "foo", foo);
+
+		ModelFactory modelFactory = createModelFactory("modelAttrWithBindingDisabled");
+		HandlerMethod handlerMethod = createHandlerMethod("handle");
+		modelFactory.initModel(this.webRequest, this.mavContainer, handlerMethod);
+
+		assertTrue(this.mavContainer.containsAttribute("foo"));
+		assertSame(foo, this.mavContainer.getModel().get("foo"));
+		assertTrue(this.mavContainer.isBindingDisabled("foo"));
 	}
 
 	@Test
 	public void sessionAttribute() throws Exception {
-		this.sessionAttributeStore.storeAttribute(this.webRequest, "sessionAttr", "sessionAttrValue");
-
-		// Resolve successfully handler session attribute once
-		assertTrue(sessionAttrsHandler.isHandlerSessionAttribute("sessionAttr", null));
+		this.attributeStore.storeAttribute(this.webRequest, "sessionAttr", "sessionAttrValue");
 
 		ModelFactory modelFactory = createModelFactory("modelAttr", Model.class);
-		ModelAndViewContainer mavContainer = new ModelAndViewContainer();
-		modelFactory.initModel(this.webRequest, mavContainer, this.handleMethod);
+		HandlerMethod handlerMethod = createHandlerMethod("handle");
+		modelFactory.initModel(this.webRequest, this.mavContainer, handlerMethod);
 
-		assertEquals("sessionAttrValue", mavContainer.getModel().get("sessionAttr"));
+		assertEquals("sessionAttrValue", this.mavContainer.getModel().get("sessionAttr"));
 	}
 
 	@Test
 	public void sessionAttributeNotPresent() throws Exception {
-		ModelFactory modelFactory = new ModelFactory(null, null, this.sessionAttrsHandler);
-
+		ModelFactory modelFactory = new ModelFactory(null, null, this.attributeHandler);
+		HandlerMethod handlerMethod = createHandlerMethod("handleSessionAttr", String.class);
 		try {
-			modelFactory.initModel(this.webRequest, new ModelAndViewContainer(), this.handleSessionAttrMethod);
+			modelFactory.initModel(this.webRequest, this.mavContainer, handlerMethod);
 			fail("Expected HttpSessionRequiredException");
 		}
 		catch (HttpSessionRequiredException e) {
 			// expected
 		}
 
-		this.sessionAttributeStore.storeAttribute(this.webRequest, "sessionAttr", "sessionAttrValue");
-		ModelAndViewContainer mavContainer = new ModelAndViewContainer();
-		modelFactory.initModel(this.webRequest, mavContainer, this.handleSessionAttrMethod);
+		// Now add attribute and try again
+		this.attributeStore.storeAttribute(this.webRequest, "sessionAttr", "sessionAttrValue");
 
-		assertEquals("sessionAttrValue", mavContainer.getModel().get("sessionAttr"));
+		modelFactory.initModel(this.webRequest, this.mavContainer, handlerMethod);
+		assertEquals("sessionAttrValue", this.mavContainer.getModel().get("sessionAttr"));
 	}
 
 	@Test
@@ -165,11 +180,12 @@ public class ModelFactoryTests {
 		WebDataBinderFactory binderFactory = mock(WebDataBinderFactory.class);
 		given(binderFactory.createBinder(this.webRequest, command, commandName)).willReturn(dataBinder);
 
-		ModelFactory modelFactory = new ModelFactory(null, binderFactory, this.sessionAttrsHandler);
+		ModelFactory modelFactory = new ModelFactory(null, binderFactory, this.attributeHandler);
 		modelFactory.updateModel(this.webRequest, container);
 
 		assertEquals(command, container.getModel().get(commandName));
-		assertSame(dataBinder.getBindingResult(), container.getModel().get(bindingResultKey(commandName)));
+		String bindingResultKey = BindingResult.MODEL_KEY_PREFIX + commandName;
+		assertSame(dataBinder.getBindingResult(), container.getModel().get(bindingResultKey));
 		assertEquals(2, container.getModel().size());
 	}
 
@@ -184,11 +200,11 @@ public class ModelFactoryTests {
 		WebDataBinderFactory binderFactory = mock(WebDataBinderFactory.class);
 		given(binderFactory.createBinder(this.webRequest, attribute, attributeName)).willReturn(dataBinder);
 
-		ModelFactory modelFactory = new ModelFactory(null, binderFactory, this.sessionAttrsHandler);
+		ModelFactory modelFactory = new ModelFactory(null, binderFactory, this.attributeHandler);
 		modelFactory.updateModel(this.webRequest, container);
 
 		assertEquals(attribute, container.getModel().get(attributeName));
-		assertEquals(attribute, this.sessionAttributeStore.retrieveAttribute(this.webRequest, attributeName));
+		assertEquals(attribute, this.attributeStore.retrieveAttribute(this.webRequest, attributeName));
 	}
 
 	@Test
@@ -198,9 +214,7 @@ public class ModelFactoryTests {
 		ModelAndViewContainer container = new ModelAndViewContainer();
 		container.addAttribute(attributeName, attribute);
 
-		// Store and resolve once (to be "remembered")
-		this.sessionAttributeStore.storeAttribute(this.webRequest, attributeName, attribute);
-		this.sessionAttrsHandler.isHandlerSessionAttribute(attributeName, null);
+		this.attributeStore.storeAttribute(this.webRequest, attributeName, attribute);
 
 		WebDataBinder dataBinder = new WebDataBinder(attribute, attributeName);
 		WebDataBinderFactory binderFactory = mock(WebDataBinderFactory.class);
@@ -208,11 +222,11 @@ public class ModelFactoryTests {
 
 		container.getSessionStatus().setComplete();
 
-		ModelFactory modelFactory = new ModelFactory(null, binderFactory, this.sessionAttrsHandler);
+		ModelFactory modelFactory = new ModelFactory(null, binderFactory, this.attributeHandler);
 		modelFactory.updateModel(this.webRequest, container);
 
 		assertEquals(attribute, container.getModel().get(attributeName));
-		assertNull(this.sessionAttributeStore.retrieveAttribute(this.webRequest, attributeName));
+		assertNull(this.attributeStore.retrieveAttribute(this.webRequest, attributeName));
 	}
 
 	// SPR-12542
@@ -233,34 +247,34 @@ public class ModelFactoryTests {
 		WebDataBinderFactory binderFactory = mock(WebDataBinderFactory.class);
 		given(binderFactory.createBinder(this.webRequest, attribute, attributeName)).willReturn(dataBinder);
 
-		ModelFactory modelFactory = new ModelFactory(null, binderFactory, this.sessionAttrsHandler);
+		ModelFactory modelFactory = new ModelFactory(null, binderFactory, this.attributeHandler);
 		modelFactory.updateModel(this.webRequest, container);
 
 		assertEquals(queryParam, container.getModel().get(queryParamName));
 		assertEquals(1, container.getModel().size());
-		assertEquals(attribute, this.sessionAttributeStore.retrieveAttribute(this.webRequest, attributeName));
+		assertEquals(attribute, this.attributeStore.retrieveAttribute(this.webRequest, attributeName));
 	}
 
 
-	private String bindingResultKey(String key) {
-		return BindingResult.MODEL_KEY_PREFIX + key;
+	private ModelFactory createModelFactory(String methodName, Class<?>... parameterTypes) throws Exception {
+		HandlerMethodArgumentResolverComposite resolvers = new HandlerMethodArgumentResolverComposite();
+		resolvers.addResolver(new ModelMethodProcessor());
+
+		InvocableHandlerMethod modelMethod = createHandlerMethod(methodName, parameterTypes);
+		modelMethod.setHandlerMethodArgumentResolvers(resolvers);
+		modelMethod.setDataBinderFactory(null);
+		modelMethod.setParameterNameDiscoverer(new LocalVariableTableParameterNameDiscoverer());
+
+		return new ModelFactory(Collections.singletonList(modelMethod), null, this.attributeHandler);
 	}
 
-	private ModelFactory createModelFactory(String methodName, Class<?>... parameterTypes) throws Exception{
-		Method method = TestController.class.getMethod(methodName, parameterTypes);
-
-		HandlerMethodArgumentResolverComposite argResolvers = new HandlerMethodArgumentResolverComposite();
-		argResolvers.addResolver(new ModelMethodProcessor());
-
-		InvocableHandlerMethod handlerMethod = new InvocableHandlerMethod(this.controller, method);
-		handlerMethod.setHandlerMethodArgumentResolvers(argResolvers);
-		handlerMethod.setDataBinderFactory(null);
-		handlerMethod.setParameterNameDiscoverer(new LocalVariableTableParameterNameDiscoverer());
-
-		return new ModelFactory(Arrays.asList(handlerMethod), null, this.sessionAttrsHandler);
+	private InvocableHandlerMethod createHandlerMethod(String methodName, Class<?>... paramTypes) throws Exception {
+		Method method = this.controller.getClass().getMethod(methodName, paramTypes);
+		return new InvocableHandlerMethod(this.controller, method);
 	}
 
-	@SessionAttributes("sessionAttr") @SuppressWarnings("unused")
+
+	@SessionAttributes({"sessionAttr", "foo"}) @SuppressWarnings("unused")
 	private static class TestController {
 
 		@ModelAttribute
@@ -283,6 +297,11 @@ public class ModelFactoryTests {
 			return null;
 		}
 
+		@ModelAttribute(name="foo", binding=false)
+		public Foo modelAttrWithBindingDisabled() {
+			return new Foo();
+		}
+
 		public void handle() {
 		}
 
@@ -290,4 +309,7 @@ public class ModelFactoryTests {
 		}
 	}
 
+	private static class Foo {
+	}
+
 }
