@@ -1,5 +1,5 @@
 /*
- * Copyright 2002-2014 the original author or authors.
+ * Copyright 2002-2015 the original author or authors.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -111,20 +111,22 @@ public class CommonsMultipartResolverTests {
 		doTestFiles(request);
 
 		doTestBinding(resolver, originalRequest, request);
+
+		wac.close();
 	}
 
 	private void doTestParameters(MultipartHttpServletRequest request) {
 		Set<String> parameterNames = new HashSet<String>();
-		Enumeration parameterEnum = request.getParameterNames();
+		Enumeration<String> parameterEnum = request.getParameterNames();
 		while (parameterEnum.hasMoreElements()) {
-			parameterNames.add((String) parameterEnum.nextElement());
+			parameterNames.add(parameterEnum.nextElement());
 		}
 		assertEquals(3, parameterNames.size());
 		assertTrue(parameterNames.contains("field3"));
 		assertTrue(parameterNames.contains("field4"));
 		assertTrue(parameterNames.contains("getField"));
 		assertEquals("value3", request.getParameter("field3"));
-		List parameterValues = Arrays.asList(request.getParameterValues("field3"));
+		List<String> parameterValues = Arrays.asList(request.getParameterValues("field3"));
 		assertEquals(1, parameterValues.size());
 		assertTrue(parameterValues.contains("value3"));
 		assertEquals("value4", request.getParameter("field4"));
@@ -164,9 +166,9 @@ public class CommonsMultipartResolverTests {
 
 	private void doTestFiles(MultipartHttpServletRequest request) throws IOException {
 		Set<String> fileNames = new HashSet<String>();
-		Iterator fileIter = request.getFileNames();
+		Iterator<String> fileIter = request.getFileNames();
 		while (fileIter.hasNext()) {
-			fileNames.add((String) fileIter.next());
+			fileNames.add(fileIter.next());
 		}
 		assertEquals(3, fileNames.size());
 		assertTrue(fileNames.contains("field1"));
@@ -381,9 +383,9 @@ public class CommonsMultipartResolverTests {
 					MockFileItem fileItem1x = new MockFileItem(
 						"field1", "type1", empty ? "" : "field1.txt", empty ? "" : "text1");
 					MockFileItem fileItem2 = new MockFileItem(
-						"field2", "type2", empty ? "" : "C:/field2.txt", empty ? "" : "text2");
+						"field2", "type2", empty ? "" : "C:\\mypath/field2.txt", empty ? "" : "text2");
 					MockFileItem fileItem2x = new MockFileItem(
-						"field2x", "type2", empty ? "" : "C:\\field2x.txt", empty ? "" : "text2");
+						"field2x", "type2", empty ? "" : "C:/mypath\\field2x.txt", empty ? "" : "text2");
 					MockFileItem fileItem3 = new MockFileItem("field3", null, null, "value3");
 					MockFileItem fileItem4 = new MockFileItem("field4", "text/html; charset=iso-8859-1", null, "value4");
 					MockFileItem fileItem5 = new MockFileItem("field4", null, null, "value5");
