@@ -1,5 +1,5 @@
 /*
- * Copyright 2002-2012 the original author or authors.
+ * Copyright 2002-2015 the original author or authors.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -58,7 +58,7 @@ import org.springframework.beans.factory.BeanFactoryAware;
 import org.springframework.beans.factory.config.BeanExpressionContext;
 import org.springframework.beans.factory.config.BeanExpressionResolver;
 import org.springframework.beans.factory.config.ConfigurableBeanFactory;
-import org.springframework.core.LocalVariableTableParameterNameDiscoverer;
+import org.springframework.core.DefaultParameterNameDiscoverer;
 import org.springframework.core.Ordered;
 import org.springframework.core.ParameterNameDiscoverer;
 import org.springframework.core.annotation.AnnotationUtils;
@@ -76,8 +76,6 @@ import org.springframework.web.bind.annotation.RequestMapping;
 import org.springframework.web.bind.annotation.RequestMethod;
 import org.springframework.web.bind.annotation.RequestParam;
 import org.springframework.web.bind.annotation.SessionAttributes;
-import org.springframework.web.bind.annotation.support.HandlerMethodInvoker;
-import org.springframework.web.bind.annotation.support.HandlerMethodResolver;
 import org.springframework.web.bind.support.DefaultSessionAttributeStore;
 import org.springframework.web.bind.support.SessionAttributeStore;
 import org.springframework.web.bind.support.WebArgumentResolver;
@@ -138,7 +136,7 @@ public class AnnotationMethodHandlerAdapter extends PortletContentGenerator
 
 	private boolean synchronizeOnSession = false;
 
-	private ParameterNameDiscoverer parameterNameDiscoverer = new LocalVariableTableParameterNameDiscoverer();
+	private ParameterNameDiscoverer parameterNameDiscoverer = new DefaultParameterNameDiscoverer();
 
 	private WebArgumentResolver[] customArgumentResolvers;
 
@@ -211,7 +209,7 @@ public class AnnotationMethodHandlerAdapter extends PortletContentGenerator
 	/**
 	 * Set the ParameterNameDiscoverer to use for resolving method parameter
 	 * names if needed (e.g. for default attribute names).
-	 * <p>Default is a {@link org.springframework.core.LocalVariableTableParameterNameDiscoverer}.
+	 * <p>Default is a {@link org.springframework.core.DefaultParameterNameDiscoverer}.
 	 */
 	public void setParameterNameDiscoverer(ParameterNameDiscoverer parameterNameDiscoverer) {
 		this.parameterNameDiscoverer = parameterNameDiscoverer;
@@ -232,7 +230,7 @@ public class AnnotationMethodHandlerAdapter extends PortletContentGenerator
 	 * having a chance to resolve an argument value before the standard
 	 * argument handling kicks in.
 	 */
-	public void setCustomArgumentResolvers(WebArgumentResolver[] argumentResolvers) {
+	public void setCustomArgumentResolvers(WebArgumentResolver... argumentResolvers) {
 		this.customArgumentResolvers = argumentResolvers;
 	}
 
@@ -250,7 +248,7 @@ public class AnnotationMethodHandlerAdapter extends PortletContentGenerator
 	 * Any such custom ModelAndViewResolver will kick in first, having a chance to
 	 * resolve an return value before the standard ModelAndView handling kicks in.
 	 */
-	public void setCustomModelAndViewResolvers(ModelAndViewResolver[] customModelAndViewResolvers) {
+	public void setCustomModelAndViewResolvers(ModelAndViewResolver... customModelAndViewResolvers) {
 		this.customModelAndViewResolvers = customModelAndViewResolvers;
 	}
 
@@ -263,10 +261,12 @@ public class AnnotationMethodHandlerAdapter extends PortletContentGenerator
 		this.order = order;
 	}
 
+	@Override
 	public int getOrder() {
 		return this.order;
 	}
 
+	@Override
 	public void setBeanFactory(BeanFactory beanFactory) {
 		if (beanFactory instanceof ConfigurableBeanFactory) {
 			this.beanFactory = (ConfigurableBeanFactory) beanFactory;
@@ -275,10 +275,12 @@ public class AnnotationMethodHandlerAdapter extends PortletContentGenerator
 	}
 
 
+	@Override
 	public boolean supports(Object handler) {
 		return getMethodResolver(handler).hasHandlerMethods();
 	}
 
+	@Override
 	public void handleAction(ActionRequest request, ActionResponse response, Object handler) throws Exception {
 		Object returnValue = doHandle(request, response, handler);
 		if (returnValue != null) {
@@ -286,16 +288,19 @@ public class AnnotationMethodHandlerAdapter extends PortletContentGenerator
 		}
 	}
 
+	@Override
 	public ModelAndView handleRender(RenderRequest request, RenderResponse response, Object handler) throws Exception {
 		checkAndPrepare(request, response);
 		return doHandle(request, response, handler);
 	}
 
+	@Override
 	public ModelAndView handleResource(ResourceRequest request, ResourceResponse response, Object handler) throws Exception {
 		checkAndPrepare(request, response);
 		return doHandle(request, response, handler);
 	}
 
+	@Override
 	public void handleEvent(EventRequest request, EventResponse response, Object handler) throws Exception {
 		Object returnValue = doHandle(request, response, handler);
 		if (returnValue != null) {
@@ -374,7 +379,8 @@ public class AnnotationMethodHandlerAdapter extends PortletContentGenerator
 				if (response instanceof EventResponse) {
 					// Update the existing model, if any, when responding to an event -
 					// whereas we're replacing the model in case of an action response.
-					Map existingModel = (Map) request.getPortletSession().getAttribute(IMPLICIT_MODEL_SESSION_ATTRIBUTE);
+					Map<String, Object> existingModel = (Map<String, Object>)
+							request.getPortletSession().getAttribute(IMPLICIT_MODEL_SESSION_ATTRIBUTE);
 					if (existingModel != null) {
 						existingModel.putAll(implicitModel);
 						modelToStore = existingModel;
@@ -394,7 +400,7 @@ public class AnnotationMethodHandlerAdapter extends PortletContentGenerator
 	 * Build a HandlerMethodResolver for the given handler type.
 	 */
 	private PortletHandlerMethodResolver getMethodResolver(Object handler) {
-		Class handlerClass = ClassUtils.getUserClass(handler);
+		Class<?> handlerClass = ClassUtils.getUserClass(handler);
 		PortletHandlerMethodResolver resolver = this.methodResolverCache.get(handlerClass);
 		if (resolver == null) {
 			synchronized (this.methodResolverCache) {
@@ -426,9 +432,10 @@ public class AnnotationMethodHandlerAdapter extends PortletContentGenerator
 
 
 	/**
-	 * Portlet-specific subclass of {@link HandlerMethodResolver}.
+	 * Portlet-specific subclass of {@code HandlerMethodResolver}.
 	 */
-	private static class PortletHandlerMethodResolver extends HandlerMethodResolver {
+	@SuppressWarnings("deprecation")
+	private static class PortletHandlerMethodResolver extends org.springframework.web.bind.annotation.support.HandlerMethodResolver {
 
 		private final Map<Method, RequestMappingInfo> mappings = new HashMap<Method, RequestMappingInfo>();
 
@@ -448,10 +455,10 @@ public class AnnotationMethodHandlerAdapter extends PortletContentGenerator
 			EventMapping eventMapping = AnnotationUtils.findAnnotation(method, EventMapping.class);
 			RequestMapping requestMapping = AnnotationUtils.findAnnotation(method, RequestMapping.class);
 			if (actionMapping != null) {
-				mappingInfo.initPhaseMapping(PortletRequest.ACTION_PHASE, actionMapping.value(), actionMapping.params());
+				mappingInfo.initPhaseMapping(PortletRequest.ACTION_PHASE, actionMapping.name(), actionMapping.params());
 			}
 			if (renderMapping != null) {
-				mappingInfo.initPhaseMapping(PortletRequest.RENDER_PHASE, renderMapping.value(), renderMapping.params());
+				mappingInfo.initPhaseMapping(PortletRequest.RENDER_PHASE, renderMapping.windowState(), renderMapping.params());
 			}
 			if (resourceMapping != null) {
 				mappingInfo.initPhaseMapping(PortletRequest.RESOURCE_PHASE, resourceMapping.value(), new String[0]);
@@ -512,7 +519,7 @@ public class AnnotationMethodHandlerAdapter extends PortletContentGenerator
 		}
 
 		private String determineDefaultPhase(Method handlerMethod) {
-			if (!void.class.equals(handlerMethod.getReturnType())) {
+			if (void.class != handlerMethod.getReturnType()) {
 				return PortletRequest.RENDER_PHASE;
 			}
 			for (Class<?> argType : handlerMethod.getParameterTypes()) {
@@ -537,17 +544,18 @@ public class AnnotationMethodHandlerAdapter extends PortletContentGenerator
 
 
 	/**
-	 * Portlet-specific subclass of {@link HandlerMethodInvoker}.
+	 * Portlet-specific subclass of {@code HandlerMethodInvoker}.
 	 */
-	private class PortletHandlerMethodInvoker extends HandlerMethodInvoker {
+	@SuppressWarnings("deprecation")
+	private class PortletHandlerMethodInvoker extends org.springframework.web.bind.annotation.support.HandlerMethodInvoker {
 
-		public PortletHandlerMethodInvoker(HandlerMethodResolver resolver) {
+		public PortletHandlerMethodInvoker(org.springframework.web.bind.annotation.support.HandlerMethodResolver resolver) {
 			super(resolver, webBindingInitializer, sessionAttributeStore,
 					parameterNameDiscoverer, customArgumentResolvers, null);
 		}
 
 		@Override
-		protected void raiseMissingParameterException(String paramName, Class paramType) throws Exception {
+		protected void raiseMissingParameterException(String paramName, Class<?> paramType) throws Exception {
 			throw new MissingPortletRequestParameterException(paramName, paramType.getSimpleName());
 		}
 
@@ -584,7 +592,7 @@ public class AnnotationMethodHandlerAdapter extends PortletContentGenerator
 		}
 
 		@Override
-		protected Object resolveCookieValue(String cookieName, Class paramType, NativeWebRequest webRequest)
+		protected Object resolveCookieValue(String cookieName, Class<?> paramType, NativeWebRequest webRequest)
 				throws Exception {
 
 			PortletRequest portletRequest = webRequest.getNativeRequest(PortletRequest.class);
@@ -642,7 +650,7 @@ public class AnnotationMethodHandlerAdapter extends PortletContentGenerator
 			else if (Principal.class.isAssignableFrom(parameterType)) {
 				return request.getUserPrincipal();
 			}
-			else if (Locale.class.equals(parameterType)) {
+			else if (Locale.class == parameterType) {
 				return request.getLocale();
 			}
 			else if (InputStream.class.isAssignableFrom(parameterType)) {
@@ -669,7 +677,7 @@ public class AnnotationMethodHandlerAdapter extends PortletContentGenerator
 				}
 				return ((MimeResponse) response).getWriter();
 			}
-			else if (Event.class.equals(parameterType)) {
+			else if (Event.class == parameterType) {
 				if (!(request instanceof EventRequest)) {
 					throw new IllegalStateException("Event can only get obtained from EventRequest");
 				}
@@ -679,7 +687,7 @@ public class AnnotationMethodHandlerAdapter extends PortletContentGenerator
 		}
 
 		@SuppressWarnings("unchecked")
-		public ModelAndView getModelAndView(Method handlerMethod, Class handlerType, Object returnValue, ExtendedModelMap implicitModel,
+		public ModelAndView getModelAndView(Method handlerMethod, Class<?> handlerType, Object returnValue, ExtendedModelMap implicitModel,
 				PortletWebRequest webRequest) {
 			// Invoke custom resolvers if present...
 			if (customModelAndViewResolvers != null) {
@@ -718,7 +726,7 @@ public class AnnotationMethodHandlerAdapter extends PortletContentGenerator
 				return new ModelAndView().addAllObjects(implicitModel);
 			}
 			else if (returnValue instanceof Map) {
-				return new ModelAndView().addAllObjects(implicitModel).addAllObjects((Map) returnValue);
+				return new ModelAndView().addAllObjects(implicitModel).addAllObjects((Map<String, Object>) returnValue);
 			}
 			else if (returnValue instanceof String) {
 				return new ModelAndView((String) returnValue).addAllObjects(implicitModel);
@@ -805,9 +813,9 @@ public class AnnotationMethodHandlerAdapter extends PortletContentGenerator
 					}
 				}
 			}
-			return PortletAnnotationMappingUtils.checkRequestMethod(this.methods, request) &&
+			return (PortletAnnotationMappingUtils.checkRequestMethod(this.methods, request) &&
 					PortletAnnotationMappingUtils.checkParameters(this.params, request) &&
-					PortletAnnotationMappingUtils.checkHeaders(this.headers, request);
+					PortletAnnotationMappingUtils.checkHeaders(this.headers, request));
 		}
 
 		public boolean isBetterMatchThan(RequestMappingInfo other) {
