@@ -1,5 +1,5 @@
 /*
- * Copyright 2002-2014 the original author or authors.
+ * Copyright 2002-2016 the original author or authors.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -42,9 +42,10 @@ import org.springframework.beans.factory.NoSuchBeanDefinitionException;
 import org.springframework.context.ApplicationContext;
 import org.springframework.context.ConfigurableApplicationContext;
 import org.springframework.context.i18n.LocaleContext;
-import org.springframework.core.OrderComparator;
+import org.springframework.core.annotation.AnnotationAwareOrderComparator;
 import org.springframework.core.io.ClassPathResource;
 import org.springframework.core.io.support.PropertiesLoaderUtils;
+import org.springframework.http.server.ServletServerHttpRequest;
 import org.springframework.ui.context.ThemeSource;
 import org.springframework.util.ClassUtils;
 import org.springframework.util.StringUtils;
@@ -59,73 +60,89 @@ import org.springframework.web.util.NestedServletException;
 import org.springframework.web.util.WebUtils;
 
 /**
- * Central dispatcher for HTTP request handlers/controllers, e.g. for web UI controllers or HTTP-based remote service
- * exporters. Dispatches to registered handlers for processing a web request, providing convenient mapping and exception
- * handling facilities.
+ * Central dispatcher for HTTP request handlers/controllers, e.g. for web UI controllers
+ * or HTTP-based remote service exporters. Dispatches to registered handlers for processing
+ * a web request, providing convenient mapping and exception handling facilities.
  *
- * <p>This servlet is very flexible: It can be used with just about any workflow, with the installation of the
- * appropriate adapter classes. It offers the following functionality that distinguishes it from other request-driven
- * web MVC frameworks:
+ * <p>This servlet is very flexible: It can be used with just about any workflow, with the
+ * installation of the appropriate adapter classes. It offers the following functionality
+ * that distinguishes it from other request-driven web MVC frameworks:
  *
- * <ul> <li>It is based around a JavaBeans configuration mechanism.
+ * <ul>
+ * <li>It is based around a JavaBeans configuration mechanism.
  *
- * <li>It can use any {@link HandlerMapping} implementation - pre-built or provided as part of an application - to
- * control the routing of requests to handler objects. Default is {@link org.springframework.web.servlet.handler.BeanNameUrlHandlerMapping}
- * and {@link org.springframework.web.servlet.mvc.annotation.DefaultAnnotationHandlerMapping}. HandlerMapping objects
- * can be defined as beans in the servlet's application context, implementing the HandlerMapping interface, overriding
- * the default HandlerMapping if present. HandlerMappings can be given any bean name (they are tested by type).
+ * <li>It can use any {@link HandlerMapping} implementation - pre-built or provided as part
+ * of an application - to control the routing of requests to handler objects. Default is
+ * {@link org.springframework.web.servlet.handler.BeanNameUrlHandlerMapping} and
+ * {@link org.springframework.web.servlet.mvc.annotation.DefaultAnnotationHandlerMapping}.
+ * HandlerMapping objects can be defined as beans in the servlet's application context,
+ * implementing the HandlerMapping interface, overriding the default HandlerMapping if
+ * present. HandlerMappings can be given any bean name (they are tested by type).
  *
- * <li>It can use any {@link HandlerAdapter}; this allows for using any handler interface. Default adapters are {@link
- * org.springframework.web.servlet.mvc.HttpRequestHandlerAdapter}, {@link org.springframework.web.servlet.mvc.SimpleControllerHandlerAdapter},
- * for Spring's {@link org.springframework.web.HttpRequestHandler} and {@link org.springframework.web.servlet.mvc.Controller}
- * interfaces, respectively. A default {@link org.springframework.web.servlet.mvc.annotation.AnnotationMethodHandlerAdapter}
- * will be registered as well. HandlerAdapter objects can be added as beans in the application context, overriding the
- * default HandlerAdapters. Like HandlerMappings, HandlerAdapters can be given any bean name (they are tested by type).
+ * <li>It can use any {@link HandlerAdapter}; this allows for using any handler interface.
+ * Default adapters are {@link org.springframework.web.servlet.mvc.HttpRequestHandlerAdapter},
+ * {@link org.springframework.web.servlet.mvc.SimpleControllerHandlerAdapter}, for Spring's
+ * {@link org.springframework.web.HttpRequestHandler} and
+ * {@link org.springframework.web.servlet.mvc.Controller} interfaces, respectively. A default
+ * {@link org.springframework.web.servlet.mvc.annotation.AnnotationMethodHandlerAdapter}
+ * will be registered as well. HandlerAdapter objects can be added as beans in the
+ * application context, overriding the default HandlerAdapters. Like HandlerMappings,
+ * HandlerAdapters can be given any bean name (they are tested by type).
  *
- * <li>The dispatcher's exception resolution strategy can be specified via a {@link HandlerExceptionResolver}, for
- * example mapping certain exceptions to error pages. Default are
+ * <li>The dispatcher's exception resolution strategy can be specified via a
+ * {@link HandlerExceptionResolver}, for example mapping certain exceptions to error pages.
+ * Default are
  * {@link org.springframework.web.servlet.mvc.annotation.AnnotationMethodHandlerExceptionResolver},
  * {@link org.springframework.web.servlet.mvc.annotation.ResponseStatusExceptionResolver}, and
- * {@link org.springframework.web.servlet.mvc.support.DefaultHandlerExceptionResolver}. These HandlerExceptionResolvers can be overridden
- * through the application context. HandlerExceptionResolver can be given any bean name (they are tested by type).
+ * {@link org.springframework.web.servlet.mvc.support.DefaultHandlerExceptionResolver}.
+ * These HandlerExceptionResolvers can be overridden through the application context.
+ * HandlerExceptionResolver can be given any bean name (they are tested by type).
  *
- * <li>Its view resolution strategy can be specified via a {@link ViewResolver} implementation, resolving symbolic view
- * names into View objects. Default is {@link org.springframework.web.servlet.view.InternalResourceViewResolver}.
- * ViewResolver objects can be added as beans in the application context, overriding the default ViewResolver.
- * ViewResolvers can be given any bean name (they are tested by type).
+ * <li>Its view resolution strategy can be specified via a {@link ViewResolver}
+ * implementation, resolving symbolic view names into View objects. Default is
+ * {@link org.springframework.web.servlet.view.InternalResourceViewResolver}.
+ * ViewResolver objects can be added as beans in the application context, overriding the
+ * default ViewResolver. ViewResolvers can be given any bean name (they are tested by type).
  *
- * <li>If a {@link View} or view name is not supplied by the user, then the configured {@link
- * RequestToViewNameTranslator} will translate the current request into a view name. The corresponding bean name is
- * "viewNameTranslator"; the default is {@link org.springframework.web.servlet.view.DefaultRequestToViewNameTranslator}.
+ * <li>If a {@link View} or view name is not supplied by the user, then the configured
+ * {@link RequestToViewNameTranslator} will translate the current request into a view name.
+ * The corresponding bean name is "viewNameTranslator"; the default is
+ * {@link org.springframework.web.servlet.view.DefaultRequestToViewNameTranslator}.
  *
- * <li>The dispatcher's strategy for resolving multipart requests is determined by a {@link
- * org.springframework.web.multipart.MultipartResolver} implementation. Implementations for Jakarta Commons FileUpload
- * and Jason Hunter's COS are included; the typical choise is {@link org.springframework.web.multipart.commons.CommonsMultipartResolver}.
+ * <li>The dispatcher's strategy for resolving multipart requests is determined by a
+ * {@link org.springframework.web.multipart.MultipartResolver} implementation.
+ * Implementations for Apache Commons FileUpload and Servlet 3 are included; the typical
+ * choice is {@link org.springframework.web.multipart.commons.CommonsMultipartResolver}.
  * The MultipartResolver bean name is "multipartResolver"; default is none.
  *
- * <li>Its locale resolution strategy is determined by a {@link LocaleResolver}. Out-of-the-box implementations work via
- * HTTP accept header, cookie, or session. The LocaleResolver bean name is "localeResolver"; default is {@link
- * org.springframework.web.servlet.i18n.AcceptHeaderLocaleResolver}.
+ * <li>Its locale resolution strategy is determined by a {@link LocaleResolver}.
+ * Out-of-the-box implementations work via HTTP accept header, cookie, or session.
+ * The LocaleResolver bean name is "localeResolver"; default is
+ * {@link org.springframework.web.servlet.i18n.AcceptHeaderLocaleResolver}.
  *
- * <li>Its theme resolution strategy is determined by a {@link ThemeResolver}. Implementations for a fixed theme and for
- * cookie and session storage are included. The ThemeResolver bean name is "themeResolver"; default is {@link
- * org.springframework.web.servlet.theme.FixedThemeResolver}. </ul>
+ * <li>Its theme resolution strategy is determined by a {@link ThemeResolver}.
+ * Implementations for a fixed theme and for cookie and session storage are included.
+ * The ThemeResolver bean name is "themeResolver"; default is
+ * {@link org.springframework.web.servlet.theme.FixedThemeResolver}.
+ * </ul>
  *
- * <p><b>NOTE: The {@code @RequestMapping} annotation will only be processed if a corresponding
- * {@code HandlerMapping} (for type level annotations) and/or {@code HandlerAdapter} (for method level
- * annotations) is present in the dispatcher.</b> This is the case by default. However, if you are defining custom
- * {@code HandlerMappings} or {@code HandlerAdapters}, then you need to make sure that a corresponding custom
- * {@code DefaultAnnotationHandlerMapping} and/or {@code AnnotationMethodHandlerAdapter} is defined as well -
- * provided that you intend to use {@code @RequestMapping}.
+ * <p><b>NOTE: The {@code @RequestMapping} annotation will only be processed if a
+ * corresponding {@code HandlerMapping} (for type-level annotations) and/or
+ * {@code HandlerAdapter} (for method-level annotations) is present in the dispatcher.</b>
+ * This is the case by default. However, if you are defining custom {@code HandlerMappings}
+ * or {@code HandlerAdapters}, then you need to make sure that a corresponding custom
+ * {@code DefaultAnnotationHandlerMapping} and/or {@code AnnotationMethodHandlerAdapter}
+ * is defined as well - provided that you intend to use {@code @RequestMapping}.
  *
- * <p><b>A web application can define any number of DispatcherServlets.</b> Each servlet will operate in its own
- * namespace, loading its own application context with mappings, handlers, etc. Only the root application context as
- * loaded by {@link org.springframework.web.context.ContextLoaderListener}, if any, will be shared.
+ * <p><b>A web application can define any number of DispatcherServlets.</b>
+ * Each servlet will operate in its own namespace, loading its own application context
+ * with mappings, handlers, etc. Only the root application context as loaded by
+ * {@link org.springframework.web.context.ContextLoaderListener}, if any, will be shared.
  *
  * <p>As of Spring 3.1, {@code DispatcherServlet} may now be injected with a web
  * application context, rather than creating its own internally. This is useful in Servlet
- * 3.0+ environments, which support programmatic registration of servlet instances. See
- * {@link #DispatcherServlet(WebApplicationContext)} Javadoc for details.
+ * 3.0+ environments, which support programmatic registration of servlet instances.
+ * See the {@link #DispatcherServlet(WebApplicationContext)} javadoc for details.
  *
  * @author Rod Johnson
  * @author Juergen Hoeller
@@ -189,7 +206,7 @@ public class DispatcherServlet extends FrameworkServlet {
 	/**
 	 * Request attribute to hold the current web application context.
 	 * Otherwise only the global web app context is obtainable by tags etc.
-	 * @see org.springframework.web.servlet.support.RequestContextUtils#getWebApplicationContext
+	 * @see org.springframework.web.servlet.support.RequestContextUtils#findWebApplicationContext
 	 */
 	public static final String WEB_APPLICATION_CONTEXT_ATTRIBUTE = DispatcherServlet.class.getName() + ".CONTEXT";
 
@@ -231,6 +248,13 @@ public class DispatcherServlet extends FrameworkServlet {
 	 */
 	public static final String FLASH_MAP_MANAGER_ATTRIBUTE = DispatcherServlet.class.getName() + ".FLASH_MAP_MANAGER";
 
+	/**
+	 * Name of request attribute that exposes an Exception resolved with an
+	 * {@link HandlerExceptionResolver} but where no view was rendered
+	 * (e.g. setting the status code).
+	 */
+	public static final String EXCEPTION_ATTRIBUTE = DispatcherServlet.class.getName() + ".EXCEPTION";
+
 	/** Log category to use when no mapped handler is found for a request. */
 	public static final String PAGE_NOT_FOUND_LOG_CATEGORY = "org.springframework.web.servlet.PageNotFound";
 
@@ -271,6 +295,9 @@ public class DispatcherServlet extends FrameworkServlet {
 	/** Detect all ViewResolvers or just expect "viewResolver" bean? */
 	private boolean detectAllViewResolvers = true;
 
+	/** Throw a NoHandlerFoundException if no Handler was found to process this request? **/
+	private boolean throwExceptionIfNoHandlerFound = false;
+
 	/** Perform cleanup of request attributes after include request? */
 	private boolean cleanupAfterInclude = true;
 
@@ -301,6 +328,7 @@ public class DispatcherServlet extends FrameworkServlet {
 	/** List of ViewResolvers used by this servlet */
 	private List<ViewResolver> viewResolvers;
 
+
 	/**
 	 * Create a new {@code DispatcherServlet} that will create its own internal web
 	 * application context based on defaults and values provided through servlet
@@ -320,6 +348,7 @@ public class DispatcherServlet extends FrameworkServlet {
 	 */
 	public DispatcherServlet() {
 		super();
+		setDispatchOptionsRequest(true);
 	}
 
 	/**
@@ -363,8 +392,10 @@ public class DispatcherServlet extends FrameworkServlet {
 	 */
 	public DispatcherServlet(WebApplicationContext webApplicationContext) {
 		super(webApplicationContext);
+		setDispatchOptionsRequest(true);
 	}
 
+
 	/**
 	 * Set whether to detect all HandlerMapping beans in this servlet's context. Otherwise,
 	 * just a single bean with name "handlerMapping" will be expected.
@@ -405,6 +436,21 @@ public class DispatcherServlet extends FrameworkServlet {
 		this.detectAllViewResolvers = detectAllViewResolvers;
 	}
 
+	/**
+	 * Set whether to throw a NoHandlerFoundException when no Handler was found for this request.
+	 * This exception can then be caught with a HandlerExceptionResolver or an
+	 * {@code @ExceptionHandler} controller method.
+	 * <p>Note that if {@link org.springframework.web.servlet.resource.DefaultServletHttpRequestHandler}
+	 * is used, then requests will always be forwarded to the default servlet and a
+	 * NoHandlerFoundException would never be thrown in that case.
+	 * <p>Default is "false", meaning the DispatcherServlet sends a NOT_FOUND error through the
+	 * Servlet response.
+	 * @since 4.0
+	 */
+	public void setThrowExceptionIfNoHandlerFound(boolean throwExceptionIfNoHandlerFound) {
+		this.throwExceptionIfNoHandlerFound = throwExceptionIfNoHandlerFound;
+	}
+
 	/**
 	 * Set whether to perform cleanup of request attributes after an include request, that is,
 	 * whether to reset the original state of all request attributes after the DispatcherServlet
@@ -421,6 +467,7 @@ public class DispatcherServlet extends FrameworkServlet {
 		this.cleanupAfterInclude = cleanupAfterInclude;
 	}
 
+
 	/**
 	 * This implementation calls {@link #initStrategies}.
 	 */
@@ -505,9 +552,8 @@ public class DispatcherServlet extends FrameworkServlet {
 			// We need to use the default.
 			this.themeResolver = getDefaultStrategy(context, ThemeResolver.class);
 			if (logger.isDebugEnabled()) {
-				logger.debug(
-						"Unable to locate ThemeResolver with name '" + THEME_RESOLVER_BEAN_NAME + "': using default [" +
-								this.themeResolver + "]");
+				logger.debug("Unable to locate ThemeResolver with name '" + THEME_RESOLVER_BEAN_NAME +
+						"': using default [" + this.themeResolver + "]");
 			}
 		}
 	}
@@ -527,7 +573,7 @@ public class DispatcherServlet extends FrameworkServlet {
 			if (!matchingBeans.isEmpty()) {
 				this.handlerMappings = new ArrayList<HandlerMapping>(matchingBeans.values());
 				// We keep HandlerMappings in sorted order.
-				OrderComparator.sort(this.handlerMappings);
+				AnnotationAwareOrderComparator.sort(this.handlerMappings);
 			}
 		}
 		else {
@@ -565,7 +611,7 @@ public class DispatcherServlet extends FrameworkServlet {
 			if (!matchingBeans.isEmpty()) {
 				this.handlerAdapters = new ArrayList<HandlerAdapter>(matchingBeans.values());
 				// We keep HandlerAdapters in sorted order.
-				OrderComparator.sort(this.handlerAdapters);
+				AnnotationAwareOrderComparator.sort(this.handlerAdapters);
 			}
 		}
 		else {
@@ -603,7 +649,7 @@ public class DispatcherServlet extends FrameworkServlet {
 			if (!matchingBeans.isEmpty()) {
 				this.handlerExceptionResolvers = new ArrayList<HandlerExceptionResolver>(matchingBeans.values());
 				// We keep HandlerExceptionResolvers in sorted order.
-				OrderComparator.sort(this.handlerExceptionResolvers);
+				AnnotationAwareOrderComparator.sort(this.handlerExceptionResolvers);
 			}
 		}
 		else {
@@ -665,7 +711,7 @@ public class DispatcherServlet extends FrameworkServlet {
 			if (!matchingBeans.isEmpty()) {
 				this.viewResolvers = new ArrayList<ViewResolver>(matchingBeans.values());
 				// We keep ViewResolvers in sorted order.
-				OrderComparator.sort(this.viewResolvers);
+				AnnotationAwareOrderComparator.sort(this.viewResolvers);
 			}
 		}
 		else {
@@ -695,8 +741,7 @@ public class DispatcherServlet extends FrameworkServlet {
 	 */
 	private void initFlashMapManager(ApplicationContext context) {
 		try {
-			this.flashMapManager =
-					context.getBean(FLASH_MAP_MANAGER_BEAN_NAME, FlashMapManager.class);
+			this.flashMapManager = context.getBean(FLASH_MAP_MANAGER_BEAN_NAME, FlashMapManager.class);
 			if (logger.isDebugEnabled()) {
 				logger.debug("Using FlashMapManager [" + this.flashMapManager + "]");
 			}
@@ -796,7 +841,8 @@ public class DispatcherServlet extends FrameworkServlet {
 
 	/**
 	 * Create a default strategy.
-	 * <p>The default implementation uses {@link org.springframework.beans.factory.config.AutowireCapableBeanFactory#createBean}.
+	 * <p>The default implementation uses
+	 * {@link org.springframework.beans.factory.config.AutowireCapableBeanFactory#createBean}.
 	 * @param context the current WebApplicationContext
 	 * @param clazz the strategy implementation class to instantiate
 	 * @return the fully configured strategy instance
@@ -851,12 +897,11 @@ public class DispatcherServlet extends FrameworkServlet {
 			doDispatch(request, response);
 		}
 		finally {
-			if (WebAsyncUtils.getAsyncManager(request).isConcurrentHandlingStarted()) {
-				return;
-			}
-			// Restore the original attribute snapshot, in case of an include.
-			if (attributesSnapshot != null) {
-				restoreAttributesAfterInclude(request, attributesSnapshot);
+			if (!WebAsyncUtils.getAsyncManager(request).isConcurrentHandlingStarted()) {
+				// Restore the original attribute snapshot, in case of an include.
+				if (attributesSnapshot != null) {
+					restoreAttributesAfterInclude(request, attributesSnapshot);
+				}
 			}
 		}
 	}
@@ -885,10 +930,10 @@ public class DispatcherServlet extends FrameworkServlet {
 
 			try {
 				processedRequest = checkMultipart(request);
-				multipartRequestParsed = processedRequest != request;
+				multipartRequestParsed = (processedRequest != request);
 
 				// Determine handler for the current request.
-				mappedHandler = getHandler(processedRequest, false);
+				mappedHandler = getHandler(processedRequest);
 				if (mappedHandler == null || mappedHandler.getHandler() == null) {
 					noHandlerFound(processedRequest, response);
 					return;
@@ -914,39 +959,45 @@ public class DispatcherServlet extends FrameworkServlet {
 					return;
 				}
 
-				try {
-					// Actually invoke the handler.
-					mv = ha.handle(processedRequest, response, mappedHandler.getHandler());
-				}
-				finally {
-					if (asyncManager.isConcurrentHandlingStarted()) {
-						return;
-					}
+				// Actually invoke the handler.
+				mv = ha.handle(processedRequest, response, mappedHandler.getHandler());
+
+				if (asyncManager.isConcurrentHandlingStarted()) {
+					return;
 				}
 
-				applyDefaultViewName(request, mv);
+				applyDefaultViewName(processedRequest, mv);
 				mappedHandler.applyPostHandle(processedRequest, response, mv);
 			}
 			catch (Exception ex) {
 				dispatchException = ex;
 			}
+			catch (Throwable err) {
+				// As of 4.3, we're processing Errors thrown from handler methods as well,
+				// making them available for @ExceptionHandler methods and other scenarios.
+				dispatchException = new NestedServletException("Handler dispatch failed", err);
+			}
 			processDispatchResult(processedRequest, response, mappedHandler, mv, dispatchException);
 		}
 		catch (Exception ex) {
 			triggerAfterCompletion(processedRequest, response, mappedHandler, ex);
 		}
-		catch (Error err) {
-			triggerAfterCompletionWithError(processedRequest, response, mappedHandler, err);
+		catch (Throwable err) {
+			triggerAfterCompletion(processedRequest, response, mappedHandler,
+					new NestedServletException("Handler processing failed", err));
 		}
 		finally {
 			if (asyncManager.isConcurrentHandlingStarted()) {
 				// Instead of postHandle and afterCompletion
-				mappedHandler.applyAfterConcurrentHandlingStarted(processedRequest, response);
-				return;
+				if (mappedHandler != null) {
+					mappedHandler.applyAfterConcurrentHandlingStarted(processedRequest, response);
+				}
 			}
-			// Clean up any resources used by a multipart request.
-			if (multipartRequestParsed) {
-				cleanupMultipart(processedRequest);
+			else {
+				// Clean up any resources used by a multipart request.
+				if (multipartRequestParsed) {
+					cleanupMultipart(processedRequest);
+				}
 			}
 		}
 	}
@@ -1014,14 +1065,17 @@ public class DispatcherServlet extends FrameworkServlet {
 	 */
 	@Override
 	protected LocaleContext buildLocaleContext(final HttpServletRequest request) {
-		return new LocaleContext() {
-			public Locale getLocale() {
-				return localeResolver.resolveLocale(request);
-			}
-			public String toString() {
-				return getLocale().toString();
-			}
-		};
+		if (this.localeResolver instanceof LocaleContextResolver) {
+			return ((LocaleContextResolver) this.localeResolver).resolveLocaleContext(request);
+		}
+		else {
+			return new LocaleContext() {
+				@Override
+				public Locale getLocale() {
+					return localeResolver.resolveLocale(request);
+				}
+			};
+		}
 	}
 
 	/**
@@ -1037,6 +1091,10 @@ public class DispatcherServlet extends FrameworkServlet {
 				logger.debug("Request is already a MultipartHttpServletRequest - if not in a forward, " +
 						"this typically results from an additional MultipartFilter in web.xml");
 			}
+			else if (request.getAttribute(WebUtils.ERROR_EXCEPTION_ATTRIBUTE) instanceof MultipartException) {
+				logger.debug("Multipart resolution failed for current request before - " +
+						"skipping re-resolution for undisturbed error rendering");
+			}
 			else {
 				return this.multipartResolver.resolveMultipart(request);
 			}
@@ -1051,25 +1109,13 @@ public class DispatcherServlet extends FrameworkServlet {
 	 * @see MultipartResolver#cleanupMultipart
 	 */
 	protected void cleanupMultipart(HttpServletRequest request) {
-		MultipartHttpServletRequest multipartRequest = WebUtils.getNativeRequest(request, MultipartHttpServletRequest.class);
+		MultipartHttpServletRequest multipartRequest =
+				WebUtils.getNativeRequest(request, MultipartHttpServletRequest.class);
 		if (multipartRequest != null) {
 			this.multipartResolver.cleanupMultipart(multipartRequest);
 		}
 	}
 
-	/**
-	 * Return the HandlerExecutionChain for this request. Try all handler mappings in order.
-	 * @param request current HTTP request
-	 * @param cache whether to cache the HandlerExecutionChain in a request attribute
-	 * @return the HandlerExecutionChain, or {@code null} if no handler could be found
-	 * @deprecated as of Spring 3.0.4, in favor of {@link #getHandler(javax.servlet.http.HttpServletRequest)},
-	 * with this method's cache attribute now effectively getting ignored
-	 */
-	@Deprecated
-	protected HandlerExecutionChain getHandler(HttpServletRequest request, boolean cache) throws Exception {
-		return getHandler(request);
-	}
-
 	/**
 	 * Return the HandlerExecutionChain for this request.
 	 * <p>Tries all handler mappings in order.
@@ -1101,7 +1147,13 @@ public class DispatcherServlet extends FrameworkServlet {
 			pageNotFoundLogger.warn("No mapping found for HTTP request with URI [" + getRequestUri(request) +
 					"] in DispatcherServlet with name '" + getServletName() + "'");
 		}
-		response.sendError(HttpServletResponse.SC_NOT_FOUND);
+		if (this.throwExceptionIfNoHandlerFound) {
+			throw new NoHandlerFoundException(request.getMethod(), getRequestUri(request),
+					new ServletServerHttpRequest(request).getHeaders());
+		}
+		else {
+			response.sendError(HttpServletResponse.SC_NOT_FOUND);
+		}
 	}
 
 	/**
@@ -1145,6 +1197,7 @@ public class DispatcherServlet extends FrameworkServlet {
 		}
 		if (exMv != null) {
 			if (exMv.isEmpty()) {
+				request.setAttribute(EXCEPTION_ATTRIBUTE, ex);
 				return null;
 			}
 			// We might still need view name translation for a plain error model...
@@ -1198,6 +1251,9 @@ public class DispatcherServlet extends FrameworkServlet {
 			logger.debug("Rendering view [" + view + "] in DispatcherServlet with name '" + getServletName() + "'");
 		}
 		try {
+			if (mv.getStatus() != null) {
+				response.setStatus(mv.getStatus().value());
+			}
 			view.render(mv.getModelInternal(), request, response);
 		}
 		catch (Exception ex) {
@@ -1254,16 +1310,6 @@ public class DispatcherServlet extends FrameworkServlet {
 		throw ex;
 	}
 
-	private void triggerAfterCompletionWithError(HttpServletRequest request, HttpServletResponse response,
-			HandlerExecutionChain mappedHandler, Error error) throws Exception, ServletException {
-
-		ServletException ex = new NestedServletException("Handler processing failed", error);
-		if (mappedHandler != null) {
-			mappedHandler.triggerAfterCompletion(request, response, ex);
-		}
-		throw ex;
-	}
-
 	/**
 	 * Restore the request attributes after an include.
 	 * @param request current HTTP request
