@@ -1,5 +1,5 @@
 /*
- * Copyright 2002-2013 the original author or authors.
+ * Copyright 2002-2015 the original author or authors.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -31,6 +31,8 @@ import org.dom4j.Document;
 import org.dom4j.Element;
 import org.dom4j.io.SAXReader;
 
+import org.junit.Test;
+
 import org.springframework.beans.propertyeditors.StringTrimmerEditor;
 import org.springframework.tests.sample.beans.Colour;
 import org.springframework.tests.sample.beans.Pet;
@@ -38,11 +40,14 @@ import org.springframework.tests.sample.beans.TestBean;
 import org.springframework.validation.BeanPropertyBindingResult;
 import org.springframework.validation.BindingResult;
 
+import static org.junit.Assert.*;
+
 /**
  * @author Rob Harrop
  * @author Juergen Hoeller
  * @author Jeremy Grelle
  */
+@SuppressWarnings({ "rawtypes", "unchecked" })
 public class CheckboxTagTests extends AbstractFormTagTests {
 
 	private CheckboxTag tag;
@@ -61,7 +66,8 @@ public class CheckboxTagTests extends AbstractFormTagTests {
 		this.tag.setPageContext(getPageContext());
 	}
 
-	public void testWithSingleValueBooleanObjectChecked() throws Exception {
+	@Test
+	public void withSingleValueBooleanObjectChecked() throws Exception {
 		this.tag.setPath("someBoolean");
 		int result = this.tag.doStartTag();
 		assertEquals(Tag.SKIP_BODY, result);
@@ -83,7 +89,8 @@ public class CheckboxTagTests extends AbstractFormTagTests {
 		assertEquals("true", checkboxElement.attribute("value").getValue());
 	}
 
-	public void testWithIndexedBooleanObjectNotChecked() throws Exception {
+	@Test
+	public void withIndexedBooleanObjectNotChecked() throws Exception {
 		this.tag.setPath("someMap[key]");
 		int result = this.tag.doStartTag();
 		assertEquals(Tag.SKIP_BODY, result);
@@ -105,7 +112,8 @@ public class CheckboxTagTests extends AbstractFormTagTests {
 		assertEquals("true", checkboxElement.attribute("value").getValue());
 	}
 
-	public void testWithSingleValueBooleanObjectCheckedAndDynamicAttributes() throws Exception {
+	@Test
+	public void withSingleValueBooleanObjectCheckedAndDynamicAttributes() throws Exception {
 		String dynamicAttribute1 = "attr1";
 		String dynamicAttribute2 = "attr2";
 
@@ -134,7 +142,8 @@ public class CheckboxTagTests extends AbstractFormTagTests {
 		assertEquals(dynamicAttribute2, checkboxElement.attribute(dynamicAttribute2).getValue());
 	}
 
-	public void testWithSingleValueBooleanChecked() throws Exception {
+	@Test
+	public void withSingleValueBooleanChecked() throws Exception {
 		this.tag.setPath("jedi");
 		int result = this.tag.doStartTag();
 		assertEquals(Tag.SKIP_BODY, result);
@@ -153,7 +162,8 @@ public class CheckboxTagTests extends AbstractFormTagTests {
 		assertEquals("true", checkboxElement.attribute("value").getValue());
 	}
 
-	public void testWithSingleValueBooleanObjectUnchecked() throws Exception {
+	@Test
+	public void withSingleValueBooleanObjectUnchecked() throws Exception {
 		this.bean.setSomeBoolean(new Boolean(false));
 		this.tag.setPath("someBoolean");
 		int result = this.tag.doStartTag();
@@ -174,7 +184,8 @@ public class CheckboxTagTests extends AbstractFormTagTests {
 		assertEquals("true", checkboxElement.attribute("value").getValue());
 	}
 
-	public void testWithSingleValueBooleanUnchecked() throws Exception {
+	@Test
+	public void withSingleValueBooleanUnchecked() throws Exception {
 		this.bean.setJedi(false);
 		this.tag.setPath("jedi");
 		int result = this.tag.doStartTag();
@@ -195,7 +206,8 @@ public class CheckboxTagTests extends AbstractFormTagTests {
 		assertEquals("true", checkboxElement.attribute("value").getValue());
 	}
 
-	public void testWithSingleValueNull() throws Exception {
+	@Test
+	public void withSingleValueNull() throws Exception {
 		this.bean.setName(null);
 		this.tag.setPath("name");
 		this.tag.setValue("Rob Harrop");
@@ -217,7 +229,8 @@ public class CheckboxTagTests extends AbstractFormTagTests {
 		assertEquals("Rob Harrop", checkboxElement.attribute("value").getValue());
 	}
 
-	public void testWithSingleValueNotNull() throws Exception {
+	@Test
+	public void withSingleValueNotNull() throws Exception {
 		this.bean.setName("Rob Harrop");
 		this.tag.setPath("name");
 		this.tag.setValue("Rob Harrop");
@@ -239,7 +252,8 @@ public class CheckboxTagTests extends AbstractFormTagTests {
 		assertEquals("Rob Harrop", checkboxElement.attribute("value").getValue());
 	}
 
-	public void testWithSingleValueAndEditor() throws Exception {
+	@Test
+	public void withSingleValueAndEditor() throws Exception {
 		this.bean.setName("Rob Harrop");
 		this.tag.setPath("name");
 		this.tag.setValue("   Rob Harrop");
@@ -265,7 +279,8 @@ public class CheckboxTagTests extends AbstractFormTagTests {
 		assertEquals("   Rob Harrop", checkboxElement.attribute("value").getValue());
 	}
 
-	public void testWithMultiValueChecked() throws Exception {
+	@Test
+	public void withMultiValueChecked() throws Exception {
 		this.tag.setPath("stringArray");
 		this.tag.setValue("foo");
 		int result = this.tag.doStartTag();
@@ -286,7 +301,8 @@ public class CheckboxTagTests extends AbstractFormTagTests {
 		assertEquals("foo", checkboxElement.attribute("value").getValue());
 	}
 
-	public void testWithMultiValueUnchecked() throws Exception {
+	@Test
+	public void withMultiValueUnchecked() throws Exception {
 		this.tag.setPath("stringArray");
 		this.tag.setValue("abc");
 		int result = this.tag.doStartTag();
@@ -307,7 +323,8 @@ public class CheckboxTagTests extends AbstractFormTagTests {
 		assertEquals("abc", checkboxElement.attribute("value").getValue());
 	}
 
-	public void testWithMultiValueWithEditor() throws Exception {
+	@Test
+	public void withMultiValueWithEditor() throws Exception {
 		this.tag.setPath("stringArray");
 		this.tag.setValue("   foo");
 		BeanPropertyBindingResult bindingResult = new BeanPropertyBindingResult(this.bean, COMMAND_NAME);
@@ -334,7 +351,8 @@ public class CheckboxTagTests extends AbstractFormTagTests {
 		assertEquals("   foo", checkboxElement.attribute("value").getValue());
 	}
 
-	public void testWithMultiValueIntegerWithEditor() throws Exception {
+	@Test
+	public void withMultiValueIntegerWithEditor() throws Exception {
 		this.tag.setPath("someIntegerArray");
 		this.tag.setValue("   1");
 		BeanPropertyBindingResult bindingResult = new BeanPropertyBindingResult(this.bean, COMMAND_NAME);
@@ -361,7 +379,8 @@ public class CheckboxTagTests extends AbstractFormTagTests {
 		assertEquals("   1", checkboxElement.attribute("value").getValue());
 	}
 
-	public void testWithCollection() throws Exception {
+	@Test
+	public void withCollection() throws Exception {
 		this.tag.setPath("someList");
 		this.tag.setValue("foo");
 		int result = this.tag.doStartTag();
@@ -382,7 +401,8 @@ public class CheckboxTagTests extends AbstractFormTagTests {
 		assertEquals("foo", checkboxElement.attribute("value").getValue());
 	}
 
-	public void testWithObjectChecked() throws Exception {
+	@Test
+	public void withObjectChecked() throws Exception {
 		this.tag.setPath("date");
 		this.tag.setValue(getDate());
 
@@ -404,7 +424,8 @@ public class CheckboxTagTests extends AbstractFormTagTests {
 		assertEquals(getDate().toString(), checkboxElement.attribute("value").getValue());
 	}
 
-	public void testWithObjectUnchecked() throws Exception {
+	@Test
+	public void withObjectUnchecked() throws Exception {
 		this.tag.setPath("date");
 		Date date = new Date();
 		this.tag.setValue(date);
@@ -427,7 +448,8 @@ public class CheckboxTagTests extends AbstractFormTagTests {
 		assertEquals(date.toString(), checkboxElement.attribute("value").getValue());
 	}
 
-	public void testCollectionOfColoursSelected() throws Exception {
+	@Test
+	public void collectionOfColoursSelected() throws Exception {
 		this.tag.setPath("otherColours");
 		this.tag.setValue("RED");
 
@@ -448,7 +470,8 @@ public class CheckboxTagTests extends AbstractFormTagTests {
 		assertEquals("checked", checkboxElement.attribute("checked").getValue());
 	}
 
-	public void testCollectionOfColoursNotSelected() throws Exception {
+	@Test
+	public void collectionOfColoursNotSelected() throws Exception {
 		this.tag.setPath("otherColours");
 		this.tag.setValue("PURPLE");
 
@@ -469,7 +492,8 @@ public class CheckboxTagTests extends AbstractFormTagTests {
 		assertNull(checkboxElement.attribute("checked"));
 	}
 
-	public void testCollectionOfPetsAsString() throws Exception {
+	@Test
+	public void collectionOfPetsAsString() throws Exception {
 		this.tag.setPath("pets");
 		this.tag.setValue("Spot");
 
@@ -490,7 +514,8 @@ public class CheckboxTagTests extends AbstractFormTagTests {
 		assertEquals("checked", checkboxElement.attribute("checked").getValue());
 	}
 
-	public void testCollectionOfPetsAsStringNotSelected() throws Exception {
+	@Test
+	public void collectionOfPetsAsStringNotSelected() throws Exception {
 		this.tag.setPath("pets");
 		this.tag.setValue("Santa's Little Helper");
 
@@ -511,7 +536,8 @@ public class CheckboxTagTests extends AbstractFormTagTests {
 		assertNull(checkboxElement.attribute("checked"));
 	}
 
-	public void testCollectionOfPets() throws Exception {
+	@Test
+	public void collectionOfPets() throws Exception {
 		this.tag.setPath("pets");
 		this.tag.setValue(new Pet("Rudiger"));
 
@@ -533,7 +559,8 @@ public class CheckboxTagTests extends AbstractFormTagTests {
 		assertEquals("checked", checkboxElement.attribute("checked").getValue());
 	}
 
-	public void testCollectionOfPetsNotSelected() throws Exception {
+	@Test
+	public void collectionOfPetsNotSelected() throws Exception {
 		this.tag.setPath("pets");
 		this.tag.setValue(new Pet("Santa's Little Helper"));
 
@@ -555,7 +582,8 @@ public class CheckboxTagTests extends AbstractFormTagTests {
 		assertNull(checkboxElement.attribute("checked"));
 	}
 
-	public void testCollectionOfPetsWithEditor() throws Exception {
+	@Test
+	public void collectionOfPetsWithEditor() throws Exception {
 		this.tag.setPath("pets");
 		this.tag.setValue(new ItemPet("Rudiger"));
 
@@ -582,7 +610,8 @@ public class CheckboxTagTests extends AbstractFormTagTests {
 		assertEquals("checked", checkboxElement.attribute("checked").getValue());
 	}
 
-	public void testWithNullValue() throws Exception {
+	@Test
+	public void withNullValue() throws Exception {
 		try {
 			this.tag.setPath("name");
 			this.tag.doStartTag();
@@ -593,7 +622,8 @@ public class CheckboxTagTests extends AbstractFormTagTests {
 		}
 	}
 
-	public void testHiddenElementOmittedOnDisabled() throws Exception {
+	@Test
+	public void hiddenElementOmittedOnDisabled() throws Exception {
 		this.tag.setPath("someBoolean");
 		this.tag.setDisabled(true);
 		int result = this.tag.doStartTag();
@@ -615,7 +645,8 @@ public class CheckboxTagTests extends AbstractFormTagTests {
 		assertEquals("true", checkboxElement.attribute("value").getValue());
 	}
 
-	public void testDynamicTypeAttribute() throws JspException {
+	@Test
+	public void dynamicTypeAttribute() throws JspException {
 		try {
 			this.tag.setDynamicAttribute(null, "type", "email");
 			fail("Expected exception");
