@@ -1,5 +1,5 @@
 /*
- * Copyright 2002-2013 the original author or authors.
+ * Copyright 2002-2015 the original author or authors.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -35,6 +35,8 @@ import org.dom4j.Document;
 import org.dom4j.Element;
 import org.dom4j.io.SAXReader;
 
+import org.junit.Test;
+
 import org.springframework.beans.propertyeditors.StringTrimmerEditor;
 import org.springframework.format.Formatter;
 import org.springframework.format.support.FormattingConversionService;
@@ -45,6 +47,8 @@ import org.springframework.util.ObjectUtils;
 import org.springframework.validation.BeanPropertyBindingResult;
 import org.springframework.validation.BindingResult;
 
+import static org.junit.Assert.*;
+
 /**
  * @author Thomas Risberg
  * @author Mark Fisher
@@ -52,6 +56,7 @@ import org.springframework.validation.BindingResult;
  * @author Benjamin Hoffmann
  * @author Jeremy Grelle
  */
+@SuppressWarnings({ "rawtypes", "unchecked" })
 public class CheckboxesTagTests extends AbstractFormTagTests {
 
 	private CheckboxesTag tag;
@@ -70,7 +75,8 @@ public class CheckboxesTagTests extends AbstractFormTagTests {
 		this.tag.setPageContext(getPageContext());
 	}
 
-	public void testWithMultiValueArray() throws Exception {
+	@Test
+	public void withMultiValueArray() throws Exception {
 		this.tag.setPath("stringArray");
 		this.tag.setItems(new Object[] {"foo", "bar", "baz"});
 		int result = this.tag.doStartTag();
@@ -108,7 +114,8 @@ public class CheckboxesTagTests extends AbstractFormTagTests {
 		assertEquals("baz", spanElement3.getStringValue());
 	}
 
-	public void testWithMultiValueArrayAndDynamicAttributes() throws Exception {
+	@Test
+	public void withMultiValueArrayAndDynamicAttributes() throws Exception {
 		String dynamicAttribute1 = "attr1";
 		String dynamicAttribute2 = "attr2";
 
@@ -161,7 +168,8 @@ public class CheckboxesTagTests extends AbstractFormTagTests {
 
 	}
 
-	public void testWithMultiValueArrayWithDelimiter() throws Exception {
+	@Test
+	public void withMultiValueArrayWithDelimiter() throws Exception {
 		this.tag.setDelimiter("<br/>");
 		this.tag.setPath("stringArray");
 		this.tag.setItems(new Object[] {"foo", "bar", "baz"});
@@ -206,7 +214,8 @@ public class CheckboxesTagTests extends AbstractFormTagTests {
 		assertEquals("baz", spanElement3.getStringValue());
 	}
 
-	public void testWithMultiValueMap() throws Exception {
+	@Test
+	public void withMultiValueMap() throws Exception {
 		this.tag.setPath("stringArray");
 		Map m = new LinkedHashMap();
 		m.put("foo", "FOO");
@@ -249,7 +258,8 @@ public class CheckboxesTagTests extends AbstractFormTagTests {
 		assertEquals("BAZ", spanElement3.getStringValue());
 	}
 
-	public void testWithPetItemsMap() throws Exception {
+	@Test
+	public void withPetItemsMap() throws Exception {
 		this.tag.setPath("someSet");
 		Map m = new LinkedHashMap();
 		m.put(new ItemPet("PET1"), "PET1Label");
@@ -293,7 +303,8 @@ public class CheckboxesTagTests extends AbstractFormTagTests {
 		assertEquals("PET3Label", spanElement3.getStringValue());
 	}
 
-	public void testWithMultiValueMapWithDelimiter() throws Exception {
+	@Test
+	public void withMultiValueMapWithDelimiter() throws Exception {
 		String delimiter = " | ";
 		this.tag.setDelimiter(delimiter);
 		this.tag.setPath("stringArray");
@@ -338,7 +349,8 @@ public class CheckboxesTagTests extends AbstractFormTagTests {
 		assertEquals(delimiter + "BAZ", spanElement3.getStringValue());
 	}
 
-	public void testWithMultiValueWithEditor() throws Exception {
+	@Test
+	public void withMultiValueWithEditor() throws Exception {
 		this.tag.setPath("stringArray");
 		this.tag.setItems(new Object[] {"   foo", "   bar", "   baz"});
 		BeanPropertyBindingResult bindingResult = new BeanPropertyBindingResult(this.bean, COMMAND_NAME);
@@ -380,7 +392,8 @@ public class CheckboxesTagTests extends AbstractFormTagTests {
 		assertEquals("   baz", checkboxElement3.attribute("value").getValue());
 	}
 
-	public void testWithMultiValueWithReverseEditor() throws Exception {
+	@Test
+	public void withMultiValueWithReverseEditor() throws Exception {
 		this.tag.setPath("stringArray");
 		this.tag.setItems(new Object[] {"FOO", "BAR", "BAZ"});
 		BeanPropertyBindingResult bindingResult = new BeanPropertyBindingResult(this.bean, COMMAND_NAME);
@@ -421,7 +434,8 @@ public class CheckboxesTagTests extends AbstractFormTagTests {
 		assertEquals("BAZ", checkboxElement3.attribute("value").getValue());
 	}
 
-	public void testWithMultiValueWithFormatter() throws Exception {
+	@Test
+	public void withMultiValueWithFormatter() throws Exception {
 		this.tag.setPath("stringArray");
 		this.tag.setItems(new Object[] {"   foo", "   bar", "   baz"});
 		BeanPropertyBindingResult bindingResult = new BeanPropertyBindingResult(this.bean, COMMAND_NAME);
@@ -472,7 +486,8 @@ public class CheckboxesTagTests extends AbstractFormTagTests {
 		assertEquals("   baz", checkboxElement3.attribute("value").getValue());
 	}
 
-	public void testCollectionOfPets() throws Exception {
+	@Test
+	public void collectionOfPets() throws Exception {
 		this.tag.setPath("pets");
 		List allPets = new ArrayList();
 		allPets.add(new ItemPet("Rudiger"));
@@ -539,7 +554,8 @@ public class CheckboxesTagTests extends AbstractFormTagTests {
 	/**
 	 * Test case where items toString() doesn't fit the item ID
 	 */
-	public void testCollectionOfItemPets() throws Exception {
+	@Test
+	public void collectionOfItemPets() throws Exception {
 		this.tag.setPath("someSet");
 		List allPets = new ArrayList();
 		allPets.add(new ItemPet("PET1"));
@@ -587,7 +603,8 @@ public class CheckboxesTagTests extends AbstractFormTagTests {
 		assertEquals("PET3", spanElement3.getStringValue());
 	}
 
-	public void testCollectionOfPetsWithEditor() throws Exception {
+	@Test
+	public void collectionOfPetsWithEditor() throws Exception {
 		this.tag.setPath("pets");
 		List allPets = new ArrayList();
 		allPets.add(new ItemPet("Rudiger"));
@@ -656,7 +673,8 @@ public class CheckboxesTagTests extends AbstractFormTagTests {
 		assertEquals("MUFTY", spanElement5.getStringValue());
 	}
 
-	public void testWithNullValue() throws Exception {
+	@Test
+	public void withNullValue() throws Exception {
 		try {
 			this.tag.setPath("name");
 			this.tag.doStartTag();
@@ -667,7 +685,8 @@ public class CheckboxesTagTests extends AbstractFormTagTests {
 		}
 	}
 
-	public void testHiddenElementOmittedOnDisabled() throws Exception {
+	@Test
+	public void hiddenElementOmittedOnDisabled() throws Exception {
 		this.tag.setPath("stringArray");
 		this.tag.setItems(new Object[] {"foo", "bar", "baz"});
 		this.tag.setDisabled(true);
@@ -692,7 +711,8 @@ public class CheckboxesTagTests extends AbstractFormTagTests {
 		assertEquals("foo", checkboxElement.attribute("value").getValue());
 	}
 
-	public void testSpanElementCustomizable() throws Exception {
+	@Test
+	public void spanElementCustomizable() throws Exception {
 		this.tag.setPath("stringArray");
 		this.tag.setItems(new Object[] {"foo", "bar", "baz"});
 		this.tag.setElement("element");
@@ -709,7 +729,8 @@ public class CheckboxesTagTests extends AbstractFormTagTests {
 		assertEquals("element", spanElement.getName());
 	}
 
-	public void testDynamicTypeAttribute() throws JspException {
+	@Test
+	public void dynamicTypeAttribute() throws JspException {
 		try {
 			this.tag.setDynamicAttribute(null, "type", "email");
 			fail("Expected exception");
